// AbstractItemBehavior.h: interface for the CAbstractItemBehavior class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "Item.h"
#include <fstream>
using namespace std;

class CSnakeManager;

/////////////////////////////////////////////////////////////////////////////
//Item Behaviors modify item placement, item movement, and snake properties
/////////////////////////////////////////////////////////////////////////////
class CAbstractItemBehavior  
{
public:
	CAbstractItemBehavior();
	virtual ~CAbstractItemBehavior();

    //cloning function used by CItemBehaviorFactory to
    //read snake items from the file
    virtual CAbstractItemBehavior* Clone(const char* pszIdentifier, ifstream& infile) = 0;

    ///////////////////////////////////////////////
    //Behavior Actions
    ///////////////////////////////////////////////

    //modifies item placement
    virtual void Reset(CItem& moditem, CCourseLoader* pCourse) {};

    //modifies item movement
    //ppSnakeVect is a NULL terminates array of CSnake*
    virtual void FrameMove(CItem& moditem, CCourseLoader* pCourse,
                           CItem* pItem, UINT iItemSize, 
                           CSnakeManager* pSnakeManager = NULL) {};
    
    //checks whether the snake is in the bounds of this item
    virtual BOOL CheckInBounds(CSnake* pSnake, CItem* pItem) { return FALSE; };

    //modifies the snake properties if it is in bounds
    //returns whether the item should be invalidated or not
    virtual BOOL ModifySnake(CSnake* pSnake) { return FALSE; };
};