// AbstractMenu.h: interface for the CAbstractMenu class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include <D3dx8.h>

class CAbstractMenu
{
public:
    //structure to hold menu strings with their ids
    struct MenuItem
    {
        char* m_pszMenuStr;
        int   m_iMenuID;
    };

    virtual ~CAbstractMenu() {}

    //updates the frame of the menu animation returns one of the
    //return codes below
    virtual int FrameMove() { return MENUID_NOTHING; }

    enum
    {
        //application menu ids
        MENUID_NOTHING       = 0,
        MENUID_EXITAPP       = 1,

        //start menu ids
        MENUID_START         = 2,
        MENUID_DISPLAY       = 3,

        //options menu ids
        MENUID_SNAKEDETAIL   = 4,
        MENUID_SOUND         = 5,
        MENUID_CONTROLS      = 6,
        MENUID_STARTMENU     = 7,

        //title menu ids
        MENUID_SINGLE        = 8,
        MENUID_NETWORK       = 9,
        MENUID_OPTIONS       = 10,

        //game over menu ids
        MENUID_RESTART       = 11,
        MENUID_MAINMENU      = 12,

        //level menu ids
        MENUID_PLAYLEVEL     = 13,
        MENUID_SERVER        = 14,

        //level complete menu ids
        MENUID_NEXTLEVEL     = 15,
        MENUID_RETURNTOMAIN  = 16,      

		//credit menu ids
		MENUID_CREDITS		 = 17,

        //network menu ids
        MENUID_HOST          = 18,
        MENUID_CLIENT        = 19,

        //client/server menu ids
        MENUID_CANCELNET     = 20,
        MENUID_STARTNETGAME  = 21,

        //player info menu ids
        MENUID_CONNECTSERVER = 22,
        MENUID_CONNECTCLIENT = 23
    };

    //updates the input to the menu using windows messaging
    //returns one of the previous return codes
    virtual int OnKeyDown(WPARAM wParam) { return MENUID_NOTHING; }

    //updates the input to the menu using windows messaging
    //returns one of the previous return codes
    virtual int OnChar(WPARAM wParam) { return MENUID_NOTHING; };

	// Device Initialization Functions
    //rendering the menu requires a screen size
    virtual void RenderMenu(int iWidth, int iHeight) = 0;
    virtual HRESULT InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice ) = 0;
    virtual HRESULT RestoreDeviceObjects() = 0;
    virtual HRESULT InvalidateDeviceObjects() = 0;
    virtual HRESULT DeleteDeviceObjects() = 0;

protected:
    virtual void Render() {}

	LPDIRECT3DDEVICE8 m_pd3dDevice;       // A D3DDevice used for rendering
};