// CBounceBehavior.cpp: implementation of the CBounceBehavior class.
//
//////////////////////////////////////////////////////////////////////

#include "BounceBehavior.h"
#include "CourseLoader.h"
#include "SnakeManager.h"
#include <crtdbg.h>     //for _ASSERT

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBounceBehavior::CBounceBehavior() : m_vMoveDir(0.0f, 0.0f, 0.0f)
{

}

CBounceBehavior::~CBounceBehavior()
{

}

CAbstractItemBehavior* CBounceBehavior::Clone(const char* pszIdentifier, ifstream& infile)
{
    _ASSERT(pszIdentifier);

    if(strcmp(pszIdentifier, "BOUNCE") == 0)
    {
        CBounceBehavior* pNewItem = new CBounceBehavior();
        infile >> pNewItem->m_vMoveDir.x
               >> pNewItem->m_vMoveDir.y;
        return pNewItem;
    }
    return NULL;
}

//move to the next frame of the animation
//ppSnakeVect is a NULL terminates array of CSnake*
void CBounceBehavior::FrameMove(CItem& moditem, CCourseLoader* pCourse,
                                CItem* pItem, UINT iItemSize, CSnakeManager* pSnakeManager)
{
    D3DXVECTOR3 vNewPos = moditem.m_vPos + 1.25f * m_vMoveDir;
 
    //find a wall intersection
    D3DXVECTOR3 vWallFrom;
    D3DXVECTOR3 vWallTo;
    if(pCourse->GetWallIntersect(moditem.m_vPos, vNewPos, &vWallFrom, &vWallTo) ||
       (pSnakeManager && pSnakeManager->FindSnakeIntersect(moditem.m_vPos, vNewPos,&vWallFrom, &vWallTo)))
    {
        //find the difference between the vectors
        vWallTo -= vWallFrom;

        //find the projection of the move direction onto the wall normal using
        //the formula |T| * cos(theta) * N / |N| = dot(T, N) / dot(N, N) * N
        FLOAT fScaleNorm = (m_vMoveDir.x * -vWallTo.y + m_vMoveDir.y * vWallTo.x) / (vWallTo.y * vWallTo.y + vWallTo.x * vWallTo.x);
        
        //add two times the projection onto the normal to the move direction
        m_vMoveDir.x -= 2 * fScaleNorm * -vWallTo.y;
        m_vMoveDir.y -= 2 * fScaleNorm * vWallTo.x;

        vNewPos = moditem.m_vPos + m_vMoveDir;
    }

    //make sure that the item conforms to terrain
    moditem.SetPos(vNewPos, pCourse);
}
