// ClientMenu.cpp: implementation of the CClientMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "ClientMenu.h"
#include "customvertex.h"
#include <crtdbg.h>     //for _ASSERT

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CClientMenu::CClientMenu(CNetworkManager* pnetman) :
    m_pNetManager(pnetman)
{
    m_pszHostAddr[0] = '\0';
    m_pStatusFont    = new CD3DFont( _T("Arial"), 18, D3DFONT_BOLD );
    m_pMenuFont      = new CD3DFont( _T("Arial"), 20, D3DFONT_BOLD );

    m_iMenuState   = m_pNetManager->GetIsConnected() ? CONNECTED : INPUT_HOSTADDR;
    m_bHasRendered = FALSE;
}

CClientMenu::~CClientMenu()
{
    DeleteDeviceObjects();
}

int CClientMenu::OnKeyDown(WPARAM wParam)
{
    switch(m_iMenuState)
    {
    case INPUT_HOSTADDR:
        switch(wParam)
        {
        case VK_RETURN:
            {
                if(lstrlen(m_pszHostAddr) > 0)
                {
                    m_bHasRendered = FALSE;
                    m_iMenuState = CONNECTING;
                }
                break;
            }
        case 8: //backspace
        case VK_DELETE:
            {
                UINT index = lstrlen(m_pszHostAddr);
                if(index > 0)
                    m_pszHostAddr[index - 1] = '\0';
                break;
            }
        }
        break;
    case WAIT_FOR_CONNECT:
        if(m_bHasRendered)
        {
            if(wParam == VK_RETURN)
            {
                m_iMenuState = CANCEL_CONNECT;
                return CAbstractMenu::MENUID_CANCELNET;
            }
        }
        break;
    }
    return CAbstractMenu::MENUID_NOTHING;
}

int CClientMenu::OnChar(WPARAM wParam)
{ 
    if(m_iMenuState == INPUT_HOSTADDR)
    {
        UINT index = lstrlen(m_pszHostAddr);
        if(((wParam >= 'A' && wParam <= 'Z') ||
            (wParam >= 'a' && wParam <= 'z') ||
            (wParam >= '0' && wParam <= '9') ||
            wParam == '.') && index < 256)
        {
            //append character onto end
            m_pszHostAddr[index]     = wParam;
            m_pszHostAddr[index + 1] = '\0';
        }
    }
    return CAbstractMenu::MENUID_NOTHING;
}

int CClientMenu::FrameMove()
{
    switch(m_iMenuState)
    {
    case CONNECTED:
        if(!m_pNetManager->GetIsConnected())
        {
            m_iDelay = 0;
            m_iMenuState = CONNECT_LOST;
        }
        else if(m_pNetManager->GetIsReadyToStart())
        {
            return CAbstractMenu::MENUID_STARTNETGAME;
        }
        break;
    case CONNECT_FAILED:
    case CONNECT_LOST:
        ++m_iDelay;
        if(m_iDelay > 70)
        {
            //return to the network menu
            return CAbstractMenu::MENUID_CANCELNET;
        }
        break;
    case CONNECTING:
        if(m_bHasRendered)
        {
            WCHAR pwszHostAddr[256];
            DXUtil_ConvertGenericStringToWide(pwszHostAddr, m_pszHostAddr);
            if(FAILED( m_pNetManager->StartClient(pwszHostAddr) ))
            {
                m_iDelay = 0;
                m_iMenuState = CONNECT_FAILED;
            }
            m_iMenuState = WAIT_FOR_CONNECT;
        }
        break;
    case WAIT_FOR_CONNECT:
        if(m_pNetManager->GetIsConnected())
            m_iMenuState = CONNECTED;
        break;
    }
    return CAbstractMenu::MENUID_NOTHING;
}

//render the title menu
void CClientMenu::RenderMenu(int iWidth, int iHeight)
{
    _ASSERT(m_pd3dDevice);
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 0x00000000, 1.0f, 0L );

    RenderText(iWidth, iHeight);
    m_bHasRendered = TRUE;
}

HRESULT CClientMenu::RenderText(int iWidth, int iHeight)
{
    _ASSERT(m_pStatusFont && m_pMenuFont);
    
    D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,128,128,128);
    D3DCOLOR fontSelectColor  = D3DCOLOR_ARGB(255,128,0,255);
    TCHAR    szMsg[MAX_PATH]  = TEXT(""); 
    SIZE     txtsize;
    FLOAT    fNextLine = 0.0f;

    switch(m_iMenuState)
    {
    case INPUT_HOSTADDR:
        m_pStatusFont->GetTextExtent("Input Host Name or IP address:", &txtsize);
        m_pStatusFont->DrawText((iWidth - txtsize.cx) / 2.0f,
                            (iHeight - txtsize.cy) / 2.0f + fNextLine,
                            fontSelectColor, "Input Host Name or IP address:");
        fNextLine += txtsize.cy;

        m_pStatusFont->GetTextExtent(m_pszHostAddr, &txtsize);
        m_pStatusFont->DrawText((iWidth - txtsize.cx) / 2.0f,
                            (iHeight - txtsize.cy) / 2.0f + fNextLine,
                            fontColor, m_pszHostAddr);
        fNextLine += txtsize.cy;
        break;
    case CONNECTING:
    case WAIT_FOR_CONNECT:
        m_pStatusFont->GetTextExtent("Establishing Connection...", &txtsize);
        m_pStatusFont->DrawText((iWidth - txtsize.cx) / 2.0f,
                            (iHeight - txtsize.cy) / 2.0f + fNextLine,
                            fontColor, "Establishing Connection...");
        fNextLine += txtsize.cy;

        m_pMenuFont->GetTextExtent("[Cancel]", &txtsize );
        m_pMenuFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                            3.0f * (iHeight - txtsize.cy) / 4.0f - txtsize.cy / 2.0f + fNextLine,
                            fontSelectColor, "[Cancel]" );
        break;
    case CONNECT_FAILED:
        m_pStatusFont->GetTextExtent("Connection to Host Failed", &txtsize);
        m_pStatusFont->DrawText((iWidth - txtsize.cx) / 2.0f,
                            (iHeight - txtsize.cy) / 2.0f + fNextLine,
                            D3DCOLOR_ARGB(255,255,0,0), "Connection to Host Failed");
        fNextLine += txtsize.cy;
        break;
    case CONNECTED:
        {
            m_pStatusFont->GetTextExtent("Connection Established", &txtsize);
            m_pStatusFont->DrawText((iWidth - txtsize.cx) / 2.0f,
                                (iHeight - txtsize.cy) / 2.0f + fNextLine,
                                fontSelectColor, "Connection Established");
            fNextLine += txtsize.cy;

            //print out the list of currently attached players with local player first
            if(CNetworkManager::m_LocalPlayerID)
            {
                m_pStatusFont->GetTextExtent( CNetworkManager::m_Players[CNetworkManager::m_LocalPlayerID].m_pszName, &txtsize );
                m_pStatusFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                                fNextLine, CNetworkManager::m_Players[CNetworkManager::m_LocalPlayerID].m_SnakeColor,
                                CNetworkManager::m_Players[CNetworkManager::m_LocalPlayerID].m_pszName );
                fNextLine += txtsize.cy;
            }
            map<DPNID, CNetworkManager::PlayerInfo>::iterator it;
            for(it = CNetworkManager::m_Players.begin(); it != CNetworkManager::m_Players.end(); ++it)
            {
                if(it->first != CNetworkManager::m_LocalPlayerID)
                {
                    m_pStatusFont->GetTextExtent( it->second.m_pszName, &txtsize );
                    m_pStatusFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                                    fNextLine, it->second.m_SnakeColor, it->second.m_pszName );
                    fNextLine += txtsize.cy;
                }
            }
            break;
        }
    case CONNECT_LOST:
        m_pStatusFont->GetTextExtent("Connection To Host Lost", &txtsize);
        m_pStatusFont->DrawText((iWidth - txtsize.cx) / 2.0f,
                            (iHeight - txtsize.cy) / 2.0f + fNextLine,
                            D3DCOLOR_ARGB(255,255,0,0), "Connection To Host Lost");
        fNextLine += txtsize.cy;
        break;
    }
    return S_OK;
}


HRESULT CClientMenu::InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice )
{
    _ASSERT(m_pStatusFont && m_pMenuFont && pd3dDevice);

    // Keep a local copy of the device
    m_pd3dDevice = pd3dDevice;

    // Init the font
    m_pStatusFont->InitDeviceObjects( m_pd3dDevice );
    m_pMenuFont->InitDeviceObjects( m_pd3dDevice );

    return S_OK;
}

HRESULT CClientMenu::RestoreDeviceObjects()
{
    _ASSERT(m_pStatusFont && m_pMenuFont && m_pd3dDevice);

    m_pStatusFont->RestoreDeviceObjects();
    m_pMenuFont->RestoreDeviceObjects();

    return S_OK;
}

HRESULT CClientMenu::InvalidateDeviceObjects()
{
    _ASSERT(m_pStatusFont && m_pMenuFont);
    m_pStatusFont->InvalidateDeviceObjects();
    m_pMenuFont->InvalidateDeviceObjects();
    return S_OK;
}

HRESULT CClientMenu::DeleteDeviceObjects()
{
    if(m_pStatusFont)
        m_pStatusFont->DeleteDeviceObjects();
    if(m_pMenuFont)
        m_pMenuFont->DeleteDeviceObjects();
    m_pd3dDevice = NULL;
    SAFE_DELETE( m_pStatusFont );
    SAFE_DELETE( m_pMenuFont );
    return S_OK;
}