// CreditsMenu.cpp: implementation of the CCreditsMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "CreditsMenu.h"
#include <crtdbg.h>     //for _ASSERT

char* CCreditsMenu::m_ppszCredits[] =
{
	"Programmed by:",
	"Ted Hung",
	" ",
	" ",
	"Artwork by:",
    "Bei Yang",
    "Jaijun Lim",
    "Ted Hung"
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCreditsMenu::CCreditsMenu() : m_iDelayCnt(0), m_bCreditsDone(false)
{
	m_pFont      = new CD3DFont( _T("Arial"), 20, D3DFONT_BOLD );
}

CCreditsMenu::~CCreditsMenu()
{
	DeleteDeviceObjects();
}

int CCreditsMenu::OnKeyDown(WPARAM wParam)
{
    return CAbstractMenu::MENUID_MAINMENU;
}

int CCreditsMenu::FrameMove()
{
	++m_iDelayCnt;
	if(m_bCreditsDone)
		return CAbstractMenu::MENUID_MAINMENU;
    return CAbstractMenu::MENUID_NOTHING;
}

void CCreditsMenu::RenderMenu(int iWidth, int iHeight)
{
    _ASSERT(m_pd3dDevice);
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 0x00000000, 1.0f, 0L );
    RenderText(iWidth, iHeight);
}


HRESULT CCreditsMenu::RenderText(int iWidth, int iHeight)
{
    _ASSERT(m_pFont);
    
    FLOAT fNextLine = 0.0f;
	UINT  iGrayVal = 0;
    int   iNumItems = sizeof(m_ppszCredits) / sizeof(char*);
	SIZE  txtsize;

	m_bCreditsDone = true;
    for(int i = 0; i < iNumItems; ++i)
    {
		m_pFont->GetTextExtent( m_ppszCredits[i], &txtsize );
		if(iHeight - m_iDelayCnt + fNextLine > 0)
		{
			iGrayVal = (UINT)(255 - 255.0f * fabs((2.0f * (iHeight - m_iDelayCnt + fNextLine) - iHeight) / iHeight ));
			m_pFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
								iHeight - m_iDelayCnt + fNextLine,
							   D3DCOLOR_ARGB(255,iGrayVal,iGrayVal,iGrayVal),
							   m_ppszCredits[i] );
			m_bCreditsDone = false;
		}
        fNextLine += txtsize.cy;
    }

    return S_OK;
}


HRESULT CCreditsMenu::InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice )
{
    _ASSERT(m_pFont && pd3dDevice);

    // Keep a local copy of the device
    m_pd3dDevice = pd3dDevice;

    // Init the font
    m_pFont->InitDeviceObjects( m_pd3dDevice );

    return S_OK;
}

HRESULT CCreditsMenu::RestoreDeviceObjects()
{
    _ASSERT(m_pd3dDevice && m_pFont);
    m_pFont->RestoreDeviceObjects();
    return S_OK;
}

HRESULT CCreditsMenu::InvalidateDeviceObjects()
{
    _ASSERT(m_pFont);
    m_pFont->InvalidateDeviceObjects();
    return S_OK;
}

HRESULT CCreditsMenu::DeleteDeviceObjects()
{
    if(m_pFont)
        m_pFont->DeleteDeviceObjects();
 
    m_pd3dDevice = NULL;
    SAFE_DELETE( m_pFont );
    return S_OK;
}