// HighScoreMenu.cpp: implementation of the CHighScoreMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "HighScoreMenu.h"
#include "customvertex.h"
#include <crtdbg.h>     //for _ASSERT

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHighScoreMenu::CHighScoreMenu(int iScore, CCourseLoader* pCourseLoader) :
    m_iAlpha(0), m_iScore(iScore), m_pVB(NULL),
    m_pSavedSurface(NULL), m_pCourseLoader(pCourseLoader)
{
    _ASSERT(pCourseLoader);

    m_pszHighName[0] = '\0';
    m_ScreenSize.cx = m_ScreenSize.cy = 0;
    m_pScoreFont    = new CD3DFont( _T("Times"), 36, D3DFONT_BOLD );
    m_pMenuFont     = new CD3DFont( _T("Arial"), 20, D3DFONT_BOLD );
}

CHighScoreMenu::~CHighScoreMenu()
{
    DeleteDeviceObjects();
}

int CHighScoreMenu::OnKeyDown(WPARAM wParam)
{
    _ASSERT(m_pCourseLoader);

    switch(wParam)
    {
    case VK_RETURN:
        if(lstrlen(m_pszHighName))
        {
            m_pCourseLoader->UpdateHighScore(m_iScore, m_pszHighName);
            return CAbstractMenu::MENUID_NEXTLEVEL;
        }
        break;
    case 8: //backspace
    case VK_DELETE:
        {
            UINT index = lstrlen(m_pszHighName);
            if(index > 0)
                m_pszHighName[index - 1] = '\0';
            break;
        }
    }
    return CAbstractMenu::MENUID_NOTHING;
}

int CHighScoreMenu::OnChar(WPARAM wParam)
{
    UINT index = lstrlen(m_pszHighName);
    if(((wParam >= 'A' && wParam <= 'Z') || (wParam >= 'a' && wParam <= 'z')) && index < 256)
    {
        //append character onto end
        m_pszHighName[index]     = wParam;
        m_pszHighName[index + 1] = '\0';
    }
    return CAbstractMenu::MENUID_NOTHING;
}

int CHighScoreMenu::FrameMove()
{
    if(m_iAlpha <= 255)
        ++m_iAlpha;
    return CAbstractMenu::MENUID_NOTHING;
}

//render the title menu
void CHighScoreMenu::RenderMenu(int iWidth, int iHeight)
{
    _ASSERT(m_pVB && m_pd3dDevice);

    LPDIRECT3DSURFACE8 pBackBuffer;
    if(SUCCEEDED( m_pd3dDevice->GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer) ))
    {
        if(!m_pSavedSurface)
        {
            D3DSURFACE_DESC   desc;
            
            if(FAILED( pBackBuffer->GetDesc(&desc) ) ||
               FAILED( m_pd3dDevice->CreateImageSurface(desc.Width, desc.Height, desc.Format, &m_pSavedSurface ) ) ||
               FAILED( m_pd3dDevice->CopyRects(pBackBuffer, NULL, 0, m_pSavedSurface, NULL) ))
            {
                SAFE_RELEASE( m_pSavedSurface );
            }
        }
        else
        {
            m_pd3dDevice->CopyRects(m_pSavedSurface, NULL, 0, pBackBuffer, NULL);
        }

        SAFE_RELEASE( pBackBuffer );
    }

    //find out whether the coordinates of the vertex buffer must be updated
    if(m_ScreenSize.cx != iWidth ||
       m_ScreenSize.cy != iHeight ||
       m_iAlpha <= 255)
    {
        m_ScreenSize.cx = iWidth;
        m_ScreenSize.cy = iHeight;

        COLORVERTEX* pVertices;
        if( FAILED( m_pVB->Lock( 0, 0, (BYTE**)&pVertices, 0  ) ) )
            return;
        pVertices[0].x = 0.0f;
        pVertices[0].y = (FLOAT)m_ScreenSize.cy - 0.1f;
        pVertices[0].z = 0.0f;
        pVertices[0].rhw = 1.0f;
        pVertices[0].color = D3DCOLOR_RGBA(0, 128, 0, m_iAlpha);

        pVertices[1].x = 0.0f;
        pVertices[1].y = 0.0f;
        pVertices[1].z = 0.0f;
        pVertices[1].rhw = 1.0f;
        pVertices[1].color = D3DCOLOR_RGBA(0, 128, 0, m_iAlpha);

        pVertices[2].x = (FLOAT)m_ScreenSize.cx - 0.1f;
        pVertices[2].y = (FLOAT)m_ScreenSize.cy - 0.1f;
        pVertices[2].z = 0.0f;
        pVertices[2].rhw = 1.0f;
        pVertices[2].color = D3DCOLOR_RGBA(0, 128, 0, m_iAlpha);

        pVertices[3].x = (FLOAT)m_ScreenSize.cx - 0.1f;
        pVertices[3].y = 0.0f;
        pVertices[3].z = 0.0f;
        pVertices[3].rhw = 1.0f;
        pVertices[3].color = D3DCOLOR_RGBA(0, 128, 0, m_iAlpha);
        m_pVB->Unlock();
    }

    m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,   D3DBLEND_SRCALPHA );
    m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND,  D3DBLEND_INVSRCALPHA );

    // Set up the textures
    m_pd3dDevice->SetTexture(0, NULL);

    m_pd3dDevice->LightEnable( 0, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE);
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
    // Some cards do not disable writing to Z when 
    // D3DRS_ZENABLE is FALSE. So do it explicitly
    m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );
    
    //darken the background
    m_pd3dDevice->SetVertexShader( D3DFVF_COLORVERTEX);
    m_pd3dDevice->SetStreamSource( 0, m_pVB, sizeof(COLORVERTEX) );
    m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2);

    //render the title mesh
    RenderText();

    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE);

    m_pd3dDevice->LightEnable( 0, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE);
    m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
}

HRESULT CHighScoreMenu::RenderText()
{
    _ASSERT(m_pScoreFont && m_pMenuFont);
    
    D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,128,128,128);
    D3DCOLOR fontSelectColor  = D3DCOLOR_ARGB(255,128,0,255);
    TCHAR    szMsg[MAX_PATH]     = TEXT(""); 
    SIZE     txtsize;

    //draw the "Game Over" and the score
    m_pScoreFont->GetTextExtent( "Level Completed", &txtsize );
    FLOAT    fNextLine = -2.0f * txtsize.cy;

    m_pScoreFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                       (m_ScreenSize.cy - txtsize.cy) / 2.0f + fNextLine,
                       D3DCOLOR_ARGB(255,255,0,0), "Level Completed" );
    fNextLine += txtsize.cy;

    lstrcpy(szMsg, "Score: ");
    itoa(m_iScore, szMsg + lstrlen(szMsg), 10);
    m_pScoreFont->GetTextExtent( szMsg, &txtsize );
    m_pScoreFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                       (m_ScreenSize.cy - txtsize.cy) / 2.0f + fNextLine,
                       D3DCOLOR_ARGB(255,255,0,0), szMsg );
    fNextLine += txtsize.cy;

    m_pScoreFont->GetTextExtent( "New High Score!", &txtsize );
    m_pScoreFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                       (m_ScreenSize.cy - txtsize.cy) / 2.0f + fNextLine,
                       D3DCOLOR_ARGB(255,255,255,0), "New High Score!");
    fNextLine += txtsize.cy;

    m_pScoreFont->GetTextExtent( "Enter Name:", &txtsize );
    m_pScoreFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                       (m_ScreenSize.cy - txtsize.cy) / 2.0f + fNextLine,
                       D3DCOLOR_ARGB(255,255,255,0), "Enter Name");
    fNextLine += txtsize.cy;

    m_pScoreFont->GetTextExtent( m_pszHighName, &txtsize );
    m_pScoreFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                       (m_ScreenSize.cy - txtsize.cy) / 2.0f + fNextLine,
                       D3DCOLOR_ARGB(255,255,255,0), m_pszHighName);
    fNextLine += txtsize.cy;
    return S_OK;
}


HRESULT CHighScoreMenu::InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice )
{
    _ASSERT(m_pScoreFont && m_pMenuFont && pd3dDevice);

    // Keep a local copy of the device
    m_pd3dDevice = pd3dDevice;

    // Init the font
    m_pScoreFont->InitDeviceObjects( m_pd3dDevice );
    m_pMenuFont->InitDeviceObjects( m_pd3dDevice );

    return S_OK;
}

HRESULT CHighScoreMenu::RestoreDeviceObjects()
{
    _ASSERT(m_pScoreFont && m_pMenuFont && m_pd3dDevice);

    m_pScoreFont->RestoreDeviceObjects();
    m_pMenuFont->RestoreDeviceObjects();

    if( FAILED( m_pd3dDevice->CreateVertexBuffer( sizeof(COLORVERTEX) * 4, D3DUSAGE_WRITEONLY, D3DFVF_COLORVERTEX, D3DPOOL_DEFAULT, &m_pVB ) ) )
        return E_FAIL;
    
    return S_OK;
}

HRESULT CHighScoreMenu::InvalidateDeviceObjects()
{
    _ASSERT(m_pScoreFont && m_pMenuFont);
    m_pScoreFont->InvalidateDeviceObjects();
    m_pMenuFont->InvalidateDeviceObjects();
    SAFE_RELEASE( m_pVB );
    SAFE_RELEASE( m_pSavedSurface );
    return S_OK;
}

HRESULT CHighScoreMenu::DeleteDeviceObjects()
{
    if(m_pScoreFont)
        m_pScoreFont->DeleteDeviceObjects();
    if(m_pMenuFont)
        m_pMenuFont->DeleteDeviceObjects();
    m_pd3dDevice = NULL;
    SAFE_RELEASE( m_pVB );
    SAFE_RELEASE( m_pSavedSurface );
    SAFE_DELETE( m_pScoreFont );
    SAFE_DELETE( m_pMenuFont );
    return S_OK;
}