// Item.h: interface for the CItem class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "d3dfile.h"
#include "Snake.h"
#include <D3dx8math.h>
#include <fstream>
using namespace std;

class CCourseLoader;
class CSnakeManager;
class CAbstractItemBehavior;

class CItem
{
public:
	CItem();
	~CItem();

    //reads the behaviors for the item from the specified file
    void ReadItemBehaviors(UINT iNumBehaviors, ifstream& infile);

    //render the snake item
    void Render(LPDIRECT3DDEVICE8 pd3dDevice);

    //set the position of this item
    //returns whether there was a terrain intersect or not
    BOOL SetPos(const D3DXVECTOR3& vPos, CCourseLoader* pCourse);

    //set the mesh to render with
    void SetMesh(CD3DMesh* pMesh);

    //returns whether this item is visible
    BOOL IsVisible();

    //returns whether this item should be collision detected
    BOOL IsValid();

    /////////////////////////////////////////////////
    // Item Behavior Functions
    /////////////////////////////////////////////////

    //reset this item to the original state: visible, valid
    void Reset(CCourseLoader* pCourseLoader = NULL);

    //move to the next frame of the animation
    //ppSnakeVect is a NULL terminates array of CSnake*
    void FrameMove(CCourseLoader* pCourse,
                   CItem* pItem, UINT iItemSize,
                   CSnakeManager* pSnakeManager = NULL);

    //checks whether the snake is in the bounds of this item
    BOOL CheckInBounds(CSnake* pSnake);

    //performs an behavior associated with the item on the given snake
    void ModifySnake(CSnake* pSnake);

    FLOAT         m_fElevation; //the elevation of this item over the terrain
    D3DXVECTOR3   m_vPos;       //the position of this item
    D3DXVECTOR3   m_vRotation;  //the rotation of this item

protected:
    //invalidate this device to turn off collision detection
    //and remove from rendering
    void Invalidate();

    BOOL                    m_bVisible;   //determines whether to render this item
    BOOL                    m_bValid;     //determines whether to collision detect this item
    CD3DMesh*               m_pMesh;

    CAbstractItemBehavior** m_ppBehaviors;
    UINT                    m_iNumBehaviors;
};