// ItemBehaviorFactory.cpp: implementation of the CItemBehaviorFactory class.
//
//////////////////////////////////////////////////////////////////////

#include "ItemBehaviorFactory.h"

#include "RadiusBehavior.h"
#include "RegionBehavior.h"

#include "BounceBehavior.h"
#include "ElevateBehavior.h"
#include "RotateBehavior.h"

#include "EatBehavior.h"
#include "LengthBehavior.h"
#include "ScoreBehavior.h"
#include "SpeedBehavior.h"

#include "PositionBehavior.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CItemBehaviorFactory::CItemBehaviorFactory()
{
    m_SnakeItemVect.push_back(new CRadiusBehavior());
    m_SnakeItemVect.push_back(new CRegionBehavior());

    m_SnakeItemVect.push_back(new CEatBehavior());
    m_SnakeItemVect.push_back(new CBounceBehavior());
    m_SnakeItemVect.push_back(new CElevateBehavior());
    m_SnakeItemVect.push_back(new CRotateBehavior());

    m_SnakeItemVect.push_back(new CLengthBehavior());
    m_SnakeItemVect.push_back(new CScoreBehavior());
    m_SnakeItemVect.push_back(new CSpeedBehavior());
    
    m_SnakeItemVect.push_back(new CPositionBehavior());
}

CItemBehaviorFactory::~CItemBehaviorFactory()
{
    vector<CAbstractItemBehavior*>::iterator it;
    for(it = m_SnakeItemVect.begin(); it != m_SnakeItemVect.end(); ++it)
        delete *it;
}

//returns a pointer to a new snake item from an identifier and a file
CAbstractItemBehavior* CItemBehaviorFactory::CreateFromFile(const char* pszIdentifier, ifstream& infile)
{
    CAbstractItemBehavior* pFoundItem;
    vector<CAbstractItemBehavior*>::iterator it;
    for(it = m_SnakeItemVect.begin(); it != m_SnakeItemVect.end(); ++it)
    {
        if(pFoundItem = (*it)->Clone(pszIdentifier, infile))
        {
            return pFoundItem;
        }
    }
    return NULL;
}
