// ItemManager.h: interface for the CItemManager class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "Item.h"
#include <fstream>
using namespace std;

class CSnakeManager;

class CItemManager  
{
public:
	CItemManager();
	~CItemManager();

    //reads the snake items from the given file
    HRESULT ReadItems(const char* pszPathname, ifstream& infile);

    //resets all of the snake items to their initial states
    void    Reset(CCourseLoader* pCourse = NULL);

    //advance to the next frame of animation for the snake items
    //ppSnakeVect is a NULL terminated array of CSnake*
    void    FrameMove(CCourseLoader* pCourse, CSnakeManager* pSnakeManager = NULL);

    //returns the item specified by the ID
    CItem*  GetItem(UINT iItemID);

    //returns whether the given snake intersects with an item and returns
    //the given item which the caller can invalidate, uses the lastitem
    //returned by GetItemIntersect to start search
    CItem*  GetItemIntersect(CSnake* pSnake, CItem* pStartItem = NULL);

    // Initializing and destroying device-dependent objects
    void    Render();
    HRESULT InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice );
    HRESULT RestoreDeviceObjects();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();

protected:
    //positions the items contained in the manager into the course
    void    PositionItems(CCourseLoader* pCourse);

    LPDIRECT3DDEVICE8           m_pd3dDevice;       // A D3DDevice used for rendering
    CD3DMesh*                   m_pMeshObject;
    UINT                        m_iMeshObjectSize;

    CItem*                      m_pItem;
    UINT                        m_iItemSize;
};