// LevelMenu.cpp: implementation of the CLevelMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "LevelMenu.h"
#include <crtdbg.h>     //for _ASSERT

CAbstractMenu::MenuItem CLevelMenu::m_pLevelMenuItems[] =
{
    {"Play Level", CAbstractMenu::MENUID_PLAYLEVEL},
    {"Main Menu",  CAbstractMenu::MENUID_MAINMENU}
};

CAbstractMenu::MenuItem CLevelMenu::m_pServerMenuItems[] =
{
    {"Select Level", CAbstractMenu::MENUID_SERVER},
    {"Cancel",       CAbstractMenu::MENUID_CANCELNET}
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLevelMenu::CLevelMenu(CCourseLoader*  pCourseLoader,
                       TCHAR**         ppszLevelFileNames,
                       UINT            iNumLevels,
                       CItemManager*   pItemManager,
                       CAbstractGoal** ppGoal,
                       BOOL            bIsServer) :
    m_pCourseLoader(pCourseLoader),
    m_pItemManager(pItemManager),
    m_ppGoal(ppGoal),
    m_pArrowVB(NULL),
    m_ppszLevelFileNames(ppszLevelFileNames),
    m_iNumLevels(iNumLevels),
    m_iCurLevel(0),
    m_fRotAngle(0.0f),
    m_iCurSel(0),
    m_iArrowOffset(0),
    m_bArrowDir(TRUE),
    m_bIsServer(bIsServer)
{
    _ASSERT(ppszLevelFileNames);
    _ASSERT(iNumLevels);

    m_pFont = new CD3DFont( _T("Arial"), 18, D3DFONT_BOLD );
}

CLevelMenu::~CLevelMenu()
{
    DeleteDeviceObjects();
}

//gets the current level
void CLevelMenu::SetCurLevel(UINT iLevel)
{
    m_iCurLevel = iLevel;
}

//sets the current level
UINT CLevelMenu::GetCurLevel()
{
    return m_iCurLevel;
}

int CLevelMenu::OnKeyDown(WPARAM wParam)
{
    switch(wParam)
    {
    case VK_RETURN:
        if(m_bIsServer)
            return m_pServerMenuItems[m_iCurSel].m_iMenuID;
        return m_pLevelMenuItems[m_iCurSel].m_iMenuID;
        break;
    case VK_DOWN:
        ++m_iCurSel;
        if(m_bIsServer)
            m_iCurSel %= sizeof(m_pServerMenuItems) / sizeof(CAbstractMenu::MenuItem);
        else
            m_iCurSel %= sizeof(m_pLevelMenuItems) / sizeof(CAbstractMenu::MenuItem);
        break;
    case VK_UP:
        --m_iCurSel;
        if(m_iCurSel < 0)
        {
            if(m_bIsServer)
                m_iCurSel = sizeof(m_pServerMenuItems) / sizeof(CAbstractMenu::MenuItem) - 1;
            else
                m_iCurSel = sizeof(m_pLevelMenuItems) / sizeof(CAbstractMenu::MenuItem) - 1;
        }
        break;
    case VK_RIGHT:
        if(m_iCurLevel < m_iNumLevels - 1)
        {
            ++m_iCurLevel;
            if(m_ppGoal)
                SAFE_DELETE( *m_ppGoal );
            m_pCourseLoader->LoadCourseFile("Levels", m_ppszLevelFileNames[m_iCurLevel], m_pItemManager, m_ppGoal);
            m_pCourseLoader->InitDeviceObjects(m_pd3dDevice);
            m_pCourseLoader->RestoreDeviceObjects();
            m_pCourseLoader->Reset();
            if(m_pItemManager)
                m_pItemManager->Reset(m_pCourseLoader);
        }
        /*if(m_bNextLevelAvailable)
        {
            if(m_bIsServer)
                return CAbstractMenu::MENUID_NEXTLEVELNET;
            return CAbstractMenu::MENUID_NEXTLEVEL;
        }*/
        break;
    case VK_LEFT:
        if(m_iCurLevel > 0)
        {
            --m_iCurLevel;
            if(m_ppGoal)
                SAFE_DELETE( *m_ppGoal );
            m_pCourseLoader->LoadCourseFile("Levels", m_ppszLevelFileNames[m_iCurLevel], m_pItemManager, m_ppGoal);
            m_pCourseLoader->InitDeviceObjects(m_pd3dDevice);
            m_pCourseLoader->RestoreDeviceObjects();
            m_pCourseLoader->Reset();
            if(m_pItemManager)
                m_pItemManager->Reset(m_pCourseLoader);
        }
        /*if(m_bPrevLevelAvailable)
        {
            if(m_bIsServer)
                return CAbstractMenu::MENUID_PREVLEVELNET;
            return CAbstractMenu::MENUID_PREVLEVEL;
        }*/
        break;
    }
    return CAbstractMenu::MENUID_NOTHING;
}

int CLevelMenu::FrameMove()
{
    _ASSERT(m_pd3dDevice);

    m_fRotAngle -= 0.01f;
    if(m_fRotAngle < 0.0f)
        m_fRotAngle += 2 * 3.1415f;

    //animate the next and previous arrows
    static UINT iDelay = 0;
    ++iDelay;
    if(iDelay > 3)
    {
        m_iArrowOffset += m_bArrowDir ? 1 : -1;
        if(m_iArrowOffset == 10 || m_iArrowOffset == 0)
            m_bArrowDir = !m_bArrowDir;
        iDelay = 0;
    }

    m_pCourseLoader->FrameMove();
    m_pItemManager->FrameMove(m_pCourseLoader);
    return CAbstractMenu::MENUID_NOTHING;
}

//render the title menu
void CLevelMenu::RenderMenu(int iWidth, int iHeight)
{
    _ASSERT(m_pCourseLoader && m_pd3dDevice);
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_ZBUFFER, 0x00000000, 1.0f, 0L );

    //set up the camera view position
    D3DXMATRIX matView;
    D3DXVECTOR3 vLookatPt   = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUpVec      = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXVECTOR3 vEyePt      = D3DXVECTOR3( 0.0f, -100.0f, 100.0f);
    D3DXMatrixLookAtLH( &matView, &vEyePt, &vLookatPt, &vUpVec );
    m_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

    //render the sky
    m_pCourseLoader->RenderSky();

    //render the course
    D3DXMATRIX matWorld;
    D3DXMatrixRotationZ(&matWorld, -m_fRotAngle);
    m_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );

    m_pItemManager->Render();
    m_pCourseLoader->Render();

    D3DXMatrixIdentity(&matWorld);
    m_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );

    //render the course name and menu options
    RenderText(iWidth, iHeight);

    //render the left and right arrows
    RenderArrows(iWidth, iHeight);

    //disable the texture transformation matrix
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE );
}

//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Renders stats and help text to the scene.
//-----------------------------------------------------------------------------
HRESULT CLevelMenu::RenderText(int iWidth, int iHeight)
{
    _ASSERT(m_pCourseLoader && m_pFont);
    
    D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,128,128,128);
    D3DCOLOR fontSelectColor  = D3DCOLOR_ARGB(255,128,0,255);

    // Output display stats
    SIZE    txtsize;
    FLOAT   fNextLine = 0.0f; 
    TCHAR   pszTemp[256];

    if(m_bIsServer)
    {
        //draw the course title
        m_pFont->GetTextExtent( "Select a Level For Network Game:", &txtsize );
        DrawTextOutlined((iWidth - txtsize.cx) / 2.0f, 5.0f, 3.0f, fontSelectColor,
            D3DCOLOR_ARGB(255,0,0,0), "Select a Level For Network Game:");
        fNextLine += txtsize.cy;
    }

    //draw the course title
    m_pFont->GetTextExtent( m_pCourseLoader->GetCourseTitle(), &txtsize );
    DrawTextOutlined((iWidth - txtsize.cx) / 2.0f, 5.0f + fNextLine, 3.0f,
                      m_bIsServer ? fontColor : fontSelectColor,
                      D3DCOLOR_ARGB(255,0,0,0), m_pCourseLoader->GetCourseTitle());
    fNextLine += txtsize.cy;

    if(!m_bIsServer)
    {
        //draw the high score
        ULONG        iHighScore;
        const TCHAR* pszHighName;
        TCHAR        pszTemp2[256];
        
        m_pCourseLoader->GetHighScore(&iHighScore, &pszHighName);
        itoa(iHighScore, pszTemp2, 10);
        strcpy(pszTemp, "High Score: ");
        strcat(pszTemp, pszHighName);
        strcat(pszTemp, " ");
        strcat(pszTemp, pszTemp2);

        m_pFont->GetTextExtent( pszTemp, &txtsize );
        DrawTextOutlined((iWidth - txtsize.cx) / 2.0f,
                        5.0f + fNextLine, 2.0f,
                        fontColor,
                        D3DCOLOR_ARGB(255,0,0,0),
                        pszTemp);
    }

    //place brackets around the selected item
    pszTemp[0] = '[';
    int iNumItems;
    if(m_bIsServer)
    {
        strcpy(pszTemp + 1, m_pServerMenuItems[m_iCurSel].m_pszMenuStr);
        iNumItems = sizeof(m_pServerMenuItems) / sizeof(CAbstractMenu::MenuItem);
    }
    else
    {
        strcpy(pszTemp + 1, m_pLevelMenuItems[m_iCurSel].m_pszMenuStr);
        iNumItems = sizeof(m_pLevelMenuItems) / sizeof(CAbstractMenu::MenuItem);
    }
    strcat(pszTemp, "]");

    fNextLine = 0.0f; 
    for(int i = iNumItems - 1; i >= 0; --i)
    {
        if(i == m_iCurSel)
        {
            m_pFont->GetTextExtent( pszTemp, &txtsize );
            DrawTextOutlined((iWidth - txtsize.cx) / 2.0f,
                             iHeight - txtsize.cy - fNextLine, 2.0f,
                             fontSelectColor,
                             D3DCOLOR_ARGB(255,0,0,0),
                             pszTemp);
        }
        else
        {
            if(m_bIsServer)
            {
                m_pFont->GetTextExtent( m_pServerMenuItems[i].m_pszMenuStr, &txtsize );
                DrawTextOutlined((iWidth - txtsize.cx) / 2.0f,
                                iHeight - txtsize.cy - fNextLine, 2.0f,
                                fontColor,
                                D3DCOLOR_ARGB(255,0,0,0),
                                m_pServerMenuItems[i].m_pszMenuStr);
            }
            else
            {
                m_pFont->GetTextExtent( m_pLevelMenuItems[i].m_pszMenuStr, &txtsize );
                DrawTextOutlined((iWidth - txtsize.cx) / 2.0f,
                                iHeight - txtsize.cy - fNextLine, 2.0f,
                                fontColor,
                                D3DCOLOR_ARGB(255,0,0,0),
                                m_pLevelMenuItems[i].m_pszMenuStr);
            }
        }
        fNextLine += txtsize.cy;
    }

    return S_OK;
}

//renders the left and right arrows if necessary
void CLevelMenu::RenderArrows(int iWidth, int iHeight)
{
    m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
    m_pd3dDevice->LightEnable( 0, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE);
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );

    m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,   D3DBLEND_SRCALPHA );
    m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND,  D3DBLEND_INVSRCALPHA );
    m_pd3dDevice->SetTexture(0, NULL);
    m_pd3dDevice->SetVertexShader( D3DFVF_COLORVERTEX);
    m_pd3dDevice->SetStreamSource( 0, m_pArrowVB, sizeof(COLORVERTEX) );

    //draw the next and previous arrows
    if(m_iCurLevel < m_iNumLevels - 1)
    {
        COLORVERTEX* pVertices;
        if( FAILED( m_pArrowVB->Lock( 0, 0, (BYTE**)&pVertices, 0  ) ) )
            return;
        pVertices[0].x = iWidth - 11.0f + m_iArrowOffset;
        pVertices[0].y = iHeight / 2.0f;
        pVertices[0].z = 0.0f;
        pVertices[0].rhw = 1.0f;
        pVertices[0].color = D3DCOLOR_RGBA(255, 0, 0, 128);

        pVertices[1].x = iWidth - 41.0f + m_iArrowOffset;
        pVertices[1].y = iHeight / 2.0f + 40.0f;
        pVertices[1].z = 0.0f;
        pVertices[1].rhw = 1.0f;
        pVertices[1].color = D3DCOLOR_RGBA(255, 0, 0, 128);

        pVertices[2].x = iWidth - 41.0f + m_iArrowOffset;
        pVertices[2].y = iHeight / 2.0f - 40.0f;
        pVertices[2].z = 0.0f;
        pVertices[2].rhw = 1.0f;
        pVertices[2].color = D3DCOLOR_RGBA(255, 0, 0, 128);
        m_pArrowVB->Unlock();

        //render the triangle
        m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 1);
    }
    if(m_iCurLevel)
    {
        COLORVERTEX* pVertices;
        if( FAILED( m_pArrowVB->Lock( 0, 0, (BYTE**)&pVertices, 0  ) ) )
            return;
        pVertices[0].x = 11.0f - m_iArrowOffset;
        pVertices[0].y = iHeight / 2.0f;
        pVertices[0].z = 0.0f;
        pVertices[0].rhw = 1.0f;
        pVertices[0].color = D3DCOLOR_RGBA(255, 0, 0, 128);

        pVertices[1].x = 41.0f - m_iArrowOffset;
        pVertices[1].y = iHeight / 2.0f - 40.0f;
        pVertices[1].z = 0.0f;
        pVertices[1].rhw = 1.0f;
        pVertices[1].color = D3DCOLOR_RGBA(255, 0, 0, 128);

        pVertices[2].x = 41.0f - m_iArrowOffset;
        pVertices[2].y = iHeight / 2.0f + 40.0f;
        pVertices[2].z = 0.0f;
        pVertices[2].rhw = 1.0f;
        pVertices[2].color = D3DCOLOR_RGBA(255, 0, 0, 128);
        m_pArrowVB->Unlock();

        //render the triangle
        m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 1);
    }

    m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE);
    m_pd3dDevice->LightEnable( 0, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
}

void CLevelMenu::DrawTextOutlined(FLOAT fx, FLOAT fy, FLOAT fOutlineWidth, DWORD dwColor, DWORD dwColorOutline, TCHAR* strText)
{
    _ASSERT(m_pFont);

    for(int ix = -1; ix <= 1; ++ix)
    {
        for(int iy = -1; iy <= 1; ++iy)
        {
            if(ix || iy)
                m_pFont->DrawText( fx + fOutlineWidth * ix, fy + fOutlineWidth * iy, dwColorOutline, strText);
        }
    }
    m_pFont->DrawText( fx, fy, dwColor, strText );
}

HRESULT CLevelMenu::InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice )
{
    _ASSERT(m_pFont && pd3dDevice);

    // Keep a local copy of the device
    m_pd3dDevice = pd3dDevice;

    // Init the font
    m_pFont->InitDeviceObjects( m_pd3dDevice );

    // Initialize the current level
    if(m_ppGoal)
        SAFE_DELETE( *m_ppGoal );
    m_pCourseLoader->LoadCourseFile("Levels", m_ppszLevelFileNames[m_iCurLevel], m_pItemManager, m_ppGoal);
    m_pCourseLoader->InitDeviceObjects(m_pd3dDevice);
    m_pCourseLoader->RestoreDeviceObjects();
    m_pCourseLoader->Reset();
    if(m_pItemManager)
        m_pItemManager->Reset(m_pCourseLoader);

    return S_OK;
}

HRESULT CLevelMenu::RestoreDeviceObjects()
{
    _ASSERT(m_pd3dDevice && m_pFont);
    m_pFont->RestoreDeviceObjects();

    //create the right arrow
    if( FAILED( m_pd3dDevice->CreateVertexBuffer( sizeof(COLORVERTEX) * 3,
        D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC, D3DFVF_COLORVERTEX, D3DPOOL_DEFAULT, &m_pArrowVB ) ) )
        return E_FAIL;

    return S_OK;
}

HRESULT CLevelMenu::InvalidateDeviceObjects()
{
    _ASSERT(m_pFont);
    m_pFont->InvalidateDeviceObjects();
    SAFE_RELEASE( m_pArrowVB );
    return S_OK;
}

HRESULT CLevelMenu::DeleteDeviceObjects()
{
    if(m_pFont)
        m_pFont->DeleteDeviceObjects();
    m_pd3dDevice = NULL;
    SAFE_DELETE( m_pFont );
    SAFE_RELEASE( m_pArrowVB );
    return S_OK;
}