// LevelMenu.h: interface for the C:LevelMenu class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "AbstractMenu.h"
#include "d3dfile.h"
#include "d3dfont.h"
#include "dxutil.h"
#include "CourseLoader.h"
#include "ItemManager.h"

class CLevelMenu : public CAbstractMenu
{
public:
	CLevelMenu(CCourseLoader*  pCourseLoader,
               TCHAR**         ppszLevelFileNames,
               UINT            iNumLevels,
               CItemManager*   pItemManager = NULL,
               CAbstractGoal** ppGoal = NULL,
               BOOL            bIsServer = FALSE);
	virtual ~CLevelMenu();

    //gets and sets the current level
    void SetCurLevel(UINT iLevel);
    UINT GetCurLevel();

    // CAbstractMenu functions
    virtual int  OnKeyDown(WPARAM wParam);
    virtual int  FrameMove();

	// Device Initialization and Rendering Functions
    virtual void    RenderMenu(int iWidth, int iHeight);
    virtual HRESULT InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice );
    virtual HRESULT RestoreDeviceObjects();
    virtual HRESULT InvalidateDeviceObjects();
    virtual HRESULT DeleteDeviceObjects();
private:
    HRESULT RenderText(int iWidth, int iHeight);
    void RenderArrows(int iWidth, int iHeight);
    void DrawTextOutlined(FLOAT x, FLOAT y, FLOAT fOutlineWidth, DWORD dwColor, DWORD dwColorOutline, TCHAR* strText);

    static CAbstractMenu::MenuItem m_pLevelMenuItems[];
    static CAbstractMenu::MenuItem m_pServerMenuItems[];

    CCourseLoader*          m_pCourseLoader;
    CItemManager*           m_pItemManager;
    CAbstractGoal**         m_ppGoal;

    LPDIRECT3DVERTEXBUFFER8 m_pArrowVB;
    CD3DFont*               m_pFont;                // Font for drawing text
    FLOAT                   m_fRotAngle;
    int                     m_iCurSel;

    UINT                    m_iArrowOffset;
    BOOL                    m_bArrowDir;

    TCHAR**                 m_ppszLevelFileNames;
    UINT                    m_iNumLevels;
    UINT                    m_iCurLevel;
    BOOL                    m_bIsServer;    //determines whether this is a server level selection
                                            //or a single player level selection
};