// Lofter.h: interface for the CLofter class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include <D3DX8.h>
#include "customvertex.h"

class CLofter  
{
public:
    CLofter();
    ~CLofter();

    enum LOFT_OPTIONS
    {
        LOFT_DEFAULT     = 0x0,     //BEVELTO, BEVELFROM, SCALEBOTH, LOFT_TRIANGLESTRIP

        //if these flags are given then bevels must be given
        LOFT_NOBEVELFROM = 0x1, 
        LOFT_NOBEVELTO   = 0x2,
        LOFT_NOBEVEL     = 0x3,

        //if these flags are given then scales must be given
        LOFT_SCALEFROM   = 0x4, 
        LOFT_SCALETO     = 0x8,
        LOFT_SCALEBOTH   = 0xc,

        //required flags for correctly ending/beginning a series of segments in a triangle strip
        LOFT_ENDFROM     = 0x10, 
        LOFT_ENDTO       = 0x20,
        LOFT_ENDBOTH     = 0x30,

        //flag to specify a particular type of rendering: triangle strip by default
        LOFT_INDEXED     = 0x40
    };

    //draw a lofted section that is beveled based on a path and a section outline
    static CUSTOMVERTEX* LoftBeveledSection(const D3DXVECTOR3* pVertArray,
                                            const VECTOR2TEX*  pSect,
                                            int                iSectSize,
                                            CUSTOMVERTEX*      pVertices,
                                            D3DCOLOR           color,
                                            DWORD              dwFlags = LOFT_DEFAULT);

    //draw a lofted section that is beveled based on a path and a section outline
    static CUSTOMVERTEX* LoftBeveledSectionIndexed(const D3DXVECTOR3* pVertArray,
                                                   const VECTOR2TEX*  pSect,
                                                   int                iSectSize,
                                                   CUSTOMVERTEX*      pVertices,
                                                   D3DCOLOR           color,
                                                   DWORD              dwFlags = LOFT_DEFAULT);

    //used to calculate the indices for the vertices calculate in LoftBeveledSection
    static WORD* CalcBeveledSectionIndices(WORD* pIndices,
                                           int iSectSize,
                                           int iNumSegments,
                                           DWORD dwFlags = LOFT_DEFAULT);

private:
    //calculates the right vector based on the bevel between the two delta vectors
    // and a right vector to the main vector in order to maintain width at the bevel
    static void CalcBevelUpRight(D3DXVECTOR3*       pBevelUp,
                                 D3DXVECTOR3*       pBevelRight,
                                 const D3DXVECTOR3& delta,
                                 const D3DXVECTOR3& deltaBevel,
                                 const D3DXVECTOR3& right,
                                 const D3DXVECTOR3& up);

    //calculates the vertices based on a given bevel 
    static void CalcSectionVertex(CUSTOMVERTEX*      pVertex,
                                  const D3DXVECTOR3& vSrcVert,
                                  const D3DXVECTOR2& vNormal,
                                  const VECTOR2TEX&  vtSectVert,
                                  const D3DXVECTOR3& vBevelUp,
                                  const D3DXVECTOR3& vBevelRight,
                                  const D3DXVECTOR3& vScale);
};