// LogoMenu.cpp: implementation of the CLogoMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "LogoMenu.h"
#include "resource.h"
#include "customvertex.h"
#include "d3dutil.h"
#include <crtdbg.h>     //for _ASSERT

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLogoMenu::CLogoMenu() : m_iDelayCnt(0), m_pLogoTex(NULL), m_iAlpha(0), m_bFadeOut(FALSE)
{
    m_pVBLogo = NULL;
    m_pVBSky  = NULL;
}

CLogoMenu::~CLogoMenu()
{
    DeleteDeviceObjects();
}

int CLogoMenu::OnKeyDown(WPARAM wParam)
{
    return CAbstractMenu::MENUID_MAINMENU;
}

int CLogoMenu::FrameMove()
{
    if(m_bFadeOut)
    {
        if(m_iDelayCnt > 50)
        {
            if(m_iAlpha > 0)
                --m_iAlpha;
            else
                return CAbstractMenu::MENUID_MAINMENU;
        }
        else
        {
            ++m_iDelayCnt;
        }
    }
    else
    {
        if(m_iAlpha < 255)
            ++m_iAlpha;
        else
            m_bFadeOut = TRUE;
    }
    return CAbstractMenu::MENUID_NOTHING;  
}

//render the title menu
void CLogoMenu::RenderMenu(int iWidth, int iHeight)
{
    _ASSERT(m_pd3dDevice && m_pLogoTex);

    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET, 0x00000000, 1.0f, 0L );

    D3DSURFACE_DESC texdesc;
    m_pLogoTex->GetLevelDesc(0, &texdesc);

    TEXTUREVERTEX* pVertices;
    if( FAILED( m_pVBLogo->Lock( 0, 0, (BYTE**)&pVertices, 0  ) ) )
        return;
    pVertices[0].x = 0.0f;
    pVertices[0].y = (iHeight + texdesc.Height) / 2.0f;
    pVertices[0].z = 0.0f;
    pVertices[0].rhw = 1.0f;
    pVertices[0].tu  = 0.0f;
    pVertices[0].tv  = 1.0f;

    pVertices[1].x = 0.0f;
    pVertices[1].y = (iHeight - texdesc.Height) / 2.0f;
    pVertices[1].z = 0.0f;
    pVertices[1].rhw = 1.0f;
    pVertices[1].tu  = 0.0f;
    pVertices[1].tv  = 0.01f;

    pVertices[2].x = (iWidth - texdesc.Width) / 2.0f;
    pVertices[2].y = (iHeight + texdesc.Height) / 2.0f;
    pVertices[2].z = 0.0f;
    pVertices[2].rhw = 1.0f;
    pVertices[2].tu  = 0.0f;
    pVertices[2].tv  = 1.0f;

    pVertices[3].x = (iWidth - texdesc.Width) / 2.0f;
    pVertices[3].y = (iHeight - texdesc.Height) / 2.0f;
    pVertices[3].z = 0.0f;
    pVertices[3].rhw = 1.0f;
    pVertices[3].tu  = 0.0f;
    pVertices[3].tv  = 0.01f;

    pVertices[4].x = (iWidth + texdesc.Width) / 2.0f;
    pVertices[4].y = (iHeight + texdesc.Height) / 2.0f;
    pVertices[4].z = 0.0f;
    pVertices[4].rhw = 1.0f;
    pVertices[4].tu  = 1.0f;
    pVertices[4].tv  = 1.0f;

    pVertices[5].x = (iWidth + texdesc.Width) / 2.0f;
    pVertices[5].y = (iHeight - texdesc.Height) / 2.0f;
    pVertices[5].z = 0.0f;
    pVertices[5].rhw = 1.0f;
    pVertices[5].tu  = 1.0f;
    pVertices[5].tv  = 0.01f;

    pVertices[6].x = iWidth - 0.1f;
    pVertices[6].y = (iHeight + texdesc.Height) / 2.0f;
    pVertices[6].z = 0.0f;
    pVertices[6].rhw = 1.0f;
    pVertices[6].tu  = 1.0f;
    pVertices[6].tv  = 1.0f;

    pVertices[7].x = iWidth - 0.1f;
    pVertices[7].y = (iHeight - texdesc.Height) / 2.0f;
    pVertices[7].z = 0.0f;
    pVertices[7].rhw = 1.0f;
    pVertices[7].tu  = 1.0f;
    pVertices[7].tv  = 0.01f;

    pVertices[0].color = pVertices[1].color = D3DCOLOR_RGBA(m_iAlpha, m_iAlpha, m_iAlpha, 255);
    pVertices[2].color = pVertices[3].color = D3DCOLOR_RGBA(m_iAlpha, m_iAlpha, m_iAlpha, 255);
    pVertices[4].color = pVertices[5].color = D3DCOLOR_RGBA(m_iAlpha, m_iAlpha, m_iAlpha, 255);
    pVertices[6].color = pVertices[7].color = D3DCOLOR_RGBA(m_iAlpha, m_iAlpha, m_iAlpha, 255);
    m_pVBLogo->Unlock();

    COLORVERTEX* pSkyVertices;
    if( FAILED( m_pVBSky->Lock( 0, 0, (BYTE**)&pSkyVertices, 0  ) ) )
        return;
    pSkyVertices[0].x = 0.0f;
    pSkyVertices[0].y = iHeight / 2.0f;
    pSkyVertices[0].z = 0.0f;
    pSkyVertices[0].rhw = 1.0f;

    pSkyVertices[1].x = 0.0f;
    pSkyVertices[1].y = 0.0f;
    pSkyVertices[1].z = 0.0f;
    pSkyVertices[1].rhw = 1.0f;

    pSkyVertices[2].x = iWidth - 0.1f;
    pSkyVertices[2].y = iHeight / 2.0f;
    pSkyVertices[2].z = 0.0f;
    pSkyVertices[2].rhw = 1.0f;

    pSkyVertices[3].x = iWidth - 0.1f;
    pSkyVertices[3].y = 0.0f;
    pSkyVertices[3].z = 0.0f;
    pSkyVertices[3].rhw = 1.0f;

    pSkyVertices[0].color = pSkyVertices[1].color = pSkyVertices[2].color = pSkyVertices[3].color = D3DCOLOR_RGBA(255 * m_iAlpha / 255, 128 * m_iAlpha / 255, 0, 255);
    m_pVBSky->Unlock();

    m_pd3dDevice->LightEnable( 0, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE);
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
    // Some cards do not disable writing to Z when 
    // D3DRS_ZENABLE is FALSE. So do it explicitly
    m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );

    //darken the sky
    m_pd3dDevice->SetTexture(0, NULL);
    m_pd3dDevice->SetVertexShader( D3DFVF_COLORVERTEX);
    m_pd3dDevice->SetStreamSource( 0, m_pVBSky, sizeof(COLORVERTEX) );
    m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2);

    //draw the logo
    m_pd3dDevice->SetTexture(0, m_pLogoTex);
    m_pd3dDevice->SetVertexShader( D3DFVF_TEXTUREVERTEX);
    m_pd3dDevice->SetStreamSource( 0, m_pVBLogo, sizeof(TEXTUREVERTEX) );
    m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 6);

    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE);

    m_pd3dDevice->LightEnable( 0, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE);
}

HRESULT CLogoMenu::InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice )
{
    _ASSERT(pd3dDevice);

    // Keep a local copy of the device
    m_pd3dDevice = pd3dDevice;

    HRSRC hrsrc = FindResource(NULL, MAKEINTRESOURCE(IDR_LOGO), "IMAGE");
    _ASSERT(hrsrc);
    DWORD   dwSize  = SizeofResource(NULL, hrsrc);
    HGLOBAL hglobal = LoadResource(NULL, hrsrc);
    _ASSERT(hglobal);
    LPVOID  pData   = LockResource(hglobal);
    _ASSERT(pData);

    if(FAILED( D3DXCreateTextureFromFileInMemory(m_pd3dDevice, pData, dwSize, &m_pLogoTex) ))
        return E_FAIL;

    return S_OK;
}

HRESULT CLogoMenu::RestoreDeviceObjects()
{
    if( FAILED( m_pd3dDevice->CreateVertexBuffer( sizeof(TEXTUREVERTEX) * 8, D3DUSAGE_WRITEONLY, D3DFVF_TEXTUREVERTEX, D3DPOOL_DEFAULT, &m_pVBLogo ) ) )
        return E_FAIL;

    if( FAILED( m_pd3dDevice->CreateVertexBuffer( sizeof(COLORVERTEX) * 4, D3DUSAGE_WRITEONLY, D3DFVF_COLORVERTEX, D3DPOOL_DEFAULT, &m_pVBSky ) ) )
        return E_FAIL;

    return S_OK;
}

HRESULT CLogoMenu::InvalidateDeviceObjects()
{
    SAFE_RELEASE( m_pVBSky );
    SAFE_RELEASE( m_pVBLogo );
    return S_OK;
}

HRESULT CLogoMenu::DeleteDeviceObjects()
{
    SAFE_RELEASE( m_pVBSky );
    SAFE_RELEASE( m_pVBLogo );
    SAFE_RELEASE( m_pLogoTex );
    m_pd3dDevice = NULL;
    return S_OK;
}