#include "NetworkManager.h"
#include "dxutil.h"
#include <crtdbg.h>

IDirectPlay8Address* CNetworkManager::m_pDeviceAddress = NULL;
IDirectPlay8Server*  CNetworkManager::m_pServer        = NULL;
IDirectPlay8Client*  CNetworkManager::m_pClient        = NULL;
DPNHANDLE            CNetworkManager::m_hEnumAsyncOp   = NULL;
DPNID                CNetworkManager::m_LocalPlayerID  = 0;

map<DPNID, CNetworkManager::PlayerInfo> CNetworkManager::m_Players;

CNetworkManager::CNetworkManager(const GUID& guidapp,
                                 CCourseLoader* pCourseLoader,
                                 CSnakeManager* pSnakeMan,
                                 CItemManager*  pItemMan,
                                 UINT iMaxPlayers) :
    m_GUIDApp(guidapp), m_pCourseLoader(pCourseLoader),
    m_pSnakeManager(pSnakeMan), m_pItemManager(pItemMan),
    m_iMaxPlayers(iMaxPlayers), m_bIsConnected(FALSE),
    m_bGameStarted(FALSE)
{
    m_pwszLocalAddr[0] = '\0';
    if(m_pDeviceAddress)
        return;

    ///////////////////////////////////////////////////////
    // Create the Device Address
    ///////////////////////////////////////////////////////

    // Create our IDirectPlay8Address Device Address
    if(FAILED( CoCreateInstance( CLSID_DirectPlay8Address, NULL,
                                 CLSCTX_INPROC_SERVER,
                                 IID_IDirectPlay8Address,
                                 (LPVOID*) &m_pDeviceAddress ) ))
    {
        m_pDeviceAddress = NULL;
        return;
    }

    // Set the Service Provider for our Device Address
    if(FAILED( m_pDeviceAddress->SetSP(&CLSID_DP8SP_TCPIP ) ))
    {
        SAFE_RELEASE(m_pDeviceAddress);
        return;
    }
}

CNetworkManager::~CNetworkManager(void)
{
    EndConnection();
    SAFE_RELEASE(m_pDeviceAddress);
}

//set the player info for the current player
void CNetworkManager::SetLocalPlayerInfo(const TCHAR* pszName, D3DCOLOR snakecolor)
{
    lstrcpy(m_LocalPlayer.m_pszName, pszName);
    m_LocalPlayer.m_SnakeColor = snakecolor;
}


//DirectPlay callback  handler for the server
HRESULT WINAPI CNetworkManager::ServerCallback(PVOID pvUserContext, DWORD dwMessageType, PVOID pMessage)
{
    CNetworkManager* pNetMan = (CNetworkManager*)pvUserContext;
    switch(dwMessageType)
    {
    case DPN_MSGID_ENUM_HOSTS_QUERY:
        return DPN_OK;
        break;
    case DPN_MSGID_CREATE_PLAYER:
        {
            PDPNMSG_CREATE_PLAYER pMsg = (PDPNMSG_CREATE_PLAYER)pMessage;
            DPN_BUFFER_DESC       dpnBuffer;
            BYTE                  pszSendBuf[MAX_PATH];
            PBYTE                 pCurBufPos;

            //check if this is the server creating its own player
            if(pMsg->pvPlayerContext)
            {
                //copy info into local player's array
                m_LocalPlayerID = pMsg->dpnidPlayer;
                lstrcpy(m_Players[m_LocalPlayerID].m_pszName, pNetMan->m_LocalPlayer.m_pszName);
                m_Players[m_LocalPlayerID].m_SnakeColor = pNetMan->m_LocalPlayer.m_SnakeColor;
                m_Players[m_LocalPlayerID].m_pSnake     = pNetMan->m_pSnakeManager->GetPrimary();
            }
            else
            {
                //send the new player its own player id
                pszSendBuf[0]         = MSG_LOCALID;
                pCurBufPos            = pszSendBuf + 1;
                *((DPNID*)pCurBufPos) = pMsg->dpnidPlayer;
                pCurBufPos += sizeof(DPNID);

                dpnBuffer.dwBufferSize = pCurBufPos - pszSendBuf;
                dpnBuffer.pBufferData  = pszSendBuf;
                m_pServer->SendTo(pMsg->dpnidPlayer, &dpnBuffer, 1, 0, NULL,
                                  NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK);

                //send the course information
                pNetMan->SendSelectedCourse(pMsg->dpnidPlayer);

                //send the item placement information
                pNetMan->SendItemPositions(pMsg->dpnidPlayer);
            }
            break;
        }
    case DPN_MSGID_DESTROY_PLAYER:
        {
            PDPNMSG_DESTROY_PLAYER pMsg = (PDPNMSG_DESTROY_PLAYER)pMessage;

            BYTE  pszSendBuf[1 + sizeof(DPNID)];
            pszSendBuf[0] = MSG_SNAKELEAVE;
            *(DPNID*)(pszSendBuf + 1) = pMsg->dpnidPlayer;

            //forward to all players
            DPN_BUFFER_DESC dpnBuffer;
            dpnBuffer.dwBufferSize = 1 + sizeof(DPNID);
            dpnBuffer.pBufferData  = pszSendBuf;
            m_pServer->SendTo(DPNID_ALL_PLAYERS_GROUP, &dpnBuffer, 1, 0, NULL,
                                NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK);

            //kill the snake that left
            if(pNetMan->m_bGameStarted)
                m_Players[pMsg->dpnidPlayer].m_pSnake->DoAction(CSnake::ACTION_DEATH);
            m_Players.erase(pMsg->dpnidPlayer);
            break;
        }
    case DPN_MSGID_INDICATE_CONNECT:
        {
            PDPNMSG_INDICATE_CONNECT pMsg = (PDPNMSG_INDICATE_CONNECT)pMessage;
            //if the game already started don't let anyone connect
            if(pNetMan->m_bGameStarted)
                return E_FAIL;
            break;
        }
    case DPN_MSGID_INDICATED_CONNECT_ABORTED:
        {
            PDPNMSG_INDICATED_CONNECT_ABORTED pMsg = (PDPNMSG_INDICATED_CONNECT_ABORTED)pMessage;
            break;
        }
    case DPN_MSGID_RECEIVE:
        {
            return pNetMan->ServerMessageProc((PDPNMSG_RECEIVE)pMessage);
            break;
        }
    case DPN_MSGID_RETURN_BUFFER:
        {
            PDPNMSG_RETURN_BUFFER pMsg = (PDPNMSG_RETURN_BUFFER)pMessage;
            break;
        }
    }
    return DPN_OK;
}

//DirectPlay callback  handler for the client
HRESULT WINAPI CNetworkManager::ClientCallback(PVOID pvUserContext, DWORD dwMessageType, PVOID pMessage)
{
    CNetworkManager* pNetMan = (CNetworkManager*)pvUserContext;
    switch(dwMessageType)
    {
    case DPN_MSGID_ENUM_HOSTS_RESPONSE:
        {
            //connect to the first host that responds and cancel the
            //operation
            _ASSERT(m_pClient && m_hEnumAsyncOp && m_pDeviceAddress);

            PDPNMSG_ENUM_HOSTS_RESPONSE pMsg = (PDPNMSG_ENUM_HOSTS_RESPONSE)pMessage;
            if(FAILED( m_pClient->Connect(pMsg->pApplicationDescription, // pdnAppDesc
                                          pMsg->pAddressSender,          // pHostAddr
                                          m_pDeviceAddress,      // pDeviceInfo
                                          NULL,                  // pdnSecurity
                                          NULL,                  // pdnCredentials
                                          NULL, 0,               // pvUserConnectData, Size
                                          NULL,                  // pvAsyncContext
                                          NULL,                  // pvAsyncHandle
                                          DPNCONNECT_SYNC) ))    // dwFlags
            {
                //continue enumeration
                return DPN_OK;
            }

            //stop the enumeration because a connection has been made
            m_pClient->CancelAsyncOperation(m_hEnumAsyncOp, DPNCANCEL_ENUM);
            pNetMan->m_bIsConnected = TRUE;

            //send the name of the player to the server
            BYTE  pszSendBuf[MAX_PATH];
            PBYTE pCurBufPos = pszSendBuf;
            *pCurBufPos = MSG_PLAYERINFO;
            ++pCurBufPos;
            *((D3DCOLOR*)(pCurBufPos)) = pNetMan->m_LocalPlayer.m_SnakeColor;
            pCurBufPos += sizeof(D3DCOLOR);
            lstrcpy((TCHAR*)pCurBufPos, pNetMan->m_LocalPlayer.m_pszName);
            pCurBufPos += (lstrlen(pNetMan->m_LocalPlayer.m_pszName) + 1) * sizeof(TCHAR);

            DPN_BUFFER_DESC dpnBuffer;
            dpnBuffer.dwBufferSize = pCurBufPos - pszSendBuf;
            dpnBuffer.pBufferData  = pszSendBuf;
            m_pClient->Send(&dpnBuffer, 1, 0, NULL, NULL,
                            DPNSEND_SYNC | DPNSEND_NOCOPY | DPNSEND_GUARANTEED);
            break;
        }
    case DPN_MSGID_RECEIVE:
        {
            return pNetMan->ClientMessageProc((PDPNMSG_RECEIVE)pMessage);
            break;
        }
    case DPN_MSGID_TERMINATE_SESSION:
        {
            PDPNMSG_TERMINATE_SESSION pMsg = (PDPNMSG_TERMINATE_SESSION)pMessage;
            pNetMan->m_bIsConnected     = FALSE;
            //Host Connection Lost
            pNetMan->m_bHostConnectLost = TRUE;
            break;
        }
    case DPN_MSGID_RETURN_BUFFER:
        {
            PDPNMSG_RETURN_BUFFER pMsg = (PDPNMSG_RETURN_BUFFER)pMessage;
            break;
        }
    }
    return DPN_OK;
}

//Server Application Specific Message Handler
HRESULT CNetworkManager::ServerMessageProc(PDPNMSG_RECEIVE pMsg)
{
    PBYTE pData      = pMsg->pReceiveData + 1;
    DWORD dwDataSize = pMsg->dwReceiveDataSize - 1;
    switch(pMsg->pReceiveData[0])
    {
    case MSG_BEGINGAME:
        {
            m_Players[pMsg->dpnidSender].m_bReadyToStart = TRUE;

            //wait for all clients to say they are ready before starting game
            map<DPNID, CNetworkManager::PlayerInfo>::iterator it;
            for(it = m_Players.begin(); it != m_Players.end(); ++it)
            {
                if(!it->second.m_bReadyToStart)
                    return DPN_OK;
            }
            BeginGame();
            return DPN_OK;
        }
        break;
    case MSG_PLAYERINFO:
        {
            DPN_BUFFER_DESC       dpnBuffer;
            BYTE                  pszSendBuf[MAX_PATH];
            PBYTE                 pCurBufPos;

            //send list of existing players to source player
            map<DPNID, CNetworkManager::PlayerInfo>::iterator it;
            pszSendBuf[0]          = MSG_PLAYERINFO;
            dpnBuffer.pBufferData  = pszSendBuf;
            for(it = m_Players.begin(); it != m_Players.end(); ++it)
            {
                pCurBufPos = pszSendBuf + 1;
                *((DPNID*)pCurBufPos) = it->first;
                pCurBufPos += sizeof(DPNID);
                *((D3DCOLOR*)(pCurBufPos)) = it->second.m_SnakeColor;
                pCurBufPos += sizeof(D3DCOLOR);
                lstrcpy((TCHAR*)pCurBufPos, it->second.m_pszName);
                pCurBufPos += (lstrlen(it->second.m_pszName) + 1) * sizeof(TCHAR);
        
                dpnBuffer.dwBufferSize = pCurBufPos - pszSendBuf;
                m_pServer->SendTo(pMsg->dpnidSender, &dpnBuffer, 1, 0, NULL,
                                    NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK);
            }

            //create new player and set its snake color and name
            _ASSERT(dwDataSize > sizeof(D3DCOLOR) + 1);
            m_Players[pMsg->dpnidSender].m_SnakeColor = *(D3DCOLOR*)pData;
            pData += sizeof(D3DCOLOR);
            lstrcpy(m_Players[pMsg->dpnidSender].m_pszName, (TCHAR*)pData);

            //send new player to all players
            pCurBufPos = pszSendBuf + 1;
            *((DPNID*)pCurBufPos) = pMsg->dpnidSender;
            pCurBufPos += sizeof(DPNID);
            *((D3DCOLOR*)(pCurBufPos)) = m_Players[pMsg->dpnidSender].m_SnakeColor;
            pCurBufPos += sizeof(D3DCOLOR);
            lstrcpy((TCHAR*)pCurBufPos, m_Players[pMsg->dpnidSender].m_pszName);
            pCurBufPos += (lstrlen(m_Players[pMsg->dpnidSender].m_pszName) + 1) * sizeof(TCHAR);
            m_pServer->SendTo(DPNID_ALL_PLAYERS_GROUP, &dpnBuffer, 1, 0, NULL,
                              NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK);
            return DPN_OK;
        }
        break;
    case MSG_SNAKEACTION:
        _ASSERT(m_Players.find(pMsg->dpnidSender) != m_Players.end());
        _ASSERT(dwDataSize == sizeof(DPNID) + sizeof(DWORD));
        _ASSERT(pMsg->dpnidSender != m_LocalPlayerID);
        m_Players[pMsg->dpnidSender].m_pSnake->DoAction(*(DWORD*)(pData + sizeof(DPNID)));
        break;
    case MSG_SNAKESTATE:
        _ASSERT(m_Players.find(pMsg->dpnidSender) != m_Players.end());
        m_Players[pMsg->dpnidSender].m_pSnake->m_fTargetLength = *(FLOAT*)(pData + sizeof(DPNID));
        m_Players[pMsg->dpnidSender].m_pSnake->m_fSpeed        = *(FLOAT*)(pData + sizeof(DPNID) + sizeof(FLOAT));
        break;
    case MSG_SNAKEDEAD:
        _ASSERT(m_Players.find(pMsg->dpnidSender) != m_Players.end());
        m_Players[pMsg->dpnidSender].m_pSnake->DoAction(CSnake::ACTION_DEATH);
        //use fall through to update the snake synchonization and send message to other players
    case MSG_SYNCSNAKE:
        {
            _ASSERT(m_Players.find(pMsg->dpnidSender) != m_Players.end());
            dwDataSize -= sizeof(DPNID);
            pData      += sizeof(DPNID);

            m_Players[pMsg->dpnidSender].m_pSnake->SetDirection(*(D3DXVECTOR3*)pData);
            pData      += sizeof(D3DXVECTOR3);
            dwDataSize -= sizeof(D3DXVECTOR3);

            //make sure there is enough room
            vector<CSnake::PATH_VERTEX>* pPathVect = &m_Players[pMsg->dpnidSender].m_pSnake->m_PathVect;
            pPathVect->resize(dwDataSize / sizeof(D3DXVECTOR3));
            
            //copy the new values into the array
            vector<CSnake::PATH_VERTEX>::iterator it;
            for(it = pPathVect->begin(); it != pPathVect->end(); ++it)
            {
                it->vPos    = *(D3DXVECTOR3*)pData;
                it->fScale  = 1.0f;
                pData      += sizeof(D3DXVECTOR3);
                dwDataSize -= sizeof(D3DXVECTOR3);
                if(dwDataSize == 0)
                    break;
            }
        }
        break;
    }

    //forward to all players
    DPN_BUFFER_DESC dpnBuffer;
    dpnBuffer.dwBufferSize = pMsg->dwReceiveDataSize;
    dpnBuffer.pBufferData  = pMsg->pReceiveData;
    m_pServer->SendTo(DPNID_ALL_PLAYERS_GROUP, &dpnBuffer, 1, 0, NULL,
                      NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK);

    return DPN_OK;
}

//Client Application Specific Message Handler
HRESULT CNetworkManager::ClientMessageProc(PDPNMSG_RECEIVE pMsg)
{
    DPNID dpnidPlayer = *(DPNID*)(pMsg->pReceiveData + 1);
    PBYTE pData       = pMsg->pReceiveData + 1 + sizeof(DPNID);
    DWORD dwDataSize  = pMsg->dwReceiveDataSize - 1 - sizeof(DPNID);
    switch(pMsg->pReceiveData[0])
    {
    case MSG_BEGINGAME:
        m_bGameStarted = TRUE;
        break;
    case MSG_PLAYERINFO:
        m_Players[dpnidPlayer].m_SnakeColor = *(D3DCOLOR*)pData;
        pData += sizeof(D3DCOLOR);
        lstrcpy(m_Players[dpnidPlayer].m_pszName, (TCHAR*)pData);
        break;
    case MSG_LOCALID:
        m_LocalPlayerID = dpnidPlayer;
        m_Players[dpnidPlayer].m_SnakeColor = m_LocalPlayer.m_SnakeColor;
        lstrcpy(m_Players[dpnidPlayer].m_pszName, m_LocalPlayer.m_pszName);
        m_Players[dpnidPlayer].m_pSnake = m_pSnakeManager->GetPrimary();
        break;
    case MSG_SNAKEACTION:
        m_Players[dpnidPlayer].m_pSnake->DoAction(*(DWORD*)pData);
        break;
    case MSG_SNAKEPOS:
        {
            RegenerateSnakeObjects();
            dwDataSize = pMsg->dwReceiveDataSize - 1;
            pData      = pMsg->pReceiveData + 1;
            D3DXVECTOR3 vPos;
            D3DXVECTOR3 vDir;
            while(dwDataSize > 0)
            {
                dpnidPlayer = *(DPNID*)pData;
                pData += sizeof(DPNID);
                dwDataSize -= sizeof(DPNID);
                vPos = *(D3DXVECTOR3*)pData;
                pData += sizeof(D3DXVECTOR3);
                dwDataSize -= sizeof(D3DXVECTOR3);
                vDir = *(D3DXVECTOR3*)pData;
                pData += sizeof(D3DXVECTOR3);
                dwDataSize -= sizeof(D3DXVECTOR3);
                m_Players[dpnidPlayer].m_pSnake->ResetPosition(vPos, vDir);
            }
            m_RecieveInfo.bSnakePosRecieved = TRUE;
            if(GetIsReadyToStart())
                WaitForStartSignal();
        }
        break;
    case MSG_ITEMPOS:
        {
            UINT iItem = 0;
            pData      = pMsg->pReceiveData + 1;
            dwDataSize = pMsg->dwReceiveDataSize - 1;
            while(dwDataSize > 0)
            {
                CItem* pItem = m_pItemManager->GetItem(iItem);
                _ASSERT(pItem);
                pItem->m_vPos = *(D3DXVECTOR3*)pData;
                pData      += sizeof(D3DXVECTOR3);
                dwDataSize -= sizeof(D3DXVECTOR3);
                ++iItem;
            }
            m_RecieveInfo.bItemPosRecieved = TRUE;
            if(GetIsReadyToStart())
                WaitForStartSignal();
        }
        break;
    case MSG_SNAKESTATE:
        m_Players[dpnidPlayer].m_pSnake->m_fTargetLength = *(FLOAT*)pData;
        m_Players[dpnidPlayer].m_pSnake->m_fSpeed  = *(FLOAT*)(pData + sizeof(FLOAT));
        break;
    case MSG_SNAKEDEAD:
        m_Players[dpnidPlayer].m_pSnake->DoAction(CSnake::ACTION_DEATH);
        //use fall through to update the snake synchonization
    case MSG_SYNCSNAKE:
        if(dpnidPlayer != m_LocalPlayerID)
        {
            m_Players[dpnidPlayer].m_pSnake->SetDirection(*(D3DXVECTOR3*)pData);
            pData      += sizeof(D3DXVECTOR3);
            dwDataSize -= sizeof(D3DXVECTOR3);

            //make sure there is enough room
            vector<CSnake::PATH_VERTEX>* pPathVect = &m_Players[dpnidPlayer].m_pSnake->m_PathVect;
            pPathVect->resize(dwDataSize / sizeof(D3DXVECTOR3));
            
            //copy the new values into the array
            vector<CSnake::PATH_VERTEX>::iterator it;
            for(it = pPathVect->begin(); it != pPathVect->end(); ++it)
            {
                it->vPos   = *(D3DXVECTOR3*)pData;
                it->fScale = 1.0f;
                pData      += sizeof(D3DXVECTOR3);
                dwDataSize -= sizeof(D3DXVECTOR3);
                if(dwDataSize == 0)
                    break;
            }
        }
        break;
    case MSG_SETCOURSE:
        //load the file or disconnect if failed
        if(FAILED( m_pCourseLoader->LoadCourseFile("Levels", (TCHAR*)(pMsg->pReceiveData + 1), m_pItemManager) ))
        {
            m_pClient->Close(0);
            m_bIsConnected = FALSE;
        }
        m_RecieveInfo.bCourseRecieved = TRUE;
        if(GetIsReadyToStart())
            WaitForStartSignal();
        break;
    case MSG_SNAKELEAVE:
        if(m_bGameStarted)
            m_Players[dpnidPlayer].m_pSnake->DoAction(CSnake::ACTION_DEATH);
        m_Players.erase(dpnidPlayer);
        break;
    }

    return DPN_OK;
}

//waits for connections
HRESULT CNetworkManager::StartServer()
{
    //if server already started or no device address then fail
    if(m_pServer || !m_pDeviceAddress)
        return E_FAIL;

    ///////////////////////////////////////////////////////
    // Create the Direct Play Server Object
    ///////////////////////////////////////////////////////

    // Create the IDirectPlay8Server object.
    if(FAILED( CoCreateInstance(CLSID_DirectPlay8Server, NULL,
                                CLSCTX_INPROC_SERVER,
                                IID_IDirectPlay8Server, 
                                (LPVOID*) &m_pServer) ))
    {
        m_pServer = NULL;
        return E_FAIL;
    }

    //initialize the server
    if(FAILED( m_pServer->Initialize(this, &CNetworkManager::ServerCallback, NULL) ))
    {
        SAFE_RELEASE( m_pServer );
        return E_FAIL;
    }

    ///////////////////////////////////////////////////////
    // Begin the Server
    ///////////////////////////////////////////////////////

    // Set up the application description.
    DPN_APPLICATION_DESC dpnAppDesc;
    ZeroMemory(&dpnAppDesc, sizeof(DPN_APPLICATION_DESC));
    dpnAppDesc.dwSize          = sizeof(DPN_APPLICATION_DESC);
    dpnAppDesc.dwFlags         = DPNSESSION_CLIENT_SERVER;  // Flag describing the application
    dpnAppDesc.guidApplication = m_GUIDApp;                 // GUID for the application
    dpnAppDesc.pwszSessionName = L"Nibbles Extreme";        // Session name
    dpnAppDesc.dwMaxPlayers    = m_iMaxPlayers;             // Maximum players for this session

    // We are now ready to host the application.
    if(FAILED( m_pServer->Host( &dpnAppDesc,                // AppDesc
                                &m_pDeviceAddress, 1,       // Device Address
                                NULL, NULL,                 // Reserved
                                &m_LocalPlayerID,           // Player Context
                                0) ))                       // dwFlags
    {
        SAFE_RELEASE( m_pServer );
        return E_FAIL;
    }

    ///////////////////////////////////////////////////////
    // Obtain the IP Address of the Local Machine
    ///////////////////////////////////////////////////////

    m_pwszLocalAddr[0] = '\0';

    //get the local address: use two just in case there is more than one IP address
    IDirectPlay8Address* pAddresses[2];
    DWORD                dwNumAddr = 2;
    if(FAILED( m_pServer->GetLocalHostAddresses(pAddresses, &dwNumAddr, 0) ) || !dwNumAddr)
    {
        m_pServer->Close(0);
        SAFE_RELEASE( m_pServer );
        return E_FAIL;
    }

    //get the IP address of the first IP address in the list
    DWORD dwType = 0;
    DWORD dwSize = 256;
    if(FAILED( pAddresses[0]->GetComponentByName(DPNA_KEY_HOSTNAME, m_pwszLocalAddr, &dwSize, &dwType) ))
    {
        m_pServer->Close(0);
        SAFE_RELEASE( m_pServer );
        for(UINT i = 0; i < dwNumAddr; ++i)
            SAFE_RELEASE( pAddresses[i] );
        return E_FAIL;
    }

    //clean up all of the addresses
    for(UINT i = 0; i < dwNumAddr; ++i)
        SAFE_RELEASE( pAddresses[i] );

    m_bIsConnected = TRUE;

    return S_OK;
}

//connects to server
HRESULT CNetworkManager::StartClient(const WCHAR* pwszHostAddr)
{
    //if client already started or no device address then fail
    if(m_pClient || !m_pDeviceAddress)
        return E_FAIL;

    m_bHostConnectLost = FALSE;

    ///////////////////////////////////////////////////////
    // Create the Direct Play Client Object
    ///////////////////////////////////////////////////////

    // Create the IDirectPlay8Client object.
    if(FAILED( CoCreateInstance(CLSID_DirectPlay8Client, NULL, 
                                CLSCTX_INPROC_SERVER,
                                IID_IDirectPlay8Client, 
                                (LPVOID*) &m_pClient) ))
    {
        m_pClient = NULL;
        return E_FAIL;
    }
   
    // Initialize IDirectPlay8Client object.
    if(FAILED( m_pClient->Initialize(this, &CNetworkManager::ClientCallback, 0) ))
    {
        SAFE_RELEASE( m_pClient );
        return E_FAIL;
    }

    ///////////////////////////////////////////////////////
    // Create the Host Address
    ///////////////////////////////////////////////////////

    // Create our IDirectPlay8Address Host Address
    IDirectPlay8Address*  pHostAddress = NULL;
    if(FAILED( CoCreateInstance( CLSID_DirectPlay8Address, NULL,
                                 CLSCTX_INPROC_SERVER,
                                 IID_IDirectPlay8Address,
                                 (LPVOID*) &pHostAddress ) ))
    {
        SAFE_RELEASE( m_pClient );
        return E_FAIL;
    }

    // Set the Service Provider for our Host Address
    if(FAILED( pHostAddress->SetSP(&CLSID_DP8SP_TCPIP ) ))
    {
        SAFE_RELEASE( m_pClient );
        SAFE_RELEASE( pHostAddress );
        return E_FAIL;
    }

    //add the specified host address to the host address
    if(FAILED( pHostAddress->AddComponent(DPNA_KEY_HOSTNAME, pwszHostAddr,
                                          (wcslen(pwszHostAddr) + 1)*sizeof(WCHAR),
                                          DPNA_DATATYPE_STRING) ))
    {
        SAFE_RELEASE( m_pClient );
        SAFE_RELEASE( pHostAddress );
        return E_FAIL;
    }

    ///////////////////////////////////////////////////////
    // Enumerate the Hosts on the given machine
    ///////////////////////////////////////////////////////

    // Set up the application description.
    DPN_APPLICATION_DESC dpnAppDesc;
    ZeroMemory(&dpnAppDesc, sizeof(DPN_APPLICATION_DESC));
    dpnAppDesc.dwSize          = sizeof(DPN_APPLICATION_DESC);
    dpnAppDesc.guidApplication = m_GUIDApp;  // GUID for the application

    HRESULT hr;
    if(FAILED( hr = m_pClient->EnumHosts(&dpnAppDesc,          // pApplicationDesc
                                    pHostAddress,         // Host Address
                                    m_pDeviceAddress,     // Device Address
                                    NULL, 0,              // pvUserEnumData, size
                                    INFINITE,             // dwEnumCount
                                    0,                    // dwRetryInterval
                                    INFINITE,             // dwTimeOut
                                    NULL,                 // pvUserContext
                                    &m_hEnumAsyncOp,      // pAsyncHandle
                                    0 ) )) // dwFlags
    {
        SAFE_RELEASE( m_pClient );
        SAFE_RELEASE( pHostAddress );
        return E_FAIL;
    }

    SAFE_RELEASE( pHostAddress );
    return S_OK;
}

//sent by server to begin the game
void CNetworkManager::BeginGame()
{
    _ASSERT(m_pServer);
    //send the action to all other players
    BYTE pszSendBuf[1];
    pszSendBuf[0] = MSG_BEGINGAME;

    DPN_BUFFER_DESC dpnBuffer;
    dpnBuffer.dwBufferSize = 1;
    dpnBuffer.pBufferData  = pszSendBuf;
    m_bGameStarted = TRUE;

    //broadcast the start message
    m_pServer->SendTo(DPNID_ALL_PLAYERS_GROUP, &dpnBuffer, 1, 0, NULL,
                    NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK);
}

//ends the game
void CNetworkManager::EndGame()
{
    m_bGameStarted                  = FALSE;

    if(m_pServer)
    {
        map<DPNID, CNetworkManager::PlayerInfo>::iterator it;
        for(it = m_Players.begin(); it != m_Players.end(); ++it)
            it->second.m_bReadyToStart = FALSE;
    }
    else
    {
        m_RecieveInfo.bSnakePosRecieved = FALSE;
    }
}

//sends the snake message to the network
void CNetworkManager::SendSnakeMessage(DWORD dwAction)
{
    _ASSERT(m_Players.find(m_LocalPlayerID) != m_Players.end());

    //send the action
    BYTE  pszSendBuf[MAX_PATH];
    pszSendBuf[0]         = MSG_SNAKEACTION;
    PBYTE pCurBufPos      = pszSendBuf + 1;
    *((DPNID*)pCurBufPos) = m_LocalPlayerID;
    pCurBufPos += sizeof(DPNID);
    *((DWORD*)pCurBufPos) = dwAction;
    pCurBufPos += sizeof(DWORD);

    DPN_BUFFER_DESC dpnBuffer;
    dpnBuffer.dwBufferSize = pCurBufPos - pszSendBuf;
    dpnBuffer.pBufferData  = pszSendBuf;

    if(m_pServer)
    {
        m_pServer->SendTo(DPNID_ALL_PLAYERS_GROUP, &dpnBuffer, 1, 0, NULL,
                          NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK);
    }
    else if(m_pClient)
    {
        m_pClient->Send(&dpnBuffer, 1, 0, NULL, NULL,
                        DPNSEND_SYNC | DPNSEND_NOCOPY | DPNSEND_GUARANTEED);
    }
}

//sends the snake synchonization message
void CNetworkManager::SendSnakeSynch(BOOL bIsDeadMessage)
{
    vector<CSnake::PATH_VERTEX>* pPathVect = &m_Players[m_LocalPlayerID].m_pSnake->m_PathVect;
    if(pPathVect->size() > 100)
        return;

    //place all of the snake points into a send buffer
    BYTE  pszSendBuf[sizeof(D3DXVECTOR3) * 101 + 1 + sizeof(DPNID)];
    pszSendBuf[0]      = bIsDeadMessage ? MSG_SNAKEDEAD : MSG_SYNCSNAKE;
    PBYTE pCurBufPos   = pszSendBuf + 1;
    *((DPNID*)pCurBufPos) = m_LocalPlayerID;
    pCurBufPos += sizeof(DPNID);
    *((D3DXVECTOR3*)pCurBufPos) = m_Players[m_LocalPlayerID].m_pSnake->GetDirection();
    pCurBufPos += sizeof(D3DXVECTOR3);

    vector<CSnake::PATH_VERTEX>::iterator it;
    for(it = pPathVect->begin(); it != pPathVect->end(); ++it)
    {
        *((D3DXVECTOR3*)pCurBufPos) = it->vPos;
        pCurBufPos += sizeof(D3DXVECTOR3);
    }

    DPN_BUFFER_DESC dpnBuffer;
    dpnBuffer.dwBufferSize = pCurBufPos - pszSendBuf;
    dpnBuffer.pBufferData  = pszSendBuf;

    if(m_pServer)
    {
        m_pServer->SendTo(DPNID_ALL_PLAYERS_GROUP, &dpnBuffer, 1, 0, NULL,
                            NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK);
    }
    else if(m_pClient)
    {
        m_pClient->Send(&dpnBuffer, 1, 0, NULL, NULL,
                        DPNSEND_SYNC | DPNSEND_NOCOPY | DPNSEND_GUARANTEED);
    }
}

//sends the snake length
void CNetworkManager::SendSnakeState()
{
    BYTE  pszSendBuf[sizeof(DPNID) + 2 * sizeof(FLOAT) + 1];
    pszSendBuf[0]        = MSG_SNAKESTATE;
    PBYTE pCurBufPos     = pszSendBuf + 1;
    *(DPNID*)pCurBufPos  = m_LocalPlayerID;
    pCurBufPos += sizeof(DPNID);
    *(FLOAT*)pCurBufPos = m_Players[m_LocalPlayerID].m_pSnake->m_fTargetLength;
    pCurBufPos += sizeof(FLOAT);
    *(FLOAT*)pCurBufPos = m_Players[m_LocalPlayerID].m_pSnake->m_fSpeed;
    pCurBufPos += sizeof(FLOAT);

    DPN_BUFFER_DESC dpnBuffer;
    dpnBuffer.dwBufferSize = sizeof(DPNID) + 2 * sizeof(FLOAT) + 1;
    dpnBuffer.pBufferData  = pszSendBuf;

    if(m_pServer)
    {
        m_pServer->SendTo(DPNID_ALL_PLAYERS_GROUP, &dpnBuffer, 1, 0, NULL,
                          NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK);
    }
    else if(m_pClient)
    {
        m_pClient->Send(&dpnBuffer, 1, 0, NULL, NULL,
                        DPNSEND_SYNC | DPNSEND_NOCOPY | DPNSEND_GUARANTEED);
    }
}

//sends the initial positioning information for all the snakes
void CNetworkManager::SendSnakePositions()
{
    _ASSERT(m_pServer);

    DPN_BUFFER_DESC       dpnBuffer;
    BYTE                  pszSendBuf[MAX_PATH];
    PBYTE                 pCurBufPos;

    pszSendBuf[0]         = MSG_SNAKEPOS;
    pCurBufPos = pszSendBuf + 1;
    dpnBuffer.pBufferData = pszSendBuf;

    //send positions for each of the snakes
    map<DPNID, CNetworkManager::PlayerInfo>::iterator it;
    for(it = m_Players.begin(); it != m_Players.end(); ++it)
    {
        *((DPNID*)pCurBufPos) = it->first;
        pCurBufPos += sizeof(DPNID);
        *((D3DXVECTOR3*)(pCurBufPos)) = *(it->second.m_pSnake->GetLeadPoint());
        pCurBufPos += sizeof(D3DXVECTOR3);
        *((D3DXVECTOR3*)(pCurBufPos)) = it->second.m_pSnake->GetDirection();
        pCurBufPos += sizeof(D3DXVECTOR3);
    }
    dpnBuffer.dwBufferSize = pCurBufPos - pszSendBuf;
    m_pServer->SendTo(DPNID_ALL_PLAYERS_GROUP, &dpnBuffer, 1, 0, NULL,
                        NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK); 
}

//sends the initial positioning information for all the items
void CNetworkManager::SendItemPositions(DPNID dpnid)
{
    _ASSERT(m_pServer);

    DPN_BUFFER_DESC       dpnBuffer;
    BYTE                  pszSendBuf[MAX_PATH];
    PBYTE                 pCurBufPos;

    pszSendBuf[0]         = MSG_ITEMPOS;
    pCurBufPos            = pszSendBuf + 1;
    dpnBuffer.pBufferData = pszSendBuf;

    UINT iItem = 0;
    CItem* pItem;
    while(pItem = m_pItemManager->GetItem(iItem))
    {
        _ASSERT(iItem < 64);
        *((D3DXVECTOR3*)(pCurBufPos)) = pItem->m_vPos;
        pCurBufPos += sizeof(D3DXVECTOR3);
        ++iItem;
    }
    dpnBuffer.dwBufferSize = pCurBufPos - pszSendBuf;
    m_pServer->SendTo(dpnid, &dpnBuffer, 1, 0, NULL,
                      NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK); 
}

//sends the selected course
void CNetworkManager::SendSelectedCourse(DPNID dpnid)
{
    _ASSERT(m_pServer);

    BYTE             pszSendBuf[MAX_PATH];
    DPN_BUFFER_DESC  dpnBuffer;

    pszSendBuf[0] = MSG_SETCOURSE;
    lstrcpy((TCHAR*)(pszSendBuf + 1), m_pCourseLoader->GetFileName());
    dpnBuffer.pBufferData = pszSendBuf;
    dpnBuffer.dwBufferSize = (lstrlen(m_pCourseLoader->GetFileName()) + 1) * sizeof(TCHAR) + 1;

    m_pServer->SendTo(dpnid, &dpnBuffer, 1, 0, NULL,
                      NULL, DPNSEND_SYNC | DPNSEND_GUARANTEED | DPNSEND_NOLOOPBACK); 

}

//cancels the client attempt to connect to host
void CNetworkManager::CancelClient()
{
    if(!m_pClient || !m_hEnumAsyncOp)
        return;

    //stop the enumeration
    m_pClient->CancelAsyncOperation(m_hEnumAsyncOp, DPNCANCEL_ENUM);
}

//waits for the start signal BeginGame() the server to be called
void CNetworkManager::WaitForStartSignal()
{
    //broadcast the start message
    if(m_pServer)
    {
        m_Players[m_LocalPlayerID].m_bReadyToStart = TRUE;
    }
    else
    {
        //send the action to all other players
        BYTE pszSendBuf[1];
        pszSendBuf[0] = MSG_BEGINGAME;

        DPN_BUFFER_DESC dpnBuffer;
        dpnBuffer.dwBufferSize = 1;
        dpnBuffer.pBufferData  = pszSendBuf;
        m_bGameStarted = TRUE;
        m_pClient->Send(&dpnBuffer, 1, 0, NULL, NULL,
                        DPNSEND_SYNC | DPNSEND_NOCOPY | DPNSEND_GUARANTEED);
    }
}

//returns whether a connection was established or not
BOOL CNetworkManager::GetIsConnected()
{
    return m_bIsConnected;
}

//returns whether or not the local machine is a game server
BOOL CNetworkManager::GetIsServer()
{
    return (m_pServer != NULL);
}

//returns whether the client is ready to start
BOOL CNetworkManager::GetIsReadyToStart()
{
    return m_RecieveInfo.AllRecieved();
}

//returns whether the game has started yet
BOOL CNetworkManager::GetGameStarted()
{
    return m_bGameStarted;
}

//returns the information of the winning snake or NULL if none
CNetworkManager::PlayerInfo* CNetworkManager::GetWinningSnakeInfo()
{
    map<DPNID, CNetworkManager::PlayerInfo>::iterator it;
    for(it = m_Players.begin(); it != m_Players.end(); ++it)
    {
        if(!it->second.m_pSnake->GetIsDead())
            return &it->second;
    }
    return NULL;
}

//returns the local IP address
WCHAR* CNetworkManager::GetLocalIPAddress()
{
    return m_pwszLocalAddr;
}


//ends all network connections
void CNetworkManager::EndConnection()
{
    if(m_pClient)
    {
        if(m_bIsConnected)
            m_pClient->Close(0);
        SAFE_RELEASE( m_pClient );
    }

    if(m_pServer)
    {
        m_pServer->Close(0);
        SAFE_RELEASE( m_pServer );
    }

    m_Players.clear();
    m_LocalPlayerID = 0;

    m_bIsConnected = FALSE;
    m_RecieveInfo.Reset();

    m_pSnakeManager->ClearSnakes(TRUE);
}

//creates all of the CSnake objects and sets their color
void CNetworkManager::RegenerateSnakeObjects()
{
    //clear all snakes except for primary
    m_pSnakeManager->ClearSnakes(TRUE);

    //add new CSnakes for all snakes except primary
    map<DPNID, CNetworkManager::PlayerInfo>::iterator it;
    for(it = m_Players.begin(); it != m_Players.end(); ++it)
    {
        if(it->first != m_LocalPlayerID)
        {
            //add a new snake to the game
            CSnake* pSnake = m_pSnakeManager->AddSnake();
            _ASSERT(pSnake);
            it->second.m_pSnake = pSnake;
        }
        it->second.m_pSnake->SetColor(it->second.m_SnakeColor);
    }
}
