// NetworkWinMenu.cpp: implementation of the CNetworkWinMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "NetworkWinMenu.h"
#include "customvertex.h"
#include <crtdbg.h>     //for _ASSERT

CAbstractMenu::MenuItem CNetworkWinMenu::m_pNetworkWinMenuItems[] =
{
    {"Play Again",    CAbstractMenu::MENUID_SERVER},
    {"Main Menu",     CAbstractMenu::MENUID_CANCELNET}
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNetworkWinMenu::CNetworkWinMenu(TCHAR* pszName, CNetworkManager* pNetManager) :
    m_iCurSel(0), m_iAlpha(0),
    m_pVB(NULL),  m_pSavedSurface(NULL),
    m_pNetManager(pNetManager)
{
    _ASSERT(pNetManager);

    m_ScreenSize.cx = m_ScreenSize.cy = 0;
    m_pNameFont    = new CD3DFont( _T("Times"), 36, D3DFONT_BOLD );
    m_pMenuFont     = new CD3DFont( _T("Arial"), 20, D3DFONT_BOLD );

    if(pszName)
    {
        lstrcpy(m_pszName, pszName);
        lstrcat(m_pszName, " Wins!!!");
    }
    else
    {
        lstrcpy(m_pszName, "Tie Game");
    }
}

CNetworkWinMenu::~CNetworkWinMenu()
{
    DeleteDeviceObjects();
}

int CNetworkWinMenu::OnKeyDown(WPARAM wParam)
{
    if(wParam == VK_RETURN)
    {
        if(m_pNetManager->GetIsServer())
            return m_pNetworkWinMenuItems[m_iCurSel].m_iMenuID;
        return CAbstractMenu::MENUID_CANCELNET;
    }

    if(m_pNetManager->GetIsServer())
    {
        switch(wParam)
        {
        case VK_DOWN:
            ++m_iCurSel;
            m_iCurSel %= sizeof(m_pNetworkWinMenuItems) / sizeof(CAbstractMenu::MenuItem);
            break;
        case VK_UP:
            --m_iCurSel;
            if(m_iCurSel < 0)
                m_iCurSel = sizeof(m_pNetworkWinMenuItems) / sizeof(CAbstractMenu::MenuItem) - 1;
            break;
        }
    }
    return CAbstractMenu::MENUID_NOTHING;
}


int CNetworkWinMenu::FrameMove()
{
    if(m_iAlpha <= 255)
        ++m_iAlpha;

    if(!m_pNetManager->GetIsServer())
    {
        if(!m_pNetManager->GetIsConnected())
            return CAbstractMenu::MENUID_CANCELNET;
        if(m_pNetManager->GetGameStarted())
            return CAbstractMenu::MENUID_STARTNETGAME;
    }
    return CAbstractMenu::MENUID_NOTHING;
}

//render the title menu
void CNetworkWinMenu::RenderMenu(int iWidth, int iHeight)
{
    _ASSERT(m_pVB && m_pd3dDevice);

    LPDIRECT3DSURFACE8 pBackBuffer;
    if(SUCCEEDED( m_pd3dDevice->GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer) ))
    {
        if(!m_pSavedSurface)
        {
            D3DSURFACE_DESC   desc;
            
            if(FAILED( pBackBuffer->GetDesc(&desc) ) ||
               FAILED( m_pd3dDevice->CreateImageSurface(desc.Width, desc.Height, desc.Format, &m_pSavedSurface ) ) ||
               FAILED( m_pd3dDevice->CopyRects(pBackBuffer, NULL, 0, m_pSavedSurface, NULL) ))
            {
                SAFE_RELEASE( m_pSavedSurface );
            }
        }
        else
        {
            m_pd3dDevice->CopyRects(m_pSavedSurface, NULL, 0, pBackBuffer, NULL);
        }

        SAFE_RELEASE( pBackBuffer );
    }

    //find out whether the coordinates of the vertex buffer must be updated
    if(m_ScreenSize.cx != iWidth ||
       m_ScreenSize.cy != iHeight ||
       m_iAlpha <= 255)
    {
        m_ScreenSize.cx = iWidth;
        m_ScreenSize.cy = iHeight;

        COLORVERTEX* pVertices;
        if( FAILED( m_pVB->Lock( 0, 0, (BYTE**)&pVertices, 0  ) ) )
            return;
        pVertices[0].x = 0.0f;
        pVertices[0].y = (FLOAT)m_ScreenSize.cy - 0.1f;
        pVertices[0].z = 0.0f;
        pVertices[0].rhw = 1.0f;
        pVertices[0].color = D3DCOLOR_RGBA(0, 0, 0, m_iAlpha);

        pVertices[1].x = 0.0f;
        pVertices[1].y = 0.0f;
        pVertices[1].z = 0.0f;
        pVertices[1].rhw = 1.0f;
        pVertices[1].color = D3DCOLOR_RGBA(0, 0, 0, m_iAlpha);

        pVertices[2].x = (FLOAT)m_ScreenSize.cx - 0.1f;
        pVertices[2].y = (FLOAT)m_ScreenSize.cy - 0.1f;
        pVertices[2].z = 0.0f;
        pVertices[2].rhw = 1.0f;
        pVertices[2].color = D3DCOLOR_RGBA(0, 0, 0, m_iAlpha);

        pVertices[3].x = (FLOAT)m_ScreenSize.cx - 0.1f;
        pVertices[3].y = 0.0f;
        pVertices[3].z = 0.0f;
        pVertices[3].rhw = 1.0f;
        pVertices[3].color = D3DCOLOR_RGBA(0, 0, 0, m_iAlpha);
        m_pVB->Unlock();
    }

    m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,   D3DBLEND_SRCALPHA );
    m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND,  D3DBLEND_INVSRCALPHA );

    // Set up the textures
    m_pd3dDevice->SetTexture(0, NULL);

    m_pd3dDevice->LightEnable( 0, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE);
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );
    // Some cards do not disable writing to Z when 
    // D3DRS_ZENABLE is FALSE. So do it explicitly
    m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );
    
    //darken the background
    m_pd3dDevice->SetVertexShader( D3DFVF_COLORVERTEX);
    m_pd3dDevice->SetStreamSource( 0, m_pVB, sizeof(COLORVERTEX) );
    m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2);

    //render the title mesh
    RenderText();

    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE);

    m_pd3dDevice->LightEnable( 0, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE);
    m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
}

HRESULT CNetworkWinMenu::RenderText()
{
    _ASSERT(m_pNameFont && m_pMenuFont);
    
    D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,128,128,128);
    D3DCOLOR fontSelectColor  = D3DCOLOR_ARGB(255,128,0,255);
    FLOAT    fNextLine = 0.0f; 
    SIZE     txtsize;

    //draw the "Game Over" and the score
    m_pNameFont->GetTextExtent(m_pszName, &txtsize );
    m_pNameFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                       (m_ScreenSize.cy - txtsize.cy) / 2.0f - txtsize.cy + fNextLine,
                       D3DCOLOR_ARGB(255,255,0,0), m_pszName );
    fNextLine += txtsize.cy;
    if(!m_pNetManager->GetIsServer())
    {
        m_pMenuFont->GetTextExtent( "Waiting for next game...", &txtsize );
        m_pMenuFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                                (m_ScreenSize.cy - txtsize.cy) / 2.0f + fNextLine,
                                fontColor, "Waiting for next game..." );
        fNextLine += 2 * txtsize.cy;
        m_pMenuFont->GetTextExtent( "[Cancel]", &txtsize );
        m_pMenuFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                                (m_ScreenSize.cy - txtsize.cy) / 2.0f + fNextLine,
                                fontSelectColor, "[Cancel]" );
        return S_OK;
    }

    //draw the menu items
    //create brackets around the selected menu item
    char pszTemp[256];
    pszTemp[0] = '[';
    strcpy(pszTemp + 1, m_pNetworkWinMenuItems[m_iCurSel].m_pszMenuStr);
    strcat(pszTemp, "]");

    int iNumItems = sizeof(m_pNetworkWinMenuItems) / sizeof(CAbstractMenu::MenuItem);
    for(int i = 0; i < iNumItems; ++i)
    {
        if(i == m_iCurSel)
        {
            m_pMenuFont->GetTextExtent( pszTemp, &txtsize );
            m_pMenuFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                                   (m_ScreenSize.cy - txtsize.cy) / 2.0f + fNextLine,
                                   fontSelectColor,
                                   pszTemp );
        }
        else
        {
            m_pMenuFont->GetTextExtent( m_pNetworkWinMenuItems[i].m_pszMenuStr, &txtsize );
            m_pMenuFont->DrawText( (m_ScreenSize.cx - txtsize.cx) / 2.0f,
                                   (m_ScreenSize.cy - txtsize.cy) / 2.0f + fNextLine,
                                   fontColor,
                                   m_pNetworkWinMenuItems[i].m_pszMenuStr );
        }
        fNextLine += txtsize.cy;
    }

    return S_OK;
}


HRESULT CNetworkWinMenu::InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice )
{
    _ASSERT(m_pNameFont && m_pMenuFont && pd3dDevice);

    // Keep a local copy of the device
    m_pd3dDevice = pd3dDevice;

    // Init the font
    m_pNameFont->InitDeviceObjects( m_pd3dDevice );
    m_pMenuFont->InitDeviceObjects( m_pd3dDevice );

    return S_OK;
}

HRESULT CNetworkWinMenu::RestoreDeviceObjects()
{
    _ASSERT(m_pNameFont && m_pMenuFont && m_pd3dDevice);

    m_pNameFont->RestoreDeviceObjects();
    m_pMenuFont->RestoreDeviceObjects();

    if( FAILED( m_pd3dDevice->CreateVertexBuffer( sizeof(COLORVERTEX) * 4, D3DUSAGE_WRITEONLY, D3DFVF_COLORVERTEX, D3DPOOL_DEFAULT, &m_pVB ) ) )
        return E_FAIL;
    
    return S_OK;
}

HRESULT CNetworkWinMenu::InvalidateDeviceObjects()
{
    if(m_pNameFont)
        m_pNameFont->InvalidateDeviceObjects();
    if(m_pMenuFont)
        m_pMenuFont->InvalidateDeviceObjects();
    SAFE_RELEASE( m_pVB );
    SAFE_RELEASE( m_pSavedSurface );
    return S_OK;
}

HRESULT CNetworkWinMenu::DeleteDeviceObjects()
{
    if(m_pNameFont)
        m_pNameFont->DeleteDeviceObjects();
    if(m_pMenuFont)
        m_pMenuFont->DeleteDeviceObjects();
    m_pd3dDevice = NULL;
    SAFE_RELEASE( m_pVB );
    SAFE_RELEASE( m_pSavedSurface );
    SAFE_DELETE( m_pNameFont );
    SAFE_DELETE( m_pMenuFont );
    return S_OK;
}