//-----------------------------------------------------------------------------
// File: Nibbles.cpp
//
// Desc: DirectX window application created by the DirectX AppWizard
//-----------------------------------------------------------------------------
#define STRICT
#define DIRECTINPUT_VERSION 0x0800
#include <windows.h>
#include <basetsd.h>
#include <math.h>
#include <stdio.h>
#include <D3DX8.h>
#include <DXErr8.h>
#include <tchar.h>
#include <dinput.h>
#include <time.h>
#include <crtdbg.h>     //for _ASSERT
#include "D3DApp.h"
#include "D3DFont.h"
#include "D3DFile.h"
#include "D3DUtil.h"
#include "DIUtil.h"
#include "DXUtil.h"
#include "resource.h"
#include "Nibbles.h"
#include "Lofter.h"
#include "GameMath.h"

//the menus
#include "StartMenu.h"
#include "LogoMenu.h"
#include "TitleMenu.h"
#include "GameOverMenu.h"
#include "OptionsMenu.h"
#include "LevelMenu.h"
#include "LevelCompleteMenu.h"
#include "HighScoreMenu.h"
#include "CreditsMenu.h"
#include "NetworkMenu.h"
#include "ServerMenu.h"
#include "ClientMenu.h"
#include "PlayerInfoMenu.h"
#include "NetworkWinMenu.h"

#include "OneSnakeGoal.h"

#define MAX_VIEW_ANGLE_CHANGE 0.02f
#define CAMERA_DISTANCE       50.0f

//-----------------------------------------------------------------------------
// Defines, and constants
//-----------------------------------------------------------------------------
// This GUID must be unique for every game, and the same for 
// every instance of this app.  // {D22D4932-A80E-11D6-BD14-C5592AC6A03D}
// The GUID allows DirectInput to remember input settings
GUID g_guidApp = { 0xd22d4932, 0xa80e, 0x11d6, { 0xbd, 0x14, 0xc5, 0x59, 0x2a, 0xc6, 0xa0, 0x3d } };

// Input semantics used by this app
enum INPUT_SEMANTICS
{
    // Gameplay semantics
    // TODO: change as needed
    INPUT_ROTATE_AXIS_LR=1, INPUT_ROTATE_AXIS_UD,       
    INPUT_ROTATE_LEFT,      INPUT_ROTATE_RIGHT,    
    INPUT_ROTATE_UP,        INPUT_ROTATE_DOWN,
};

// Actions used by this app
DIACTION g_rgGameAction[] =
{
    // TODO: change as needed.  Be sure to delete user map files 
    // (C:\Program Files\DirectX\DirectInput\User Maps\*.ini)
    // after changing this, otherwise settings won't reset and will be read 
    // from the out of date ini files 

    // Device input (joystick, etc.) that is pre-defined by DInput, according
    // to genre type. The genre for this app is space simulators.
    { INPUT_ROTATE_AXIS_LR,  DIAXIS_3DCONTROL_LATERAL,      0, TEXT("Rotate left/right"), },
    { INPUT_ROTATE_AXIS_UD,  DIAXIS_3DCONTROL_MOVE,         0, TEXT("Jump/Dive"), },

    // Keyboard input mappings
    { INPUT_ROTATE_LEFT,     DIKEYBOARD_LEFT,               0, TEXT("Rotate left"), },
    { INPUT_ROTATE_RIGHT,    DIKEYBOARD_RIGHT,              0, TEXT("Rotate right"), },
    { INPUT_ROTATE_UP,       DIKEYBOARD_UP,                 0, TEXT("Jump"), },
    { INPUT_ROTATE_DOWN,     DIKEYBOARD_DOWN,               0, TEXT("Dive"), },
    //{ INPUT_CONFIG_DISPLAY,  DIKEYBOARD_F2,                 DIA_APPFIXED, TEXT("Configure Display"), },    
    //{ INPUT_CONFIG_INPUT,    DIKEYBOARD_F3,                 DIA_APPFIXED, TEXT("Configure Input"), },    
};

#define NUMBER_OF_GAMEACTIONS    (sizeof(g_rgGameAction)/sizeof(DIACTION))

//-----------------------------------------------------------------------------
// Global access to the app (needed for the global WndProc())
//-----------------------------------------------------------------------------
CMyD3DApplication* g_pApp  = NULL;
HINSTANCE          g_hInst = NULL;
TCHAR*             g_pszLevelFileNames[] = 
{
    "level1.txt",
    "level2.txt",
    "level3.txt",
    "level4.txt",
    "level5.txt",
    "level6.txt",
    "level7.txt",
    "level8.txt",
    "level9.txt",
    "level10.txt"
};



//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    CoInitialize(NULL);

    INT iRetVal = 0;

    //place d3dApp in a scope so its destructors get called
    //before COM is uninitialized
    {
        CMyD3DApplication d3dApp;

        g_pApp  = &d3dApp;
        g_hInst = hInst;

        if( FAILED( d3dApp.Create( hInst ) ) )
            return 0;

        iRetVal = d3dApp.Run();
    }

    CoUninitialize();
    return iRetVal;
}




//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
CMyD3DApplication::CMyD3DApplication() :
    m_CourseLoader(DXAPP_KEY),
    m_NetworkManager(g_guidApp, &m_CourseLoader, &m_SnakeManager, &m_ItemManager, MAX_SNAKES)
{
    m_dwCreationWidth           = 500;
    m_dwCreationHeight          = 400;
    m_strWindowTitle            = TEXT( "Nibbles Extreme" );
    m_bUseDepthBuffer           = TRUE;

    // Create a D3D font using d3dfont.cpp
    m_pFont                     = new CD3DFont( _T("Arial"), 12, D3DFONT_BOLD );
    m_bLoadingApp               = TRUE;
    m_pInputDeviceManager       = NULL;
    m_pDIConfigSurface          = NULL;

    ZeroMemory( &m_UserInput, sizeof(m_UserInput) );
    m_fWorldRotX                = 3.1415f / 4.0f;
    m_fWorldRotY                = 0.0f;
    m_fViewAngle                = 3.1415f / 2.0f;
    m_pGoal                     = NULL;
    m_iLevelCompleted           = 0;
    m_iCurLevel                 = 0;
    m_bWaitForGoal              = FALSE;

#ifdef _DEBUG
    m_bStepRun                  = FALSE;
    m_bNextStep                 = FALSE;
    m_bWireframe                = FALSE;
#endif //_DEBUG

    m_pPrimarySnake = m_SnakeManager.AddSnake(TRUE);
    _ASSERT(m_pPrimarySnake);

    //begin with the start menu
    m_pMenu = new CStartMenu();

    ReadWindowSettings();
    ReadGameSettings();
}

CMyD3DApplication::~CMyD3DApplication()
{
}


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::OneTimeSceneInit()
{
    // TODO: perform one time initialization
    srand( (unsigned)time( NULL ) );

    //make sure that the display changes the first time
    m_bDeviceChanged = TRUE;

    // Drawing loading status message until app finishes loading
    SendMessage( m_hWnd, WM_PAINT, 0, 0 );

    // Initialize DirectInput
    InitInput( m_hWnd );

    m_bLoadingApp = FALSE;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: ReadWindowSettings()
// Desc: Read the window settings from the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::ReadWindowSettings()
{
    HKEY hkey;
    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        // TODO: change as needed
        // Read the stored window width/height.  This is just an example,
        // of how to use DXUtil_Read*() functions.
        DXUtil_ReadIntRegKey( hkey, TEXT("Width"), &m_dwCreationWidth, m_dwCreationWidth );
        DXUtil_ReadIntRegKey( hkey, TEXT("Height"), &m_dwCreationHeight, m_dwCreationHeight );
        RegCloseKey( hkey );
    }
}


//-----------------------------------------------------------------------------
// Name: WriteWindowSettings()
// Desc: Write the window settings to the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::WriteWindowSettings()
{
    HKEY hkey;
    DWORD dwType = REG_DWORD;
    DWORD dwLength = sizeof(DWORD);

    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        // Write the window width/height.
        DXUtil_WriteIntRegKey( hkey, TEXT("Width"), m_rcWindowClient.right - m_rcWindowClient.left );
        DXUtil_WriteIntRegKey( hkey, TEXT("Height"), m_rcWindowClient.bottom - m_rcWindowClient.top );
        RegCloseKey( hkey );
    }
}


//-----------------------------------------------------------------------------
// Name: ReadDisplaySettings()
// Desc: Read the display settings from the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::ReadDisplaySettings()
{
    static D3DMULTISAMPLE_TYPE pSampleTypes[] =
        {
            D3DMULTISAMPLE_NONE,
            D3DMULTISAMPLE_2_SAMPLES,
            D3DMULTISAMPLE_3_SAMPLES,
            D3DMULTISAMPLE_4_SAMPLES,
            D3DMULTISAMPLE_5_SAMPLES,
            D3DMULTISAMPLE_6_SAMPLES,
            D3DMULTISAMPLE_7_SAMPLES,
            D3DMULTISAMPLE_8_SAMPLES,
            D3DMULTISAMPLE_9_SAMPLES,
            D3DMULTISAMPLE_10_SAMPLES,
            D3DMULTISAMPLE_11_SAMPLES,
            D3DMULTISAMPLE_12_SAMPLES,
            D3DMULTISAMPLE_13_SAMPLES,
            D3DMULTISAMPLE_14_SAMPLES,
            D3DMULTISAMPLE_15_SAMPLES,
            D3DMULTISAMPLE_16_SAMPLES
        };

    HKEY hkey;
    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        DXUtil_ReadIntRegKey( hkey, TEXT("Adapter"),
            &m_dwAdapter, m_dwAdapter);
        D3DAdapterInfo* pAdapterInfo = &m_Adapters[m_dwAdapter];
        DXUtil_ReadIntRegKey( hkey, TEXT("Device"),
            &pAdapterInfo->dwCurrentDevice,
             pAdapterInfo->dwCurrentDevice);
        D3DDeviceInfo*  pDeviceInfo  = &pAdapterInfo->devices[pAdapterInfo->dwCurrentDevice];
        DXUtil_ReadIntRegKey( hkey, TEXT("Mode"),
            &pDeviceInfo->dwCurrentMode,
             pDeviceInfo->dwCurrentMode );
        DXUtil_ReadBoolRegKey( hkey, TEXT("Windowed"),
            &pDeviceInfo->bWindowed, FALSE);
        ULONG iTemp;
        DXUtil_ReadIntRegKey( hkey, TEXT("MultisampleWindowed"),
            &iTemp, pDeviceInfo->MultiSampleTypeWindowed);
        pDeviceInfo->MultiSampleTypeWindowed = pSampleTypes[iTemp ? iTemp - 1 : 0];
        DXUtil_ReadIntRegKey( hkey, TEXT("MultisampleFullscreen"),
            &iTemp, pDeviceInfo->MultiSampleTypeFullscreen);
        pDeviceInfo->MultiSampleTypeFullscreen = pSampleTypes[iTemp ? iTemp - 1 : 0];

        // Get access to the newly selected adapter, device, and mode
        DWORD dwDevice;
        dwDevice    = m_Adapters[m_dwAdapter].dwCurrentDevice;
        m_bWindowed = m_Adapters[m_dwAdapter].devices[dwDevice].bWindowed;
        RegCloseKey( hkey );
    }
}


//-----------------------------------------------------------------------------
// Name: WriteDisplaySettings()
// Desc: Write the display settings to the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::WriteDisplaySettings()
{
    HKEY hkey;
    DWORD dwType = REG_DWORD;
    DWORD dwLength = sizeof(DWORD);

    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        D3DAdapterInfo* pAdapterInfo = &m_Adapters[m_dwAdapter];
        D3DDeviceInfo*  pDeviceInfo  = &pAdapterInfo->devices[pAdapterInfo->dwCurrentDevice];
        D3DModeInfo*    pModeInfo    = &pDeviceInfo->modes[pDeviceInfo->dwCurrentMode];
        DXUtil_WriteIntRegKey( hkey, TEXT("Adapter"), m_dwAdapter );
        DXUtil_WriteIntRegKey( hkey, TEXT("Device"), pAdapterInfo->dwCurrentDevice );
        DXUtil_WriteIntRegKey( hkey, TEXT("Mode"), pDeviceInfo->dwCurrentMode );
        DXUtil_WriteBoolRegKey( hkey, TEXT("Windowed"), pDeviceInfo->bWindowed );
        DXUtil_WriteIntRegKey( hkey, TEXT("MultisampleWindowed"), pDeviceInfo->MultiSampleTypeWindowed);
        DXUtil_WriteIntRegKey( hkey, TEXT("MultisampleFullscreen"), pDeviceInfo->MultiSampleTypeFullscreen);
        RegCloseKey( hkey );
    }
}

//-----------------------------------------------------------------------------
// Name: ReadGameSettings()
// Desc: Read the game settings from the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::ReadGameSettings()
{
    HKEY hkey;
    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        DXUtil_ReadIntRegKey( hkey, TEXT("LevelCompleted"), &m_iLevelCompleted, m_iLevelCompleted );
        //for showcase purposes make the level completed the maximum so
        //all levels are open
        m_iLevelCompleted = 9;
        RegCloseKey( hkey );
    }
}


//-----------------------------------------------------------------------------
// Name: WriteGameSettings()
// Desc: Write the game settings to the registry
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::WriteGameSettings()
{
    HKEY hkey;
    DWORD dwType = REG_DWORD;
    DWORD dwLength = sizeof(DWORD);

    if( ERROR_SUCCESS == RegCreateKeyEx( HKEY_CURRENT_USER, DXAPP_KEY, 
        0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkey, NULL ) )
    {
        DXUtil_WriteIntRegKey( hkey, TEXT("LevelCompleted"), m_iLevelCompleted );
        RegCloseKey( hkey );
    }
}





//-----------------------------------------------------------------------------
// Name: StaticInputAddDeviceCB()
// Desc: Static callback helper to call into CMyD3DApplication class
//-----------------------------------------------------------------------------
HRESULT CALLBACK CMyD3DApplication::StaticInputAddDeviceCB( 
                                         CInputDeviceManager::DeviceInfo* pDeviceInfo, 
                                         const DIDEVICEINSTANCE* pdidi, 
                                         LPVOID pParam )
{
    CMyD3DApplication* pApp = (CMyD3DApplication*) pParam;
    return pApp->InputAddDeviceCB( pDeviceInfo, pdidi );
}




//-----------------------------------------------------------------------------
// Name: InputAddDeviceCB()
// Desc: Called from CInputDeviceManager whenever a device is added. 
//       Set the dead zone, and creates a new InputDeviceState for each device
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InputAddDeviceCB( CInputDeviceManager::DeviceInfo* pDeviceInfo, 
                                                   const DIDEVICEINSTANCE* pdidi )
{
    // Setup the deadzone 
    DIPROPDWORD dipdw;
    dipdw.diph.dwSize       = sizeof(DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    dipdw.diph.dwObj        = 0;
    dipdw.diph.dwHow        = DIPH_DEVICE;
    dipdw.dwData            = 500;
    pDeviceInfo->pdidDevice->SetProperty( DIPROP_DEADZONE, &dipdw.diph );

    // Create a new InputDeviceState for each device so the 
    // app can record its state 
    InputDeviceState* pNewInputDeviceState = new InputDeviceState;
    ZeroMemory( pNewInputDeviceState, sizeof(InputDeviceState) );
    pDeviceInfo->pParam = (LPVOID) pNewInputDeviceState;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: InitInput()
// Desc: Initialize DirectInput objects
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InitInput( HWND hWnd )
{
    HRESULT hr;

    // Setup action format for the actual gameplay
    ZeroMemory( &m_diafGame, sizeof(DIACTIONFORMAT) );
    m_diafGame.dwSize          = sizeof(DIACTIONFORMAT);
    m_diafGame.dwActionSize    = sizeof(DIACTION);
    m_diafGame.dwDataSize      = NUMBER_OF_GAMEACTIONS * sizeof(DWORD);
    m_diafGame.guidActionMap   = g_guidApp;

    // TODO: change the genre as needed
    m_diafGame.dwGenre         = DIVIRTUAL_CAD_3DCONTROL; 

    m_diafGame.dwNumActions    = NUMBER_OF_GAMEACTIONS;
    m_diafGame.rgoAction       = g_rgGameAction;
    m_diafGame.lAxisMin        = -100;
    m_diafGame.lAxisMax        = 100;
    m_diafGame.dwBufferSize    = 16;
    _tcscpy( m_diafGame.tszActionMap, _T("Nibbles Game") );

    // Create a new input device manager
    m_pInputDeviceManager = new CInputDeviceManager();

    if( FAILED( hr = m_pInputDeviceManager->Create( hWnd, NULL, m_diafGame, 
                                                    StaticInputAddDeviceCB, this ) ) )
        return DXTRACE_ERR_NOMSGBOX( "m_pInputDeviceManager->Create", hr );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the display device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::ConfirmDevice( D3DCAPS8* pCaps, DWORD dwBehavior,
                                          D3DFORMAT Format )
{
    BOOL bCapsAcceptable;

    // TODO: Perform checks to see if these display caps are acceptable.
    bCapsAcceptable = TRUE;

    if( bCapsAcceptable )         
        return S_OK;
    else
        return E_FAIL;
}




//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InitDeviceObjects()
{
    _ASSERT(m_pd3dDevice);

    // TODO: create device objects
    // Init the font
    m_pFont->InitDeviceObjects( m_pd3dDevice );

    HRESULT hr;

    // initialize the course
    if( FAILED( hr = m_CourseLoader.InitDeviceObjects( m_pd3dDevice ) ) )
        return DXTRACE_ERR( "CCourseLoader::InitDeviceObjects", hr );

    // initialize the snake item manager
    if( FAILED( hr = m_ItemManager.InitDeviceObjects( m_pd3dDevice ) ) )
        return DXTRACE_ERR( "CCourseLoader::InitDeviceObjects", hr );

    //initialize the snakes
    m_SnakeManager.InitDeviceObjects(m_pd3dDevice);

    // initialize the menu
    if(m_pMenu && FAILED( hr = m_pMenu->InitDeviceObjects( m_pd3dDevice ) ) )
        return DXTRACE_ERR_NOMSGBOX( "CCourseLoader::InitDeviceObjects", hr );

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Restores scene objects.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RestoreDeviceObjects()
{
    // TODO: setup render states
    HRESULT hr;

    // Setup a material
    D3DMATERIAL8 mtrl;
    D3DUtil_InitMaterial( mtrl, 1.0f, 0.0f, 1.0f );
    m_pd3dDevice->SetMaterial( &mtrl );

    // Set up the textures
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
    m_pd3dDevice->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );

    // Set miscellaneous render states
    m_pd3dDevice->SetRenderState( D3DRS_DITHERENABLE,   FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_SPECULARENABLE, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE,        TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_AMBIENT,        0x000F0F0F );

    // Set the world matrix
    D3DXMATRIX matIdentity;
    D3DXMatrixIdentity( &matIdentity );
    m_pd3dDevice->SetTransform( D3DTS_WORLD,  &matIdentity );

    // Set up our view matrix. A view matrix can be defined given an eye point,
    // a point to lookat, and a direction for which way is up. Here, we set the
    // eye five units back along the z-axis and up three units, look at the
    // origin, and define "up" to be in the y-direction.
    D3DXMATRIX matView;
    D3DXVECTOR3 vEyePt   = D3DXVECTOR3( 0.0f, 0.0f, -250.0f );
    D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUpVec    = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &matView, &vEyePt, &vLookatPt, &vUpVec );
    m_pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

    // Set the projection matrix
    D3DXMATRIX matProj;
    FLOAT fAspect = ((FLOAT)m_d3dsdBackBuffer.Width) / m_d3dsdBackBuffer.Height;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, fAspect, 1, 6 * CAMERA_DISTANCE);
    m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &matProj );

    // Set up lighting states
    D3DLIGHT8 light;
    D3DUtil_InitLight( light, D3DLIGHT_DIRECTIONAL, -1.0f, -1.0f, -2.0f );
    m_pd3dDevice->SetLight( 0, &light );
    m_pd3dDevice->LightEnable( 0, TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, TRUE );

    // Restore the font
    m_pFont->RestoreDeviceObjects();

    // Restore the terrain
    m_CourseLoader.RestoreDeviceObjects();

    // Restore the terrain
    m_ItemManager.RestoreDeviceObjects();

    // Restore the snakes
    m_SnakeManager.RestoreDeviceObjects();

    // Restore the menu
    if(m_pMenu)
        m_pMenu->RestoreDeviceObjects();

    //flat shading
    //if( FAILED( m_pd3dDevice->SetRenderState(D3DRS_SHADEMODE, D3DSHADE_FLAT) ))
    //  return E_FAIL;

    if( !m_bWindowed )
    {
        // Create a surface for configuring DInput devices
        if( FAILED( hr = m_pd3dDevice->CreateImageSurface( 640, 480, 
                                        m_d3dsdBackBuffer.Format, &m_pDIConfigSurface ) ) ) 
            return DXTRACE_ERR_NOMSGBOX( "CreateImageSurface", hr );
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: StaticConfigureInputDevicesCB()
// Desc: Static callback helper to call into CMyD3DApplication class
//-----------------------------------------------------------------------------
BOOL CALLBACK CMyD3DApplication::StaticConfigureInputDevicesCB( 
                                            IUnknown* pUnknown, VOID* pUserData )
{
    CMyD3DApplication* pApp = (CMyD3DApplication*) pUserData;
    return pApp->ConfigureInputDevicesCB( pUnknown );
}




//-----------------------------------------------------------------------------
// Name: ConfigureInputDevicesCB()
// Desc: Callback function for configuring input devices. This function is
//       called in fullscreen modes, so that the input device configuration
//       window can update the screen.
//-----------------------------------------------------------------------------
BOOL CMyD3DApplication::ConfigureInputDevicesCB( IUnknown* pUnknown )
{
    // Get access to the surface
    LPDIRECT3DSURFACE8 pConfigSurface = NULL;
    if( FAILED( pUnknown->QueryInterface( IID_IDirect3DSurface8,
                                          (VOID**)&pConfigSurface ) ) )
        return TRUE;

    // Render the scene, with the config surface blitted on top
    Render();

    RECT  rcSrc;
    SetRect( &rcSrc, 0, 0, 640, 480 );

    POINT ptDst;
    ptDst.x = (m_d3dsdBackBuffer.Width-640)/2;
    ptDst.y = (m_d3dsdBackBuffer.Height-480)/2;

    LPDIRECT3DSURFACE8 pBackBuffer;
    m_pd3dDevice->GetBackBuffer( 0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer );
    m_pd3dDevice->CopyRects( pConfigSurface, &rcSrc, 1, pBackBuffer, &ptDst );
    pBackBuffer->Release();

    // Present the backbuffer contents to the front buffer
    m_pd3dDevice->Present( 0, 0, 0, 0 );

    // Release the surface
    pConfigSurface->Release();

    return TRUE;
}




//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FrameMove()
{
    if(!m_bHasFocus || !m_bActive)
        return S_OK;

    static FLOAT fLastTime = DXUtil_Timer( TIMER_GETAPPTIME );
    FLOAT fTime = DXUtil_Timer( TIMER_GETAPPTIME );

    //if more than 20 frames skipped don't move and
    //make up for lost time if more than 20 frames skipped
    if(fTime - fLastTime >= 0.20f && !m_NetworkManager.GetIsConnected())
    {  
        fLastTime = fTime;
        return S_OK;
    }

    //if there is a menu only move the menu
    if(m_pMenu)
    {
        HRESULT hr;
        while(fTime - fLastTime > 0.01f && m_pMenu)
        {
            fLastTime += 0.01f;
            if(FAILED( hr = MenuProc( m_pMenu->FrameMove() ) ))
                DisplayErrorMsg(hr, 0);
        }
        return S_OK;
    }

    //check for a lost connection to the host
    if(!m_NetworkManager.GetIsConnected() && m_NetworkManager.GetGameStarted())
    {
        HRESULT hr;
        if(FAILED( hr = MenuProc(CAbstractMenu::MENUID_CANCELNET) ))
            DisplayErrorMsg(hr, 0);
        return S_OK;
    }

    //set up the camera view position
    D3DXMATRIX matView;
    D3DXVECTOR3 vLookatPt   = *m_pPrimarySnake->GetLeadPoint();
    D3DXVECTOR3 vUpVec      = D3DXVECTOR3( 0.0f, 0.0f, 1.0f );
    D3DXVECTOR3 vEyePt      = *m_pPrimarySnake->GetLeadPoint();
    vEyePt.z                = CAMERA_DISTANCE;
    vEyePt.x += (float)cos(m_fViewAngle) * -CAMERA_DISTANCE;
    vEyePt.y += (float)sin(m_fViewAngle) * -CAMERA_DISTANCE;
    m_Camera.SetViewParams(vEyePt, vLookatPt, vUpVec);

    if(m_NetworkManager.GetIsConnected() && !m_NetworkManager.GetGameStarted())
    {
        //wait for the start signal
        return S_OK;
    }

    //check for whether the level is finished
    if(m_pGoal && m_pGoal->GoalCompleted(&m_ItemManager, &m_SnakeManager))
    {
        SAFE_DELETE(m_pMenu);
        
        //in networked mode show the network screen again
        if(m_NetworkManager.GetIsConnected())
        {
            m_bWaitForGoal = FALSE;
            m_NetworkManager.EndGame();
            CNetworkManager::PlayerInfo* pInfo = m_NetworkManager.GetWinningSnakeInfo();
            m_pMenu = new CNetworkWinMenu(pInfo ? pInfo->m_pszName : NULL, &m_NetworkManager);
        }
        else
        {
            //update the high score
            if(m_pPrimarySnake->m_iScore > m_CourseLoader.GetHighScore())
                m_pMenu = new CHighScoreMenu(m_pPrimarySnake->m_iScore, &m_CourseLoader);
            else
                m_pMenu = new CLevelCompleteMenu(m_pPrimarySnake->m_iScore);
        }

        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        return S_OK;
    }

    //find the current angle of the snake
    FLOAT curangle = CGameMath::CalculateCameraAngle(m_pPrimarySnake->GetDirection());

    //loop until time is caught up
    UINT iNumLoops = 0;
    while(fTime - fLastTime > 0.01f)
    {
        fLastTime += 0.01f;
       
        static UINT iNumSnakeMove = 0;
        ++iNumSnakeMove;
        
        //handle input every 5 snake moves
        if((iNumSnakeMove % 5) == 0)
        {
            //update the user input
            UpdateInput( &m_UserInput );

            DWORD dwAction = CSnake::ACTION_NONE;
            if(m_UserInput.fAxisRotateLR < 0.0f)
                dwAction = CSnake::ACTION_TURNLEFT;
            else if(m_UserInput.fAxisRotateLR > 0.0f)
                dwAction = CSnake::ACTION_TURNRIGHT;

            if(m_UserInput.fAxisRotateUD < 0.0f)
                dwAction = CSnake::ACTION_DIVE;
            else if(m_UserInput.fAxisRotateUD > 0.0f)
                dwAction = CSnake::ACTION_JUMP;

            if(dwAction != CSnake::ACTION_NONE)
            {
                if(m_NetworkManager.GetIsConnected())
                    m_NetworkManager.SendSnakeMessage(dwAction);
                m_pPrimarySnake->DoAction(dwAction);
            }
        }

        //send snake synchronization message every 25 snake moves
        if((iNumSnakeMove % 50) == 0 && m_NetworkManager.GetIsConnected())
            m_NetworkManager.SendSnakeSynch();

        //adjust eyepoint in increments
        if(fabs(curangle - m_fViewAngle) > MAX_VIEW_ANGLE_CHANGE)
        {
            if(fabs(curangle - m_fViewAngle) < 3.1415f)
                m_fViewAngle += 10.0f * m_pPrimarySnake->GetSpeed() * ((curangle - m_fViewAngle) > 0) ? MAX_VIEW_ANGLE_CHANGE : -MAX_VIEW_ANGLE_CHANGE;
            else
                m_fViewAngle += 10.0f * m_pPrimarySnake->GetSpeed() * ((curangle - m_fViewAngle) > 0) ? -MAX_VIEW_ANGLE_CHANGE : MAX_VIEW_ANGLE_CHANGE;

            if(m_fViewAngle > 2.0f * 3.1415f)
                m_fViewAngle -= 2.0f * 3.1415f;
            if(m_fViewAngle < 0.0f)
                m_fViewAngle += 2.0f * 3.1415f;
        }
        else
        {
            m_fViewAngle = curangle;
        }
        
        //move the course
        m_CourseLoader.FrameMove();

        //move the snake items
        m_ItemManager.FrameMove(&m_CourseLoader, &m_SnakeManager);

#ifdef _DEBUG
        if(!m_bStepRun || m_bNextStep)
        {
#endif //_DEBUG
            //only move when the network manager is not controlling it
            m_SnakeManager.MoveSnakes(m_CourseLoader.GetTerrain());
#ifdef _DEBUG
            m_bNextStep = FALSE;
        }
#endif //_DEBUG

        //check for collisions with walls and items
        if(m_SnakeManager.CheckCollisions(&m_CourseLoader, &m_ItemManager, m_NetworkManager.GetIsConnected()) &&
           m_NetworkManager.GetIsConnected())
        {
            m_NetworkManager.SendSnakeState();
        }

        //in network setting exit because lag will just build up and
        //responsiveness will decrease
        if(m_NetworkManager.GetIsConnected())
        {
            ++iNumLoops;
            if(iNumLoops > 3)
            {
                fLastTime = fTime;
                break;
            }
        }
    }

    //check whether the snake is dead
    if(m_pPrimarySnake->GetIsDead())
    {
        if(!m_NetworkManager.GetIsConnected())
        {
            //create the game over menu
            SAFE_DELETE(m_pMenu);
            m_pMenu = new CGameOverMenu(m_pPrimarySnake->m_iScore);
            m_pMenu->InitDeviceObjects(m_pd3dDevice);
            m_pMenu->RestoreDeviceObjects();
        }
        else if(!m_bWaitForGoal)
        {
            //send the dead snake message to server once then wait for the goal
            m_NetworkManager.SendSnakeSynch(TRUE);
            m_bWaitForGoal = TRUE;
        }
        return S_OK;
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: UpdateInput()
// Desc: Update the user input.  Called once per frame 
//-----------------------------------------------------------------------------
void CMyD3DApplication::UpdateInput( UserInput* pUserInput )
{
    if( NULL == m_pInputDeviceManager )
        return;

    // Get access to the list of semantically-mapped input devices
    CInputDeviceManager::DeviceInfo* pDeviceInfos;
    DWORD dwNumDevices;
    m_pInputDeviceManager->GetDevices( &pDeviceInfos, &dwNumDevices );

    // Loop through all devices and check game input
    for( DWORD i=0; i<dwNumDevices; i++ )
    {
        DIDEVICEOBJECTDATA rgdod[10];
        DWORD   dwItems = 10;
        HRESULT hr;
        LPDIRECTINPUTDEVICE8 pdidDevice = pDeviceInfos[i].pdidDevice;
        InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;

        hr = pdidDevice->Acquire();
        hr = pdidDevice->Poll();
        hr = pdidDevice->GetDeviceData( sizeof(DIDEVICEOBJECTDATA),
                                        rgdod, &dwItems, 0 );
        if( FAILED(hr) )
            continue;

        // Get the sematics codes for the game menu
        for( DWORD j=0; j<dwItems; j++ )
        {
            BOOL  bButtonState = (rgdod[j].dwData==0x80) ? TRUE : FALSE;
            FLOAT fButtonState = (rgdod[j].dwData==0x80) ? 1.0f : 0.0f;
            FLOAT fAxisState   = (FLOAT)((int)rgdod[j].dwData)/100.0f;

            switch( rgdod[j].uAppData )
            {
                // TODO: Handle semantics for the game 

                // Handle relative axis data
                case INPUT_ROTATE_AXIS_LR: 
                    pInputDeviceState->fAxisRotateLR = -fAxisState;
                    break;
                case INPUT_ROTATE_AXIS_UD:
                    pInputDeviceState->fAxisRotateUD = -fAxisState;
                    break;

                // Handle buttons separately so the button state data
                // doesn't overwrite the axis state data, and handle
                // each button separately so they don't overwrite each other
                case INPUT_ROTATE_LEFT:  pInputDeviceState->bButtonRotateLeft  = bButtonState; break;
                case INPUT_ROTATE_RIGHT: pInputDeviceState->bButtonRotateRight = bButtonState; break;
                case INPUT_ROTATE_UP:    pInputDeviceState->bButtonRotateUp    = bButtonState; break;
                case INPUT_ROTATE_DOWN:  pInputDeviceState->bButtonRotateDown  = bButtonState; break;

                // Handle one-shot buttons
                //case INPUT_CONFIG_INPUT:   if( bButtonState ) pUserInput->bDoConfigureInput = TRUE; break;
                //case INPUT_CONFIG_DISPLAY: if( bButtonState ) pUserInput->bDoConfigureDisplay = TRUE; break;
            }
        }
    }

    // TODO: change process code as needed

    // Process user input and store result into pUserInput struct
    pUserInput->fAxisRotateLR = 0.0f;
    pUserInput->fAxisRotateUD = 0.0f;

    // Concatinate the data from all the DirectInput devices
    for( i=0; i<dwNumDevices; i++ )
    {
        InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;

        // Use the axis data that is furthest from zero
        if( fabs(pInputDeviceState->fAxisRotateLR) > fabs(pUserInput->fAxisRotateLR) )
            pUserInput->fAxisRotateLR = pInputDeviceState->fAxisRotateLR;

        if( fabs(pInputDeviceState->fAxisRotateUD) > fabs(pUserInput->fAxisRotateUD) )
            pUserInput->fAxisRotateUD = pInputDeviceState->fAxisRotateUD;

        // Process the button data 
        if( pInputDeviceState->bButtonRotateLeft )
            pUserInput->fAxisRotateLR = 1.0f;
        else if( pInputDeviceState->bButtonRotateRight )
            pUserInput->fAxisRotateLR = -1.0f;

        if( pInputDeviceState->bButtonRotateUp )
            pUserInput->fAxisRotateUD = 1.0f;
        else if( pInputDeviceState->bButtonRotateDown )
            pUserInput->fAxisRotateUD = -1.0f;

    }
}




//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::Render()
{
    // Begin the scene
    if( FAILED( m_pd3dDevice->BeginScene() ) )
        return S_OK;

    ///////////////////////////////////////////////////////
    // Render the menu
    ///////////////////////////////////////////////////////
    if(m_pMenu)
    {
        m_pMenu->RenderMenu(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);
        m_pd3dDevice->EndScene();
        return S_OK;
    }

    // Clear the viewport
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 0x00000000, 1.0f, 0L );

    D3DXMATRIX matView = m_Camera.GetViewMatrix();
    m_pd3dDevice->SetTransform( D3DTS_VIEW, &matView);

    ///////////////////////////////////////////////////////
    // Render the sky
    ///////////////////////////////////////////////////////
    m_CourseLoader.RenderSky();

    ///////////////////////////////////////////////////////
    // Render the snakes
    ///////////////////////////////////////////////////////
#ifdef _DEBUG
    if(m_bWireframe)
        m_pd3dDevice->SetRenderState( D3DRS_FILLMODE,   D3DFILL_WIREFRAME );
#endif //_DEBUG

    m_SnakeManager.Render();

#ifdef _DEBUG
    if(m_bWireframe)
        m_pd3dDevice->SetRenderState( D3DRS_FILLMODE,   D3DFILL_SOLID );
#endif //_DEBUG

    ///////////////////////////////////////////////////////
    // Render the snake items
    ///////////////////////////////////////////////////////
    m_ItemManager.Render();

    ///////////////////////////////////////////////////////
    // Render the course
    ///////////////////////////////////////////////////////
    m_CourseLoader.Render(&m_Camera, m_pPrimarySnake);

    ///////////////////////////////////////////////////////
    // Render the onscreen text
    ///////////////////////////////////////////////////////
    RenderText();

    // End the scene.
    m_pd3dDevice->EndScene();

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Renders stats and help text to the scene.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::RenderText()
{
    D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,255,255,0);
    D3DCOLOR fontWarningColor = D3DCOLOR_ARGB(255,0,255,255);
    TCHAR szMsg[MAX_PATH]     = TEXT("");

    lstrcpy(szMsg, "Score: ");
    itoa(m_pPrimarySnake->m_iScore, szMsg + lstrlen(szMsg), 10);
    m_pFont->DrawText( 2, 2, fontColor, szMsg );

#ifdef _DEBUG
    // Output display stats
    FLOAT fNextLine = 55.0f; 

    lstrcpy( szMsg, m_strDeviceStats );
    fNextLine -= 20.0f;
    m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

    lstrcpy( szMsg, m_strFrameStats );
    fNextLine -= 20.0f;
    m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

    // Output statistics & help
    fNextLine = (FLOAT) m_d3dsdBackBuffer.Height; 

    sprintf( szMsg, TEXT("Left/Right Axis: %0.2f Up/Down Axis: %0.2f "), 
              m_UserInput.fAxisRotateLR, m_UserInput.fAxisRotateUD );
    fNextLine -= 20.0f; m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

    lstrcpy( szMsg, TEXT("Use arrow keys or joystick to rotate object") );
    fNextLine -= 20.0f; m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

    lstrcpy( szMsg, TEXT("Press 'F3' to configure input") );
    fNextLine -= 20.0f; m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );

    lstrcpy( szMsg, TEXT("Press 'F2' to configure display") );
    fNextLine -= 20.0f; m_pFont->DrawText( 2, fNextLine, fontColor, szMsg );
#endif //_DEBUG

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: Pause()
// Desc: Called in to toggle the pause state of the app.
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::Pause( BOOL bPause )
{
    // Get access to the list of semantically-mapped input devices
    // to zero the state of all InputDeviceState structs.  This is needed
    // because when using DISCL_FOREGROUND, the action mapper will not 
    // record actions when the focus switches, for example if a dialog appears.
    // This causes a problem when a button held down when loosing focus, and let
    // go when the focus is lost.  The app will not record that the button 
    // has been let go, so the state will be incorrect when focus returns.  
    // To fix this either use DISCL_BACKGROUND or zero the state when 
    // loosing focus.
    CInputDeviceManager::DeviceInfo* pDeviceInfos;
    DWORD dwNumDevices;
    m_pInputDeviceManager->GetDevices( &pDeviceInfos, &dwNumDevices );

    for( DWORD i=0; i<dwNumDevices; i++ )
    {
        InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;
        ZeroMemory( pInputDeviceState, sizeof(InputDeviceState) );
    }

    CD3DApplication::Pause( bPause );
}




//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Overrrides the main WndProc, so the sample can do custom message
//       handling (e.g. processing mouse, keyboard, or menu commands).
//-----------------------------------------------------------------------------
LRESULT CMyD3DApplication::MsgProc( HWND hWnd, UINT msg, WPARAM wParam,
                                    LPARAM lParam )
{
    switch( msg )
    {
        case WM_PAINT:
        {
            if( m_bLoadingApp )
            {
                // Draw on the window tell the user that the app is loading
                // TODO: change as needed
                HDC hDC = GetDC( hWnd );
                RECT rct;
                GetClientRect( hWnd, &rct );
                HBRUSH hbr = CreateSolidBrush(RGB(0,0,0));
                FillRect(hDC, &rct, hbr);
                DeleteObject(hbr);
                ReleaseDC( hWnd, hDC );
            }
            break;
        }

        case WM_ACTIVATE:
        {
            //restore the window when it is activated
            m_UserInput.fAxisRotateUD = m_UserInput.fAxisRotateLR = 0.0f;
            ShowWindow(hWnd, SW_RESTORE);
            break;
        }

        /*case WM_COMMAND:
        {
            switch( LOWORD(wParam) )
            {
                case IDM_CONFIGINPUT:
                    m_UserInput.bDoConfigureInput = TRUE;
                    break;

                case IDM_CHANGEDEVICE:
                    m_UserInput.bDoConfigureDisplay = TRUE;
                    return 0; // Don't hand off to parent
            }
            break;
        }*/

        case WM_KEYDOWN:
        {
            //send the keystrokes to the current menu and
            //close the menu if necessary
            if(m_pMenu)
            {
                HRESULT hr;
                if(FAILED( hr = MenuProc(m_pMenu->OnKeyDown(wParam)) ))
                    DisplayErrorMsg(hr, 0);
                return DefWindowProc( hWnd, msg, wParam, lParam );;
            }
#ifdef _DEBUG
            else
            {
                if(m_bStepRun)
                    m_bNextStep = TRUE;

                switch(wParam)
                {
                case VK_SPACE:
                    m_bStepRun = !m_bStepRun;
                    break;
                case 'W':   
                    m_bWireframe = !m_bWireframe;
                    break;
                }
            }
#endif _DEBUG
            break;
        }
        case WM_CHAR:
        {
            //send the keystrokes to the current menu and
            //close the menu if necessary
            if(m_pMenu)
            {
                HRESULT hr;
                if(FAILED( hr = MenuProc(m_pMenu->OnChar(wParam)) ))
                    DisplayErrorMsg(hr, 0);
                return DefWindowProc( hWnd, msg, wParam, lParam );
            }
            break;
        }
        case WM_GETMINMAXINFO:
        {
            MINMAXINFO* lpmmi = (LPMINMAXINFO) lParam;
            _ASSERT(lpmmi);
            lpmmi->ptMinTrackSize.x = 500;
            lpmmi->ptMinTrackSize.y = 400;
            return 0;
            break;
        }
    }

    return CD3DApplication::MsgProc( hWnd, msg, wParam, lParam );
}




//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::InvalidateDeviceObjects()
{
    _ASSERT(m_pFont);

    // TODO: Cleanup any objects created in RestoreDeviceObjects()
    m_pd3dDevice->SetStreamSource( 0, NULL, sizeof(CUSTOMVERTEX) );

    m_pFont->InvalidateDeviceObjects();
    SAFE_RELEASE( m_pDIConfigSurface );
    m_CourseLoader.InvalidateDeviceObjects();
    m_ItemManager.InvalidateDeviceObjects();

    if(m_pMenu)
        m_pMenu->InvalidateDeviceObjects();

    m_SnakeManager.InvalidateDeviceObjects();
    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.  
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::DeleteDeviceObjects()
{
    // TODO: Cleanup any objects created in InitDeviceObjects()
    m_pFont->DeleteDeviceObjects();
    m_CourseLoader.DeleteDeviceObjects();
    m_ItemManager.DeleteDeviceObjects();
    if(m_pMenu)
        m_pMenu->DeleteDeviceObjects();
    SAFE_DELETE( m_pGoal );

    m_SnakeManager.DeleteDeviceObjects();

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
HRESULT CMyD3DApplication::FinalCleanup()
{
    // TODO: Perform any final cleanup needed
    // Cleanup DirectInput
    CleanupDirectInput();

    // Cleanup D3D font
    SAFE_DELETE( m_pFont );
    SAFE_DELETE( m_pMenu );

    m_SnakeManager.ClearSnakes();

    // Write the settings to the registry
    WriteWindowSettings();
    WriteDisplaySettings();
    WriteGameSettings();

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: CleanupDirectInput()
// Desc: Cleanup DirectInput 
//-----------------------------------------------------------------------------
VOID CMyD3DApplication::CleanupDirectInput()
{
    if( NULL == m_pInputDeviceManager )
        return;

    // Get access to the list of semantically-mapped input devices
    // to delete all InputDeviceState structs
    CInputDeviceManager::DeviceInfo* pDeviceInfos;
    DWORD dwNumDevices;
    m_pInputDeviceManager->GetDevices( &pDeviceInfos, &dwNumDevices );

    for( DWORD i=0; i<dwNumDevices; i++ )
    {
        InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;
        SAFE_DELETE( pInputDeviceState );
        pDeviceInfos[i].pParam = NULL;
    }

    // Cleanup DirectX input objects
    SAFE_DELETE( m_pInputDeviceManager );

}

//performs an action based on the menuid return value
HRESULT CMyD3DApplication::MenuProc(int menuid)
{
    switch(menuid)
    {
    case CAbstractMenu::MENUID_EXITAPP:
        SendMessage( m_hWnd, WM_CLOSE, 0, 0 );
        break;
    case CAbstractMenu::MENUID_NOTHING:
        break;

    ////////////////////////////////////////////////////////////////////
    //start menu commands
    ////////////////////////////////////////////////////////////////////
    case CAbstractMenu::MENUID_START:
        SAFE_DELETE(m_pMenu);
        // Read settings from registry
        ReadDisplaySettings();
        UpdateNewDisplaySettings();
        m_pMenu = new CLogoMenu();
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;
    case CAbstractMenu::MENUID_OPTIONS:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new COptionsMenu();
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;

    ////////////////////////////////////////////////////////////////////
    //options menu commands
    ////////////////////////////////////////////////////////////////////
    case CAbstractMenu::MENUID_SNAKEDETAIL:
        break;
    case CAbstractMenu::MENUID_DISPLAY:
        // Prompt the user to select a new device or mode
        if( m_bActive && m_bReady )
        {
            Pause(TRUE);
            ReadDisplaySettings();
            if( SUCCEEDED( UserSelectNewDevice() ) )
            {
                WriteDisplaySettings();
            }
            Pause(FALSE);
        }
        break;
    case CAbstractMenu::MENUID_SOUND:
        break;
    case CAbstractMenu::MENUID_CONTROLS:
        {
            Pause( TRUE );

            // Get access to the list of semantically-mapped input devices
            // to delete all InputDeviceState structs before calling ConfigureDevices()
            CInputDeviceManager::DeviceInfo* pDeviceInfos;
            DWORD dwNumDevices;
            m_pInputDeviceManager->GetDevices( &pDeviceInfos, &dwNumDevices );

            for( DWORD i=0; i<dwNumDevices; i++ )
            {
                InputDeviceState* pInputDeviceState = (InputDeviceState*) pDeviceInfos[i].pParam;
                SAFE_DELETE( pInputDeviceState );
                pDeviceInfos[i].pParam = NULL;
            }

            // Configure the devices (with edit capability)
            if( m_bWindowed )
                m_pInputDeviceManager->ConfigureDevices( m_hWnd, NULL, NULL, DICD_EDIT, NULL );
            else
                m_pInputDeviceManager->ConfigureDevices( m_hWnd,
                                                         m_pDIConfigSurface,
                                                         (VOID*)StaticConfigureInputDevicesCB,
                                                         DICD_EDIT, (LPVOID) this );

            Pause( FALSE );
        }
        break;
    case CAbstractMenu::MENUID_STARTMENU:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CStartMenu();
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;

    ////////////////////////////////////////////////////////////////////
    //title menu commands
    ////////////////////////////////////////////////////////////////////
    case CAbstractMenu::MENUID_SINGLE:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CLevelMenu(&m_CourseLoader, g_pszLevelFileNames,
                                 m_iLevelCompleted + 1, &m_ItemManager,
                                 &m_pGoal, FALSE);
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;
    case CAbstractMenu::MENUID_NETWORK:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CNetworkMenu();
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;
    case CAbstractMenu::MENUID_CREDITS:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CCreditsMenu();
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;

    ////////////////////////////////////////////////////////////////////
    //game over menu commands
    ////////////////////////////////////////////////////////////////////
    case CAbstractMenu::MENUID_RESTART:
        m_CourseLoader.Reset();
        m_CourseLoader.ResetSnakePositions(&m_SnakeManager);
        m_SnakeManager.Reset();
        m_ItemManager.Reset(&m_CourseLoader);
        m_fViewAngle = CGameMath::CalculateCameraAngle(m_pPrimarySnake->GetDirection());
        SAFE_DELETE(m_pMenu);
        break;
    case CAbstractMenu::MENUID_MAINMENU:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CTitleMenu();
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;

    ////////////////////////////////////////////////////////////////////
    //level menu commands
    ////////////////////////////////////////////////////////////////////
    case CAbstractMenu::MENUID_PLAYLEVEL:
        m_iCurLevel = ((CLevelMenu*)m_pMenu)->GetCurLevel();
        m_CourseLoader.Reset();
        m_CourseLoader.ResetSnakePositions(&m_SnakeManager);
        m_SnakeManager.Reset();
        m_ItemManager.Reset(&m_CourseLoader);
        m_fViewAngle = CGameMath::CalculateCameraAngle(m_pPrimarySnake->GetDirection());
        SAFE_DELETE(m_pMenu);
        break;
    case CAbstractMenu::MENUID_SERVER:
        SAFE_DELETE(m_pMenu);
        m_CourseLoader.Reset();
        m_ItemManager.Reset(&m_CourseLoader);
        m_pMenu = new CServerMenu(&m_NetworkManager);
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;

    ////////////////////////////////////////////////////////////////////
    //level complete commands
    ////////////////////////////////////////////////////////////////////
    case CAbstractMenu::MENUID_NEXTLEVEL:
        SAFE_DELETE(m_pMenu);
        if(m_iCurLevel < sizeof(g_pszLevelFileNames) / sizeof(TCHAR*) - 1)
            ++m_iCurLevel;
        if(m_iLevelCompleted < m_iCurLevel)
            m_iLevelCompleted = m_iCurLevel;
        m_pMenu = new CLevelMenu(&m_CourseLoader, g_pszLevelFileNames,
                                 m_iLevelCompleted + 1, &m_ItemManager,
                                 &m_pGoal, FALSE);
        ((CLevelMenu*)m_pMenu)->SetCurLevel(m_iCurLevel);
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;
    case CAbstractMenu::MENUID_RETURNTOMAIN:
        if(m_iCurLevel < sizeof(g_pszLevelFileNames) / sizeof(TCHAR*) - 1)
            ++m_iCurLevel;
        if(m_iLevelCompleted < m_iCurLevel)
            m_iLevelCompleted = m_iCurLevel;
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CTitleMenu();
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;

    ////////////////////////////////////////////////////////////////////
    //network menu commands
    ////////////////////////////////////////////////////////////////////
    case CAbstractMenu::MENUID_HOST:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CPlayerInfoMenu(&m_NetworkManager, CPlayerInfoMenu::SERVER_PLAYER);
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;
    case CAbstractMenu::MENUID_CLIENT:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CPlayerInfoMenu(&m_NetworkManager, CPlayerInfoMenu::CLIENT_PLAYER);
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;

    ////////////////////////////////////////////////////////////////////
    //player info menu commands
    ////////////////////////////////////////////////////////////////////
    case CAbstractMenu::MENUID_CONNECTSERVER:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CLevelMenu(&m_CourseLoader, g_pszLevelFileNames,
                            m_iLevelCompleted, &m_ItemManager,
                            &m_pGoal, TRUE);
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;
    case CAbstractMenu::MENUID_CONNECTCLIENT:
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CClientMenu(&m_NetworkManager);
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;

    ////////////////////////////////////////////////////////////////////
    //client/server menu commands
    ////////////////////////////////////////////////////////////////////
    case CAbstractMenu::MENUID_CANCELNET:
        m_NetworkManager.EndConnection();
        SAFE_DELETE(m_pMenu);
        m_pMenu = new CTitleMenu();
        m_pMenu->InitDeviceObjects(m_pd3dDevice);
        m_pMenu->RestoreDeviceObjects();
        break;
    case CAbstractMenu::MENUID_STARTNETGAME:
        //course should have already been loaded

        //reset the game
        m_CourseLoader.Reset();
        m_SnakeManager.Reset();

        SAFE_DELETE(m_pGoal);
        m_pGoal = new COneSnakeGoal();

        //send information about the course
        if(m_NetworkManager.GetIsServer())
        {
            m_NetworkManager.RegenerateSnakeObjects();
            m_CourseLoader.ResetSnakePositions(&m_SnakeManager, TRUE);
            m_SnakeManager.Reset();
            m_NetworkManager.SendSnakePositions();
            m_NetworkManager.WaitForStartSignal();
        }
        else
        {
            m_CourseLoader.InitDeviceObjects(m_pd3dDevice);
            m_CourseLoader.RestoreDeviceObjects();
            m_ItemManager.Reset();
        }

        m_fViewAngle   = CGameMath::CalculateCameraAngle(m_pPrimarySnake->GetDirection());
        m_bWaitForGoal = FALSE;
        SAFE_DELETE(m_pMenu);
        break;
    }

    return S_OK;
}
