//-----------------------------------------------------------------------------
// File: Nibbles.h
//
// Desc: Header file Nibbles sample app
//-----------------------------------------------------------------------------
#pragma once

#include "CourseLoader.h"
#include "ItemManager.h"
#include "NetworkManager.h"
#include "SnakeManager.h"
#include "AbstractGoal.h"
#include "AbstractMenu.h"

//-----------------------------------------------------------------------------
// Defines, and constants
//-----------------------------------------------------------------------------
// TODO: change "DirectX AppWizard Apps" to your name or the company name
#define DXAPP_KEY        TEXT("Software\\HorizonFire\\NibblesExtreme")
#define MAX_SNAKES       4

// DirectInput action mapper reports events only when buttons/axis change
// so we need to remember the present state of relevant axis/buttons for 
// each DirectInput device.  The CInputDeviceManager will store a 
// pointer for each device that points to this struct
struct InputDeviceState
{
    // TODO: change as needed
    FLOAT fAxisRotateLR;
    BOOL  bButtonRotateLeft;
    BOOL  bButtonRotateRight;

    FLOAT fAxisRotateUD;
    BOOL  bButtonRotateUp;
    BOOL  bButtonRotateDown;
};


// Struct to store the current input state
struct UserInput
{
    // TODO: change as needed
    FLOAT fAxisRotateUD;
    FLOAT fAxisRotateLR;
};




//-----------------------------------------------------------------------------
// Name: class CMyD3DApplication
// Desc: Application class. The base class (CD3DApplication) provides the 
//       generic functionality needed in all Direct3D samples. CMyD3DApplication 
//       adds functionality specific to this sample program.
//-----------------------------------------------------------------------------
class CMyD3DApplication : public CD3DApplication
{
public:
    CMyD3DApplication();
    ~CMyD3DApplication();

protected:
    LRESULT MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
    HRESULT InputAddDeviceCB( CInputDeviceManager::DeviceInfo* pDeviceInfo, const DIDEVICEINSTANCE* pdidi );
    BOOL    ConfigureInputDevicesCB( IUnknown* pUnknown );

    static HRESULT CALLBACK StaticInputAddDeviceCB( CInputDeviceManager::DeviceInfo* pDeviceInfo, const DIDEVICEINSTANCE* pdidi, LPVOID pParam );   
    static BOOL CALLBACK StaticConfigureInputDevicesCB( IUnknown* pUnknown, VOID* pUserData );
    
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects();
    HRESULT RestoreDeviceObjects();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();
    HRESULT Render();
    HRESULT FrameMove();
    HRESULT FinalCleanup();
    HRESULT ConfirmDevice( D3DCAPS8*, DWORD, D3DFORMAT );
    VOID    Pause( BOOL bPause );

    HRESULT RenderText();

    HRESULT InitInput( HWND hWnd );
    void    UpdateInput( UserInput* pUserInput );
    void    CleanupDirectInput();

    //performs an action based on the menuid return value
    HRESULT MenuProc(int menuid);

    VOID    ReadWindowSettings();
    VOID    WriteWindowSettings();

    VOID    ReadDisplaySettings();
    VOID    WriteDisplaySettings();

    VOID    ReadGameSettings();
    VOID    WriteGameSettings();

private:
    BOOL                    m_bLoadingApp;          // TRUE, if the app is loading
    CD3DCamera              m_Camera;

    CInputDeviceManager*    m_pInputDeviceManager;  // DirectInput device manager
    DIACTIONFORMAT          m_diafGame;             // Action format for game play
    LPDIRECT3DSURFACE8      m_pDIConfigSurface;     // Surface for config'ing DInput devices
    UserInput               m_UserInput;            // Struct for storing user input 

    FLOAT                   m_fWorldRotX;           // World rotation state X-axis
    FLOAT                   m_fWorldRotY;           // World rotation state Y-axis
    FLOAT                   m_fViewAngle;

    CD3DFont*               m_pFont;                // Font for drawing text
    CCourseLoader           m_CourseLoader;         // loads the course
    CItemManager            m_ItemManager;          // Manages the items in the course
    CNetworkManager         m_NetworkManager;       // Manages network capabilities
    CSnakeManager           m_SnakeManager;         // Manages the snakes

    CAbstractGoal*          m_pGoal;                // the goal for the game
    CSnake*                 m_pPrimarySnake;        //the primary snake that the camera follows
    CAbstractMenu*          m_pMenu;                // current menu being displayed

    ULONG                   m_iLevelCompleted;      //highest level completed so far
    ULONG                   m_iCurLevel;            //the current level that is selected
    BOOL                    m_bWaitForGoal;         //flag that says to wait for the goal to be accomplished

#ifdef _DEBUG
    BOOL                    m_bStepRun;
    BOOL                    m_bNextStep;
    BOOL                    m_bWireframe;
#endif //_DEBUG
};