// OptionsMenu.cpp: implementation of the COptionsMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "OptionsMenu.h"
#include <crtdbg.h>     //for _ASSERT

CAbstractMenu::MenuItem COptionsMenu::m_pOptionsMenuItems[] =
{
    //{"Snake Detail",        CAbstractMenu::MENUID_SNAKEDETAIL},
    {"Display",             CAbstractMenu::MENUID_DISPLAY},
    //{"Sound",               CAbstractMenu::MENUID_SOUND},
    {"Controls",            CAbstractMenu::MENUID_CONTROLS},
    {"Main Menu",           CAbstractMenu::MENUID_STARTMENU}
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

COptionsMenu::COptionsMenu() : m_iCurSel(0)
{
    m_pFont      = new CD3DFont( _T("Arial"), 20, D3DFONT_BOLD );
}

COptionsMenu::~COptionsMenu()
{
    DeleteDeviceObjects();
}

int COptionsMenu::OnKeyDown(WPARAM wParam)
{
    switch(wParam)
    {
    case VK_RETURN:
        return m_pOptionsMenuItems[m_iCurSel].m_iMenuID;
        break;
    case VK_DOWN:
        ++m_iCurSel;
        m_iCurSel %= sizeof(m_pOptionsMenuItems) / sizeof(CAbstractMenu::MenuItem);
        break;
    case VK_UP:
        --m_iCurSel;
        if(m_iCurSel < 0)
            m_iCurSel = sizeof(m_pOptionsMenuItems) / sizeof(CAbstractMenu::MenuItem) - 1;
        break;
    }
    return CAbstractMenu::MENUID_NOTHING;
}

int COptionsMenu::FrameMove()
{
    return CAbstractMenu::MENUID_NOTHING;
}

//render the title menu
void COptionsMenu::RenderMenu(int iWidth, int iHeight)
{
    _ASSERT(m_pd3dDevice);
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 0x00000000, 1.0f, 0L );
    RenderText(iWidth, iHeight);
}


HRESULT COptionsMenu::RenderText(int iWidth, int iHeight)
{
    _ASSERT(m_pFont);
    
    D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,128,128,128);
    D3DCOLOR fontSelectColor  = D3DCOLOR_ARGB(255,128,0,255);

    // Output display stats
    FLOAT fNextLine = -40.0f; 
    SIZE txtsize;

    //place brackets around the selected item
    char pszTemp[256];
    pszTemp[0] = '[';
    strcpy(pszTemp + 1, m_pOptionsMenuItems[m_iCurSel].m_pszMenuStr);
    strcat(pszTemp, "]");

    int iNumItems = sizeof(m_pOptionsMenuItems) / sizeof(CAbstractMenu::MenuItem);
    for(int i = 0; i < iNumItems; ++i)
    {
        if(i == m_iCurSel)
        {
            m_pFont->GetTextExtent( pszTemp, &txtsize );
            m_pFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                               (iHeight - txtsize.cy) / 2.0f - txtsize.cy / 2.0f + fNextLine,
                               fontSelectColor,
                               pszTemp );
        }
        else
        {
            m_pFont->GetTextExtent( m_pOptionsMenuItems[i].m_pszMenuStr, &txtsize );
            m_pFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                               (iHeight - txtsize.cy) / 2.0f - txtsize.cy / 2.0f + fNextLine,
                               fontColor,
                               m_pOptionsMenuItems[i].m_pszMenuStr );
        }
        fNextLine += txtsize.cy;
    }

    return S_OK;
}


HRESULT COptionsMenu::InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice )
{
    _ASSERT(m_pFont && pd3dDevice);

    // Keep a local copy of the device
    m_pd3dDevice = pd3dDevice;

    // Init the font
    m_pFont->InitDeviceObjects( m_pd3dDevice );

    return S_OK;
}

HRESULT COptionsMenu::RestoreDeviceObjects()
{
    _ASSERT(m_pd3dDevice && m_pFont);
    m_pFont->RestoreDeviceObjects();
    return S_OK;
}

HRESULT COptionsMenu::InvalidateDeviceObjects()
{
    _ASSERT(m_pFont);
    m_pFont->InvalidateDeviceObjects();
    return S_OK;
}

HRESULT COptionsMenu::DeleteDeviceObjects()
{
    if(m_pFont)
        m_pFont->DeleteDeviceObjects();
 
    m_pd3dDevice = NULL;
    SAFE_DELETE( m_pFont );
    return S_OK;
}