// PlayerInfoMenu.cpp: implementation of the CPlayerInfoMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "PlayerInfoMenu.h"
#include "customvertex.h"
#include <crtdbg.h>     //for _ASSERT

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPlayerInfoMenu::CPlayerInfoMenu(CNetworkManager* pNetMan, UINT iPlayerType) :
    m_pNetworkManager(pNetMan), m_iPlayerType(iPlayerType),
    m_iCurField(FIELD_NAME), m_pVB(NULL), m_bUpdateColor(TRUE),
    m_pTexture(NULL)
{
    _ASSERT(m_pNetworkManager);

    lstrcpy(m_pszName, "<None>");
    m_pInputFont    = new CD3DFont( _T("Arial"), 18, D3DFONT_BOLD );
    m_pMenuFont     = new CD3DFont( _T("Arial"), 20, D3DFONT_BOLD );

    m_iRed    = 0;
    m_iGreen  = 255;
    m_iBlue   = 0;

    m_fBotMargin = 10.0f;
}

CPlayerInfoMenu::~CPlayerInfoMenu()
{
    DeleteDeviceObjects();
}

int CPlayerInfoMenu::OnKeyDown(WPARAM wParam)
{
    _ASSERT(m_pNetworkManager);

    //deal with field specific inputs
    switch(m_iCurField)
    {
    case FIELD_NAME:
        switch(wParam)
        {
        case VK_RETURN:
            m_iCurField = FIELD_RED;
            break;
        case 8: //backspace
        case VK_DELETE:
            {
                UINT index = lstrlen(m_pszName);
                if(index > 0)
                    m_pszName[index - 1] = '\0';
                break;
            }
        }
        break;
    case FIELD_RED:
        if(wParam == VK_LEFT)
        {
            if(m_iRed != 0)
                m_iRed -= 5;
            m_bUpdateColor = TRUE;
        }
        else if(wParam == VK_RIGHT)
        {
            if(m_iRed != 255)
                m_iRed += 5;
            m_bUpdateColor = TRUE;
        }
        break;
    case FIELD_GREEN:
        if(wParam == VK_LEFT)
        {
            if(m_iGreen != 0)
                m_iGreen -= 5;
            m_bUpdateColor = TRUE;
        }
        else if(wParam == VK_RIGHT)
        {
            if(m_iGreen != 255)
                m_iGreen += 5;
            m_bUpdateColor = TRUE;
        }
        break;
    case FIELD_BLUE:
        if(wParam == VK_LEFT)
        {
            if(m_iBlue != 0)
                m_iBlue -= 5;
            m_bUpdateColor = TRUE;
        }
        else if(wParam == VK_RIGHT)
        {
            if(m_iBlue != 255)
                m_iBlue += 5;
            m_bUpdateColor = TRUE;
        }
        break;
    case FIELD_CONTINUE:
        if(wParam == VK_RETURN)
        {
            m_pNetworkManager->SetLocalPlayerInfo(m_pszName, D3DCOLOR_RGBA(m_iRed, m_iGreen, m_iBlue, 255));
            switch(m_iPlayerType)
            {
            case SINGLE_PLAYER:
                return CAbstractMenu::MENUID_SINGLE;
                break;
            case SERVER_PLAYER:
                return CAbstractMenu::MENUID_CONNECTSERVER;
                break;
            case CLIENT_PLAYER:
                return CAbstractMenu::MENUID_CONNECTCLIENT;
                break;
            }
        }
        break;
    case FIELD_CANCEL:
        if(wParam == VK_RETURN)
            return CAbstractMenu::MENUID_MAINMENU;
        break;
    }

    //handle field switching
    switch(wParam)
    {
    case VK_DOWN:
        ++m_iCurField;
        if(m_iCurField > FIELD_CANCEL)
            m_iCurField = FIELD_CANCEL;
        break;
    case VK_UP:
        if(m_iCurField != 0)
            --m_iCurField;
        break;
    }
    return CAbstractMenu::MENUID_NOTHING;
}

int CPlayerInfoMenu::OnChar(WPARAM wParam)
{
    if(m_iCurField == FIELD_NAME)
    {

        UINT index = lstrlen(m_pszName);
        if(((wParam >= 'A' && wParam <= 'Z') || (wParam >= 'a' && wParam <= 'z')) && index < 256)
        {
            if(lstrcmp(m_pszName, "<None>") == 0)
            {
                m_pszName[0] = '\0';
                index = 0;
            }
            //append character onto end
            m_pszName[index]     = wParam;
            m_pszName[index + 1] = '\0';
        }
    }
    return CAbstractMenu::MENUID_NOTHING;
}

int CPlayerInfoMenu::FrameMove()
{
    return CAbstractMenu::MENUID_NOTHING;
}

//render the title menu
void CPlayerInfoMenu::RenderMenu(int iWidth, int iHeight)
{
    _ASSERT(m_pd3dDevice);
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 0x00000000, 1.0f, 0L );

    //render the title mesh
    RenderText(iWidth, iHeight);

    if(m_bUpdateColor)
    {
        TEXTUREVERTEX* pVertices;
        if( FAILED( m_pVB->Lock( 0, 0, (BYTE**)&pVertices, 0  ) ) )
            return;
        pVertices[0].x = iWidth / 2 + 10.0f;
        pVertices[0].y = 10.0f;
        pVertices[0].z = 0.0f;
        pVertices[0].rhw = 1.0f;
        pVertices[0].color = D3DCOLOR_RGBA(m_iRed, m_iGreen, m_iBlue, 255);
        pVertices[0].tu = 0.0f;
        pVertices[0].tv = 0.0f; 

        pVertices[1].x = iWidth - 10.0f;
        pVertices[1].y = 10.0f;
        pVertices[1].z = 0.0f;
        pVertices[1].rhw = 1.0f;
        pVertices[1].color = D3DCOLOR_RGBA(m_iRed, m_iGreen, m_iBlue, 255);
        pVertices[1].tu = 2.0f;
        pVertices[1].tv = 0.0f; 

        pVertices[2].x = iWidth / 2 + 10.0f;
        pVertices[2].y = iHeight - m_fBotMargin;
        pVertices[2].z = 0.0f;
        pVertices[2].rhw = 1.0f;
        pVertices[2].color = D3DCOLOR_RGBA(m_iRed, m_iGreen, m_iBlue, 255);
        pVertices[2].tu = 2.0f;
        pVertices[2].tv = 3.0f; 

        pVertices[3].x = iWidth - 10.0f;
        pVertices[3].y = iHeight - m_fBotMargin;
        pVertices[3].z = 0.0f;
        pVertices[3].rhw = 1.0f;
        pVertices[3].color = D3DCOLOR_RGBA(m_iRed, m_iGreen, m_iBlue, 255);
        pVertices[3].tu = 0.0f;
        pVertices[3].tv = 0.0f; 

        m_pVB->Unlock();
        m_bUpdateColor = FALSE;
    }

    m_pd3dDevice->SetTexture( 0, m_pTexture);
    m_pd3dDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
    m_pd3dDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
    m_pd3dDevice->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);

    //render the color square
    m_pd3dDevice->SetVertexShader( D3DFVF_TEXTUREVERTEX);
    m_pd3dDevice->SetStreamSource( 0, m_pVB, sizeof(TEXTUREVERTEX) );
    m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2);
}

HRESULT CPlayerInfoMenu::RenderText(int iWidth, int iHeight)
{
    _ASSERT(m_pInputFont && m_pMenuFont);
    
    D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,128,128,128);
    D3DCOLOR fontSelectColor  = D3DCOLOR_ARGB(255,128,0,255);
    TCHAR    szMsg[MAX_PATH]     = TEXT(""); 
    SIZE     txtsize;

    //draw player name
    FLOAT    fNextLine  = 50.0f;
    m_pInputFont->GetTextExtent( "Player Name:", &txtsize );
    m_pInputFont->DrawText(10.0f, fNextLine,
                           (m_iCurField == FIELD_NAME) ? fontSelectColor : fontColor, "Player Name:");
    fNextLine += txtsize.cy;
    m_pInputFont->GetTextExtent( m_pszName, &txtsize );
    m_pInputFont->DrawText(iWidth / 4.0f - txtsize.cx / 2.0f, fNextLine, D3DCOLOR_ARGB(255,255,255,0), m_pszName);
    fNextLine += 2 * txtsize.cy;

    m_pInputFont->GetTextExtent( "Snake Color:", &txtsize );
    m_pInputFont->DrawText(10.0f, fNextLine, fontColor, "Snake Color:");
    fNextLine += txtsize.cy;

    //draw red component
    char buf[50];
    m_pInputFont->GetTextExtent( "Green: ", &txtsize );
    m_pInputFont->DrawText(30.0f, fNextLine,
                           (m_iCurField == FIELD_RED) ? fontSelectColor : fontColor, "Red:");
    lstrcpy(szMsg, itoa(m_iRed, buf, 10));
    m_pInputFont->DrawText(40.0f + txtsize.cx, fNextLine, D3DCOLOR_ARGB(255,255,0,0), szMsg);
    fNextLine += txtsize.cy;

    //draw green component
    m_pInputFont->DrawText(30.0f, fNextLine,
                           (m_iCurField == FIELD_GREEN) ? fontSelectColor : fontColor, "Green:");
    lstrcpy(szMsg, itoa(m_iGreen, buf, 10));
    m_pInputFont->DrawText(40.0f + txtsize.cx, fNextLine, D3DCOLOR_ARGB(255,0,255,0), szMsg);
    fNextLine += txtsize.cy;

    //draw blue component
    m_pInputFont->DrawText(30.0f, fNextLine,
                           (m_iCurField == FIELD_BLUE) ? fontSelectColor : fontColor, "Blue:");
    lstrcpy(szMsg, itoa(m_iBlue, buf, 10));
    m_pInputFont->DrawText(40.0f + txtsize.cx, fNextLine, D3DCOLOR_ARGB(255,0,0,255), szMsg);
    fNextLine += txtsize.cy;

    //draw other menu options
    fNextLine = 0;
    if(m_iCurField == FIELD_CONTINUE)
    {
        m_pInputFont->GetTextExtent( "[Continue]", &txtsize );
        m_pMenuFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                            iHeight - 3 * txtsize.cy + fNextLine,
                            fontSelectColor, "[Continue]" );
    }
    else
    {
        m_pInputFont->GetTextExtent( "Continue", &txtsize );
        m_pMenuFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                            iHeight - 3 * txtsize.cy + fNextLine,
                            fontColor, "Continue" );
    }
    fNextLine += txtsize.cy;

    if(m_iCurField == FIELD_CANCEL)
    {
        m_pInputFont->GetTextExtent( "[Cancel]", &txtsize );
        m_pMenuFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                            iHeight - 3 * txtsize.cy + fNextLine,
                            fontSelectColor, "[Cancel]" );
    }
    else
    {
        m_pInputFont->GetTextExtent( "Cancel", &txtsize );
        m_pMenuFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                            iHeight - 3 * txtsize.cy + fNextLine,
                            fontColor, "Cancel" );
    }

    m_fBotMargin = 3 * txtsize.cy + 10.0f;
    return S_OK;
}


HRESULT CPlayerInfoMenu::InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice )
{
    _ASSERT(m_pInputFont && m_pMenuFont && pd3dDevice);

    // Keep a local copy of the device
    m_pd3dDevice = pd3dDevice;

    // Init the font
    m_pInputFont->InitDeviceObjects( m_pd3dDevice );
    m_pMenuFont->InitDeviceObjects( m_pd3dDevice );

    // Create a new texture for the font
    if( FAILED( D3DXCreateTextureFromFile( m_pd3dDevice, "Core\\snake.jpg", &m_pTexture ) ) )
        return E_FAIL;

    return S_OK;
}

HRESULT CPlayerInfoMenu::RestoreDeviceObjects()
{
    _ASSERT(m_pInputFont && m_pMenuFont && m_pd3dDevice);

    m_pInputFont->RestoreDeviceObjects();
    m_pMenuFont->RestoreDeviceObjects();

    if( FAILED( m_pd3dDevice->CreateVertexBuffer(sizeof(TEXTUREVERTEX) * 4,
                  D3DUSAGE_WRITEONLY, D3DFVF_TEXTUREVERTEX, D3DPOOL_DEFAULT, &m_pVB ) ) )
        return E_FAIL;

    m_bUpdateColor = TRUE;
    return S_OK;
}

HRESULT CPlayerInfoMenu::InvalidateDeviceObjects()
{
    _ASSERT(m_pInputFont && m_pMenuFont);
    SAFE_RELEASE( m_pVB );
    m_pInputFont->InvalidateDeviceObjects();
    m_pMenuFont->InvalidateDeviceObjects();
    return S_OK;
}

HRESULT CPlayerInfoMenu::DeleteDeviceObjects()
{
    if(m_pInputFont)
        m_pInputFont->DeleteDeviceObjects();
    if(m_pMenuFont)
        m_pMenuFont->DeleteDeviceObjects();
    m_pd3dDevice = NULL;
    SAFE_RELEASE( m_pVB );
    SAFE_DELETE( m_pInputFont );
    SAFE_DELETE( m_pMenuFont );
    SAFE_RELEASE( m_pTexture );
    return S_OK;
}