// PlayerInfoMenu.h: interface for the CPlayerInfoMenu class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "AbstractMenu.h"
#include "NetworkManager.h"
#include "d3dfile.h"
#include "d3dfont.h"
#include "dxutil.h"

class CPlayerInfoMenu : public CAbstractMenu
{
public:
    //different variations for different types of players
    enum
    {
        SINGLE_PLAYER = 0,
        SERVER_PLAYER = 1,
        CLIENT_PLAYER = 2
    };

	CPlayerInfoMenu(CNetworkManager* pNetMan, UINT iPlayerType = SINGLE_PLAYER);
	virtual ~CPlayerInfoMenu();

    // CAbstractMenu functions
    virtual int OnKeyDown(WPARAM wParam);
    virtual int OnChar(WPARAM wParam);
    virtual int FrameMove();

	// Device Initialization and Rendering Functions
    virtual void RenderMenu(int iWidth, int iHeight);
    virtual HRESULT InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice );
    virtual HRESULT RestoreDeviceObjects();
    virtual HRESULT InvalidateDeviceObjects();
    virtual HRESULT DeleteDeviceObjects();

private:
    //Dialog fields
    enum
    {
        FIELD_NAME     = 0,
        FIELD_RED      = 1,
        FIELD_GREEN    = 2,
        FIELD_BLUE     = 3,
        FIELD_CONTINUE = 4,
        FIELD_CANCEL   = 5
    };

    HRESULT RenderText(int iWidth, int iHeight);
    
    CD3DFont*               m_pInputFont;
    CD3DFont*               m_pMenuFont;     // Font for drawing text
    TCHAR                   m_pszName[256];
    CNetworkManager*        m_pNetworkManager;
    LPDIRECT3DTEXTURE8      m_pTexture;      // texture for the snake body
    LPDIRECT3DVERTEXBUFFER8 m_pVB;           // vertex buffer for color square
    BOOL                    m_bUpdateColor;

    FLOAT                   m_fBotMargin;

    UINT                    m_iPlayerType;
    UINT                    m_iCurField;

    UINT                    m_iRed;
    UINT                    m_iGreen;
    UINT                    m_iBlue;
};