// RegionBehavior.cpp: implementation of the CRegionBehavior class.
//
//////////////////////////////////////////////////////////////////////

#include "RegionBehavior.h"
#include "dxutil.h"
#include <crtdbg.h>     //for _ASSERT

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRegionBehavior::CRegionBehavior() : m_pRegion(NULL), m_iRegionSize(0), m_hRegion(NULL)
{
}

CRegionBehavior::~CRegionBehavior()
{
    SAFE_DELETE_ARRAY(m_pRegion);
    DeleteObject(m_hRegion);
}

CAbstractItemBehavior* CRegionBehavior::Clone(const char* pszIdentifier, ifstream& infile)
{
    _ASSERT(pszIdentifier);

    if(strcmp(pszIdentifier, "REGION") == 0)
    {
        CRegionBehavior* pNewItem = new CRegionBehavior();
        pNewItem->ReadBoundsFromFile(infile);
        return pNewItem;
    }
    return NULL;
}

BOOL CRegionBehavior::CheckInBounds(CSnake* pSnake, CItem* pItem)
{
    _ASSERT(pSnake && pItem);

    //generate a region in order to place the items in the scene
    return PtInRegion(m_hRegion,
                      (int)(pSnake->GetLeadPoint()->x * 100.0f),
                      (int)(pSnake->GetLeadPoint()->y * 100.0f));
}

void CRegionBehavior::ReadBoundsFromFile(ifstream &infile)
{
    SAFE_DELETE_ARRAY(m_pRegion);
    DeleteObject(m_hRegion);

    infile >> m_iRegionSize;

    FLOAT  fTemp;
    m_pRegion = new POINT[m_iRegionSize];
    for(UINT i = 0; i < m_iRegionSize; ++i)
    {
        //multiply each point by 100 so that integer values for Point
        //don't have as much of an effect
        infile >> fTemp;
        m_pRegion[i].x = (LONG)(fTemp * 100.0f);
        infile >> fTemp;
        m_pRegion[i].y = (LONG)(fTemp * 100.0f);
    }
    m_hRegion = CreatePolygonRgn(m_pRegion, m_iRegionSize, WINDING);
}