// ServerMenu.cpp: implementation of the CServerMenu class.
//
//////////////////////////////////////////////////////////////////////

#include "ServerMenu.h"
#include "customvertex.h"
#include "dxutil.h"
#include <crtdbg.h>     //for _ASSERT

CAbstractMenu::MenuItem CServerMenu::m_pServerMenuItems[] =
{
    {"Start Game",          CAbstractMenu::MENUID_STARTNETGAME},
    {"Cancel",              CAbstractMenu::MENUID_CANCELNET},
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CServerMenu::CServerMenu(CNetworkManager* pnetman) :
    m_pNetManager(pnetman), m_iCurSel(0)
{
    _ASSERT(pnetman);

    m_pMenuFont          = new CD3DFont( _T("Arial"), 20, D3DFONT_BOLD );
    m_pStatusFont        = new CD3DFont( _T("Arial"), 16, D3DFONT_BOLD );
    m_pszIPAddr[0] = '\0';
    m_bHasRendered = FALSE;
}

CServerMenu::~CServerMenu()
{
    DeleteDeviceObjects();
}

int CServerMenu::OnKeyDown(WPARAM wParam)
{
    if(m_pszIPAddr[0] != '\0')
    {
        switch(wParam)
        {
        case VK_RETURN:
            if(CNetworkManager::m_Players.size() > 1)
                return m_pServerMenuItems[m_iCurSel].m_iMenuID;
            else
                return CAbstractMenu::MENUID_CANCELNET;
            break;
        case VK_DOWN:
            ++m_iCurSel;
            m_iCurSel %= sizeof(m_pServerMenuItems) / sizeof(CAbstractMenu::MenuItem);
            break;
        case VK_UP:
            --m_iCurSel;
            if(m_iCurSel < 0)
                m_iCurSel = sizeof(m_pServerMenuItems) / sizeof(CAbstractMenu::MenuItem) - 1;
            break;
        }
    }
    return CAbstractMenu::MENUID_NOTHING;
}

int CServerMenu::OnChar(WPARAM wParam)
{
    return CAbstractMenu::MENUID_NOTHING;
}

int CServerMenu::FrameMove()
{
    //if ip address is not initialized yet initialize it and
    //start the server if necessary
    if(m_bHasRendered && m_pszIPAddr[0] == '\0')
    {
        if(!m_pNetManager->GetIsConnected())
        {
            if(FAILED( m_pNetManager->StartServer() ))
                return CAbstractMenu::MENUID_CANCELNET;
        }
        DXUtil_ConvertWideStringToGeneric(m_pszIPAddr, m_pNetManager->GetLocalIPAddress());
    }
    return CAbstractMenu::MENUID_NOTHING;
}

//render the title menu
void CServerMenu::RenderMenu(int iWidth, int iHeight)
{
    _ASSERT(m_pd3dDevice);
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 0x00000000, 1.0f, 0L );

    RenderText(iWidth, iHeight);
    m_bHasRendered = TRUE;
}

HRESULT CServerMenu::RenderText(int iWidth, int iHeight)
{
    _ASSERT(m_pMenuFont && m_pStatusFont);
    
    D3DCOLOR fontColor        = D3DCOLOR_ARGB(255,128,128,128);
    D3DCOLOR fontSelectColor  = D3DCOLOR_ARGB(255,128,0,255);
    TCHAR    szMsg[MAX_PATH]  = TEXT(""); 
    SIZE     txtsize;
    FLOAT    fNextLine = 5.0f; 

    //if server is not initialized yet
    if(m_pszIPAddr[0] == '\0')
    {
        m_pMenuFont->GetTextExtent( "Initializing Host...", &txtsize );
        m_pMenuFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                                (iHeight - txtsize.cy) / 2.0f, fontSelectColor,
                                "Initializing Host..." );
        return S_OK;
    }

    lstrcpy(szMsg, "Waiting for players on ");
    lstrcat(szMsg, m_pszIPAddr);
    lstrcat(szMsg, "...");

    m_pStatusFont->GetTextExtent( szMsg, &txtsize );
    if(m_pszIPAddr[0] != '\0')
    {
        m_pStatusFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                            fNextLine,
                            fontColor, szMsg );
    }
    fNextLine += 2 * txtsize.cy;

    //print out the list of currently attached players with local player first
    if(CNetworkManager::m_LocalPlayerID)
    {
        m_pStatusFont->GetTextExtent( CNetworkManager::m_Players[CNetworkManager::m_LocalPlayerID].m_pszName, &txtsize );
        m_pStatusFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                                fNextLine, CNetworkManager::m_Players[CNetworkManager::m_LocalPlayerID].m_SnakeColor,
                                CNetworkManager::m_Players[CNetworkManager::m_LocalPlayerID].m_pszName );
        fNextLine += txtsize.cy;
    }
    map<DPNID, CNetworkManager::PlayerInfo>::iterator it;
    for(it = CNetworkManager::m_Players.begin(); it != CNetworkManager::m_Players.end(); ++it)
    {
        if(it->first != CNetworkManager::m_LocalPlayerID)
        {
            m_pStatusFont->GetTextExtent( it->second.m_pszName, &txtsize );
            m_pStatusFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                            fNextLine, it->second.m_SnakeColor, it->second.m_pszName );
            fNextLine += txtsize.cy;
        }
    }

    //place brackets around the selected item
    char pszTemp[256];
    pszTemp[0] = '[';
    strcpy(pszTemp + 1, m_pServerMenuItems[m_iCurSel].m_pszMenuStr);
    strcat(pszTemp, "]");

    if(CNetworkManager::m_Players.size() > 1)
    {
        fNextLine = 0;
        int iNumItems = sizeof(m_pServerMenuItems) / sizeof(CAbstractMenu::MenuItem);
        for(int i = 0; i < iNumItems; ++i)
        {
            if(i == m_iCurSel)
            {
                m_pMenuFont->GetTextExtent( pszTemp, &txtsize );
                m_pMenuFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                                iHeight - (iNumItems + 1) * txtsize.cy + fNextLine,
                                fontSelectColor, pszTemp );
            }
            else
            {
                m_pMenuFont->GetTextExtent( m_pServerMenuItems[i].m_pszMenuStr, &txtsize );
                m_pMenuFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                                    iHeight - (iNumItems + 1) * txtsize.cy + fNextLine,
                                    fontColor, m_pServerMenuItems[i].m_pszMenuStr );
            }
            fNextLine += txtsize.cy;
        }
    }
    else    //only allow for cancel if no one has joined the session yet
    {
        m_pMenuFont->GetTextExtent( "[Cancel]", &txtsize );
        m_pMenuFont->DrawText( (iWidth - txtsize.cx) / 2.0f,
                        iHeight - 2.0f * txtsize.cy,
                        fontSelectColor, "[Cancel]" );
    }
    return S_OK;
}


HRESULT CServerMenu::InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice )
{
    _ASSERT( m_pMenuFont && m_pStatusFont && pd3dDevice);

    // Keep a local copy of the device
    m_pd3dDevice = pd3dDevice;

    // Init the font
    m_pMenuFont->InitDeviceObjects( m_pd3dDevice );
    m_pStatusFont->InitDeviceObjects( m_pd3dDevice );

    return S_OK;
}

HRESULT CServerMenu::RestoreDeviceObjects()
{
    _ASSERT(m_pMenuFont && m_pStatusFont && m_pd3dDevice);

    m_pMenuFont->RestoreDeviceObjects();
    m_pStatusFont->RestoreDeviceObjects();

    return S_OK;
}

HRESULT CServerMenu::InvalidateDeviceObjects()
{
    _ASSERT(m_pMenuFont && m_pStatusFont);
    m_pMenuFont->InvalidateDeviceObjects();
    m_pStatusFont->InvalidateDeviceObjects();
    return S_OK;
}

HRESULT CServerMenu::DeleteDeviceObjects()
{
    if(m_pMenuFont)
        m_pMenuFont->DeleteDeviceObjects();
    if(m_pStatusFont)
        m_pStatusFont->DeleteDeviceObjects();

    m_pd3dDevice  = NULL;
    m_pNetManager = NULL;
    SAFE_DELETE( m_pMenuFont );
    SAFE_DELETE( m_pStatusFont );
    return S_OK;
}