// Snake.h: interface for the CSnake class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "d3dfile.h"
#include "Lofter.h"
#include <D3dx8math.h>
#include <vector>
using namespace std;

class CSnake
{
public:
    friend class CNetworkManager;

    CSnake();
    ~CSnake();

    enum
    {
        ACTION_NONE         = 0x0,

        ACTION_MOVE         = 0x1,

        ACTION_TURNRIGHT    = 0x2,
        ACTION_TURNLEFT     = 0x4,

        ACTION_EAT          = 0x8,
        ACTION_JUMP         = 0x10,
        ACTION_DIVE         = 0x20,

        ACTION_DEATH        = 0x40,
        ACTION_STOP         = 0x80
    };

    //does a specified action based on the flag
    void DoAction(DWORD dwAction, CD3DMesh* pTerrain = NULL);

    //reset the snake's position, size, speed, movedir, and path 
    void ResetPosition(const D3DXVECTOR3& pos, const D3DXVECTOR3& movedir);

    //resest the state of the snake
    void ResetState();

    //change the direction of the snake
    void SetDirection(const D3DXVECTOR3& movedir);

    //sets the color of the snake
    void SetColor(D3DCOLOR color);

    //set the speed of the snake
    void SetSpeed(FLOAT speed);

    //increase/decrease the length of the snake
    void ModifyLength(FLOAT incamt);

    //increase/decrease the length of the snake
    FLOAT GetSpeed();

    //get the length squared of the leading segment
    FLOAT GetLeadSegmentLenSq();

    //gets the two points that define the leading segment
    void GetLeadSegment(D3DXVECTOR3* vFrom, D3DXVECTOR3* vTo);

    //gets the leading point of the path
    D3DXVECTOR3* GetLeadPoint();

    //change the direction of the snake
    D3DXVECTOR3 GetDirection();

    //returns whether the segment defined by vFrom and vTo intersects the snake
    //and the from and to points of the section it intersected
    //if this is a self intersection then don't check the last segment
    BOOL        GetCollision(const D3DXVECTOR3& vFrom, const D3DXVECTOR3& vTo,
                             const CSnake* pSnake = NULL,
                             D3DXVECTOR3* pvSectFrom = NULL, D3DXVECTOR3* pvSectTo = NULL);

    //returns the current state of the snake
    DWORD       GetCurrentState();

    //returns whether the snake is dead
    BOOL        GetIsDead();

    void Render();
    HRESULT InitDeviceObjects( LPDIRECT3DDEVICE8 pd3dDevice );
    HRESULT RestoreDeviceObjects();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();

    FLOAT  m_fScaleRadius;     // radius of the snake to scale
    ULONG  m_iScore;

protected:
    //moves the tail of the snake by a given amount
    void MoveSnakeTail(FLOAT moveamt);

    //move the snake by a multiplier amount
    void Move(FLOAT speed);

    //signal that the snake has just eaten something
    void Eat();

    //turns the snake
    void Turn();

    //signal that the snake should jump
    void Jump(FLOAT fTerrainDelta);

    //signal that the snake should dig
    void Dive(FLOAT fTerrainDelta);

    //perform the snake's way too elaborate death sequence
    void DeathSequence();

    //the snake cross section
    static VECTOR2TEX m_pSnakeSect[];

    struct PATH_VERTEX
    {
        PATH_VERTEX() : vPos(0.0f, 0.0f, 0.0f), fScale(1.0f) {}
        PATH_VERTEX(D3DXVECTOR3 pos) : vPos(pos), fScale(1.0f) {}
        PATH_VERTEX(D3DXVECTOR3 pos, FLOAT scale) : vPos(pos), fScale(scale) {}

        D3DXVECTOR3 vPos;
        FLOAT       fScale;
    };

    CD3DMesh*               m_pSnakeHead;
	LPDIRECT3DDEVICE8		m_pd3dDevice;
    LPDIRECT3DTEXTURE8      m_pTexture;         // texture for the snake body
    LPDIRECT3DINDEXBUFFER8  m_pIndexBuf;        // index buffer for rendering the snake section
    LPDIRECT3DVERTEXBUFFER8 m_pVB;              // vertex buffer for rendering the snake section
    
    D3DXVECTOR3             m_vMoveDir;
    FLOAT                   m_fSpeed;
    vector<PATH_VERTEX>     m_PathVect;

    FLOAT                   m_fTargetLength;    // target length for the snake 
    FLOAT                   m_fCurLength;       // current length of the snake

    DWORD                   m_dwActionSeq;      // flag to show that snake is in an action sequence
	int                     m_iActionSeqFrame;	// the current frame in the action sequence 

    BOOL                    m_bIsDead;          // tells whether this is dead or not
    D3DCOLOR                m_Color;            // color of the snake
};