#pragma once

#include "Snake.h"
#include "CourseLoader.h"
#define MAX_SNAKES 4

class CItemManager;

class CSnakeManager
{
public:
    CSnakeManager(void);
    ~CSnakeManager(void);

    //adds a snake to the list of snakes and makes it the primary snake
    CSnake*  AddSnake(BOOL bPrimary = FALSE);

    //gets the primary snake
    CSnake*  GetPrimary();

    //gets the number of living snakes left
    UINT GetNumLivingSnakes();

    //removes a snake from the list
    void RemoveSnake(CSnake* pSnake);

    //sets the snakes positions/directions to the given positions/directions
    void SetSnakePositions(int iCount, D3DXVECTOR3* pSnakeStarts, D3DXVECTOR3* pSnakeDirs);

    //resets the speed of the snakes
    void Reset();

    //moves all of the snakes
    void MoveSnakes(CD3DMesh* pTerrain);

    //updates the snakes by checking for collisions
    //bAffectPrimaryOnly allows the death to only affect the primary one
    // used in networking to avoid non-primary snakes from being killed by lag
    //returns whether an item was hit or a snake hit something
    BOOL CheckCollisions(CCourseLoader* pCourseLoader, CItemManager* pItemManager, BOOL bAffectPrimaryOnly = FALSE);

    //finds whether a line segment intersects with a snake and
    //returns the endpoints of the snake segment that it intersected with
    BOOL FindSnakeIntersect(const D3DXVECTOR3& vFrom, const D3DXVECTOR3& vTo,
                            D3DXVECTOR3* pvSnakeFrom = NULL,  D3DXVECTOR3* pvSnakeTo = NULL);

    //removes all of the snakes from the vector
    void ClearSnakes(BOOL bSavePrimary = FALSE);

    // Initializing and destroying device-dependent objects
    void    Render();
    HRESULT InitDeviceObjects(LPDIRECT3DDEVICE8 pd3dDevice);
    HRESULT RestoreDeviceObjects();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();
private:
    BOOL    FindSnakeCollision(CSnake* pSnake, CCourseLoader* pCourseLoader = NULL);

    LPDIRECT3DDEVICE8 m_pd3dDevice;

    CSnake*           m_pPrimarySnake;
    CSnake*           m_ppSnakeVect[MAX_SNAKES + 1];     // Array that holds the list of snakes
};
